@@ -125,7 +125,7 @@ static void mpc_decoder_bitstream_jump(mpc_decoder *d, const mpc_uint32_t bits)
     }
 }
 
-void mpc_decoder_update_buffer(mpc_decoder *d, mpc_uint32_t RING)
+static void mpc_decoder_update_buffer(mpc_decoder *d, mpc_uint32_t RING)
 {
     if ((RING ^ d->Zaehler) & MEMSIZE2 ) {
         // update buffer
@@ -1227,7 +1227,7 @@ mpc_bool_t mpc_decoder_seek_sample(mpc_decoder *d, mpc_int64_t destsample)
     if (d->seeking_table_frames > d->DecodedFrames || fwd < d->DecodedFrames) {
         d->DecodedFrames = 0;
         if (fwd > d->seeking_window)
-            d->DecodedFrames = (fwd - d->seeking_window) & (-1 << d->seeking_pwr);
+            d->DecodedFrames = (fwd - d->seeking_window) & ((~0u) << d->seeking_pwr);
         if (d->DecodedFrames > d->seeking_table_frames)
             d->DecodedFrames = d->seeking_table_frames;
         fpos = d->seeking_table[d->DecodedFrames >> d->seeking_pwr];
