@@ -24,7 +24,9 @@
 ** $Id: vrcvisnd.c,v 1.2 2003/12/05 15:55:01 f1rmb Exp $
 */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include "types.h"
 #include "vrcvisnd.h"
