@@ -89,6 +89,8 @@
 #endif
 #include <xine/attributes.h>
 
+/* convenience */
+#define snprintf_buf(buffer, ...) snprintf(buffer, sizeof(buffer), __VA_ARGS__)
 
 #if defined(WIN32) || defined(__CYGWIN__)
 #  define XINE_PATH_SEPARATOR_STRING ";"
@@ -164,6 +166,12 @@ char *xine_private_strcasestr(const char *haystack, const char *needle);
 char *xine_private_strtok_r(char *s, const char *delim, char **ptrptr);
 #endif
 
+#ifndef HAVE_SWAB
+#define HAVE_SWAB
+#define swab(A,B,C) xine_private_swab((A),(B),(C))
+void xine_private_swab(const void *from, void *to, ssize_t n);
+#endif
+
 /* replacement of gettimeofday */
 #ifndef HAVE_GETTIMEOFDAY
 #  define HAVE_GETTIMEOFDAY
