@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2018 the xine project
+ * Copyright (C) 2000-2020 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -64,7 +64,6 @@ typedef struct {
   audio_decoder_t  audio_decoder;
 
   xine_stream_t    *stream;
-  audio_decoder_class_t *class;
 
   dts_state_t     *dts_state;
   int64_t          pts;
@@ -326,7 +325,7 @@ static void dts_decode_frame (dts_decoder_t *this, const int64_t pts) {
 
 static void dts_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 
-  dts_decoder_t  *const this = (dts_decoder_t *) this_gen;
+  dts_decoder_t  *const this = xine_container_of(this_gen, dts_decoder_t, audio_decoder);
   uint8_t        *current = (uint8_t *)buf->content;
   uint8_t        *sync_start=current + 1;
   uint8_t        *const end = buf->content + buf->size;
@@ -457,17 +456,21 @@ static void dts_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 }
 
 static void dts_dispose (audio_decoder_t *this_gen) {
-  dts_decoder_t *const this = (dts_decoder_t *) this_gen;
+  dts_decoder_t *const this = xine_container_of(this_gen, dts_decoder_t, audio_decoder);
 
   if (this->output_open)
     this->stream->audio_out->close (this->stream->audio_out, this->stream);
 
+  dca_free(this->dts_state);
+  this->dts_state = NULL;
   free (this);
 }
 
 static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stream_t *stream) {
   dts_decoder_t *this ;
 
+  (void)class_gen;
+
   lprintf("open_plugin\n");
 
   this = calloc(1, sizeof (dts_decoder_t));
@@ -553,7 +556,6 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
     }
   }
   this->stream        = stream;
-  this->class         = class_gen;
   this->output_open   = 0;
 
   return &this->audio_decoder;
