@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2012 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -96,7 +96,6 @@ typedef struct coreaudio_driver_s {
 typedef struct {
   audio_driver_class_t  driver_class;
 
-  config_values_t      *config;
   xine_t               *xine;
 } coreaudio_class_t;
 
@@ -526,7 +525,6 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen,
                                  const void *data) {
 
   coreaudio_class_t     *class = (coreaudio_class_t *) class_gen;
-  /* config_values_t *config = class->config; */
   coreaudio_driver_t    *this;
 
   lprintf ("open_plugin called\n");
@@ -561,7 +559,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen,
 /*
  * class functions
  */
-static void *init_class (xine_t *xine, void *data) {
+static void *init_class (xine_t *xine, const void *data) {
 
   coreaudio_class_t        *this;
 
@@ -576,14 +574,13 @@ static void *init_class (xine_t *xine, void *data) {
   this->driver_class.description     = N_("xine output plugin for Coreaudio/Mac OS X");
   this->driver_class.dispose         = default_audio_driver_class_dispose;
 
-  this->config = xine->config;
   this->xine   = xine;
 
   return this;
 }
 
 static const ao_info_t ao_info_coreaudio = {
-  1
+  .priority = 1,
 };
 
 /*
@@ -593,6 +590,6 @@ static const ao_info_t ao_info_coreaudio = {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_AUDIO_OUT, AO_OUT_COREAUDIO_IFACE_VERSION, "coreaudio", XINE_VERSION_CODE, &ao_info_coreaudio, init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 
