@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2013-2017 the xine project
- * Copyright (C) 2013 Petri Hintukainen <phintuka@users.sourceforge.net>
+ * Copyright (C) 2013-2020 the xine project
+ * Copyright (C) 2013-2020 Petri Hintukainen <phintuka@users.sourceforge.net>
  *
  * This file is part of xine, a free video player.
  *
@@ -44,9 +44,6 @@
 
 #include "ffmpeg_decoder.h"
 
-#include "ff_video_list.h"
-#include "ff_audio_list.h"
-
 #include "ffmpeg_compat.h"
 
 #ifndef LIBAVFORMAT_VERSION_INT
@@ -75,30 +72,43 @@ typedef struct {
 } avformat_input_plugin_t;
 
 static off_t input_avformat_read (input_plugin_t *this_gen, void *buf_gen, off_t len) {
+  (void)this_gen;
+  (void)buf_gen;
+  (void)len;
   return 0;
 }
 
 static buf_element_t *input_avformat_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo, off_t todo) {
+  (void)this_gen;
+  (void)fifo;
+  (void)todo;
   return NULL;
 }
 
 static off_t input_avformat_get_length (input_plugin_t *this_gen) {
+  (void)this_gen;
   return -1;
 }
 
 static uint32_t input_avformat_get_capabilities (input_plugin_t *this_gen) {
+  (void)this_gen;
   return INPUT_CAP_SEEKABLE;
 }
 
 static uint32_t input_avformat_get_blocksize (input_plugin_t *this_gen) {
+  (void)this_gen;
   return 0;
 }
 
 static off_t input_avformat_get_current_pos (input_plugin_t *this_gen) {
+  (void)this_gen;
   return 0;
 }
 
 static off_t input_avformat_seek (input_plugin_t *this_gen, off_t offset, int origin) {
+  (void)this_gen;
+  (void)offset;
+  (void)origin;
   return -1;
 }
 
@@ -132,6 +142,7 @@ static int input_avformat_get_optional_data (input_plugin_t *this_gen,
 }
 
 static int input_avformat_open (input_plugin_t *this_gen) {
+  (void)this_gen;
   return 1;
 }
 
@@ -160,6 +171,8 @@ static input_plugin_t *input_avformat_get_instance (input_class_t *cls_gen, xine
     return NULL;
   }
 
+  init_once_routine();
+
   /* always accept own protocol */
   /* avformat+http://... --> use avformat instead of xine native input/demux plugins */
   if (!strncasecmp (mrl, DEMUX_AVFORMAT_ID"+", proto_len)) {
@@ -231,32 +244,21 @@ static input_plugin_t *input_avformat_get_instance (input_class_t *cls_gen, xine
   return &this->input_plugin;
 }
 
-void *init_avformat_input_plugin (xine_t *xine, void *data) {
+void *init_avformat_input_plugin (xine_t *xine, const void *data) {
 
-  input_class_t  *this;
+  static const input_class_t input_class = {
+    .get_instance      = input_avformat_get_instance,
+    .description       = N_("libavformat input plugin"),
+    .identifier        = DEMUX_AVFORMAT_ID,
+    .dispose           = NULL,
+  };
 
-  this = calloc(1, sizeof(input_class_t));
-  if (!this) {
-    return NULL;
-  }
+  (void)xine;
+  (void)data;
 
-  pthread_once( &once_control, init_once_routine );
-
-  this->get_instance      = input_avformat_get_instance;
-  this->description       = N_("libavformat input plugin");
-  this->identifier        = DEMUX_AVFORMAT_ID;
-  this->get_dir           = NULL;
-  this->get_autoplay_list = NULL;
-  this->dispose           = default_input_class_dispose;
-  this->eject_media       = NULL;
-
-  return this;
+  return (void *)&input_class;
 }
 
-const input_info_t input_info_avformat = {
-  -2   /* priority */
-};
-
 /*
  * avformat demux plugin
  */
@@ -305,7 +307,7 @@ static void check_newpts(avformat_demux_plugin_t *this, int64_t pts) {
 static uint32_t video_codec_lookup(avformat_demux_plugin_t *this, unsigned id) {
 
   size_t i;
-  for (i = 0; i < sizeof(ff_video_lookup)/sizeof(ff_codec_t); i++) {
+  for (i = 0; i < ff_video_lookup_entries; i++) {
     if (ff_video_lookup[i].id == id) {
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
                LOG_MODULE": found video codec '%s'\n", ff_video_lookup[i].name);
@@ -319,7 +321,7 @@ static uint32_t video_codec_lookup(avformat_demux_plugin_t *this, unsigned id) {
 static uint32_t audio_codec_lookup(avformat_demux_plugin_t *this, unsigned id) {
 
   size_t i;
-  for (i = 0; i < sizeof(ff_audio_lookup)/sizeof(ff_codec_t); i++) {
+  for (i = 0; i < ff_audio_lookup_entries; i++) {
     if (ff_audio_lookup[i].id == id) {
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
                LOG_MODULE": found audio codec '%s'\n", ff_audio_lookup[i].name);
@@ -352,7 +354,7 @@ static AVProgram *_find_program_from_stream(AVFormatContext *ic, AVProgram *last
     } else {
       if (!last)
         for (j = 0; j < ic->programs[i]->nb_stream_indexes; j++)
-          if (ic->programs[i]->stream_index[j] == s)
+          if (ic->programs[i]->stream_index[j] == (unsigned int)s)
             return ic->programs[i];
     }
   }
@@ -362,7 +364,8 @@ static AVProgram *_find_program_from_stream(AVFormatContext *ic, AVProgram *last
 static int find_avformat_streams(avformat_demux_plugin_t *this) {
 
   AVProgram *p = NULL;
-  int i, nb_streams;
+  unsigned int nb_streams;
+  unsigned int u;
 
   /* find avformat streams */
 
@@ -404,23 +407,23 @@ static int find_avformat_streams(avformat_demux_plugin_t *this) {
   }
   nb_streams = p ? p->nb_stream_indexes : this->fmt_ctx->nb_streams;
 
-  for (i = 0; i < nb_streams; i++) {
-    int stream_index = p ? p->stream_index[i] : i;
+  for (u = 0; u < nb_streams; u++) {
+    unsigned int stream_index = p ? p->stream_index[u] : u;
     AVStream *st = this->fmt_ctx->streams[stream_index];
 
     if (stream_index >= this->num_streams) {
-      xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": Too many streams, ignoring stream #%d\n", i);
+      xprintf (this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": Too many streams, ignoring stream #%u\n", u);
       continue;
     }
 
 #ifdef XFF_CODECPAR
     if (st->codecpar && st->codecpar->codec_type == AVMEDIA_TYPE_AUDIO &&
-        st->codecpar->sample_rate != 0 && st->codecpar->channels != 0) {
+        st->codecpar->sample_rate != 0 && st->codecpar->channels != 0)
 #else
     if (st->codec && st->codec->codec_type == AVMEDIA_TYPE_AUDIO &&
-        st->codec->sample_rate != 0 && st->codec->channels != 0) {
+        st->codec->sample_rate != 0 && st->codec->channels != 0)
 #endif
-
+    {
       uint32_t xine_audio_type = audio_codec_lookup(this, CODEC_ID(st));
       if (!xine_audio_type) {
         xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
@@ -610,6 +613,7 @@ static int demux_avformat_get_stream_length (demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_avformat_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_AUDIOLANG;
 }
 
@@ -750,6 +754,14 @@ static int pb_input_read_packet(void *opaque, uint8_t *buf, int buf_size) {
 
 static int64_t pb_input_seek(void *opaque, int64_t offset, int whence) {
   input_plugin_t *input = (input_plugin_t *)opaque;
+
+  if (whence == AVSEEK_SIZE) {
+    off_t len = input->get_length(input);
+    if (len > 0)
+      return len;
+    return -1;
+  }
+
   return input->seek(input, offset, whence);
 }
 
@@ -771,6 +783,11 @@ static AVIOContext *get_io_context(xine_stream_t *stream, input_plugin_t *input)
     /* create AVIO wrapper for native input plugin */
     xprintf (stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": creating AVIOContext wrapper for input plugin\n");
     pb = avio_alloc_context(av_malloc(4096), 4096, 0/*write_flag*/, input, pb_input_read_packet, NULL, pb_input_seek);
+
+    if (input->get_capabilities(input) & INPUT_CAP_SEEKABLE)
+      pb->seekable = AVIO_SEEKABLE_NORMAL;
+    else
+      pb->seekable = 0;
   }
 
   avio_seek(pb, 0, SEEK_SET);
@@ -820,6 +837,8 @@ static demux_plugin_t *open_demux_avformat_plugin (demux_class_t *class_gen,
                                                    xine_stream_t *stream,
                                                    input_plugin_t *input) {
 
+  init_once_routine();
+
   /* get AVFormatContext */
   AVFormatContext *fmt_ctx = get_format_context(stream, input);
   if (!fmt_ctx) {
@@ -867,21 +886,16 @@ static demux_plugin_t *open_demux_avformat_plugin (demux_class_t *class_gen,
   return &this->demux_plugin;
 }
 
-void *init_avformat_demux_plugin (xine_t *xine, void *data) {
-  demux_class_t     *this;
-
-  this  = calloc(1, sizeof(demux_class_t));
-
-  this->open_plugin     = open_demux_avformat_plugin;
-  this->description     = N_("libavformat demux plugin");
-  this->identifier      = DEMUX_AVFORMAT_ID;
-  this->mimetypes       = NULL;
-  this->extensions      = "";
-  this->dispose         = default_demux_class_dispose;
-
-  return this;
+void *init_avformat_demux_plugin (xine_t *xine, const void *data) {
+  static const demux_class_t this = {
+    .open_plugin     = open_demux_avformat_plugin,
+    .description     = N_("libavformat demux plugin"),
+    .identifier      = DEMUX_AVFORMAT_ID,
+    .mimetypes       = NULL,
+    .extensions      = "",
+    .dispose         = NULL
+  };
+  (void)xine;
+  (void)data;
+  return (demux_class_t *)&this;
 }
-
-const demuxer_info_t demux_info_avformat = {
-  -1                       /* priority */
-};
