@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2019 the xine project
+ * Copyright (C) 2001-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -32,6 +32,13 @@
 #include <math.h>
 #include <assert.h>
 
+#ifdef HAVE_FFMPEG_AVUTIL_H
+#  include <avcodec.h>
+#else
+#  include <libavcodec/avcodec.h>
+#  include <libavutil/mem.h>
+#endif
+
 #define LOG_MODULE "ffmpeg_video_dec"
 #define LOG_VERBOSE
 /*
@@ -102,14 +109,12 @@ struct ff_video_decoder_s {
   ff_video_class_t *class;
 
   xine_stream_t    *stream;
+
   int64_t           pts;
   int64_t           last_pts;
-  uint64_t          pts_tag_mask;
-  uint64_t          pts_tag;
-  int               pts_tag_counter;
-  int               pts_tag_stable_counter;
   int               video_step;
   int               reported_video_step;
+  uint8_t           pts_tag_pass;
 
   uint8_t           decoder_ok:1;
   uint8_t           decoder_init_mode:1;
@@ -183,7 +188,7 @@ struct ff_video_decoder_s {
   int                   vaapi_width, vaapi_height;
   int                   vaapi_profile;
   struct vaapi_context  vaapi_context;
-  vaapi_accel_t         *accel;
+  const struct vaapi_accel_funcs_s *accel;
   vo_frame_t            *accel_img;
 #endif
 
@@ -396,7 +401,7 @@ static void release_frame (void *saved_frame, uint8_t *data) {
       return;
 #  ifdef ENABLE_VAAPI
     if (ffsf->va_surface)
-      ffsf->this->accel->f->release_vaapi_surface (ffsf->this->accel_img, ffsf->va_surface);
+      ffsf->this->accel->release_vaapi_surface (ffsf->this->accel_img, ffsf->va_surface);
 #  endif
     if (ffsf->vo_frame)
       ffsf->vo_frame->free (ffsf->vo_frame);
@@ -409,10 +414,10 @@ static void release_buffer(struct AVCodecContext *context, AVFrame *av_frame){
 
 #  ifdef ENABLE_VAAPI
   if( this->context->pix_fmt == PIX_FMT_VAAPI_VLD ) {
-    if(this->accel->f->guarded_render(this->accel_img)) {
+    if(this->accel->guarded_render(this->accel_img)) {
       ff_vaapi_surface_t *va_surface = (ff_vaapi_surface_t *)av_frame->data[0];
       if(va_surface != NULL) {
-        this->accel->f->release_vaapi_surface(this->accel_img, va_surface);
+        this->accel->release_vaapi_surface(this->accel_img, va_surface);
         lprintf("release_buffer: va_surface_id 0x%08x\n", (unsigned int)av_frame->data[3]);
       }
     }
@@ -470,10 +475,10 @@ static int get_buffer_vaapi_vld (AVCodecContext *context, AVFrame *av_frame)
 
     this->vaapi_width  = width;
     this->vaapi_height = height;
-    status = this->accel->f->vaapi_init (this->accel_img, this->vaapi_profile, width, height);
+    status = this->accel->vaapi_init (this->accel_img, this->vaapi_profile, width, height);
 
     if (status == VA_STATUS_SUCCESS) {
-      ff_vaapi_context_t *va_context = this->accel->f->get_context (this->accel_img);
+      ff_vaapi_context_t *va_context = this->accel->get_context (this->accel_img);
 
       if (va_context) {
         this->vaapi_context.config_id  = va_context->va_config_id;
@@ -483,7 +488,7 @@ static int get_buffer_vaapi_vld (AVCodecContext *context, AVFrame *av_frame)
     }
   }
 
-  if(!this->accel->f->guarded_render(this->accel_img)) {
+  if(!this->accel->guarded_render(this->accel_img)) {
     vo_frame_t *img;
     img = this->stream->video_out->get_frame (this->stream->video_out,
                                               width,
@@ -501,7 +506,7 @@ static int get_buffer_vaapi_vld (AVCodecContext *context, AVFrame *av_frame)
     }
     ffsf->vo_frame = img;
   } else {
-    ff_vaapi_surface_t *va_surface = this->accel->f->get_vaapi_surface(this->accel_img);
+    ff_vaapi_surface_t *va_surface = this->accel->get_vaapi_surface(this->accel_img);
 
     if(va_surface) {
       av_frame->data[0] = (void *)va_surface;//(void *)(uintptr_t)va_surface->va_surface_id;
@@ -604,7 +609,7 @@ static int get_buffer (AVCodecContext *context, AVFrame *av_frame)
   }
 
   if(this->accel)
-    guarded_render = this->accel->f->guarded_render(this->accel_img);
+    guarded_render = this->accel->guarded_render(this->accel_img);
 # endif /* ENABLE_VAAPI */
 
   /* The alignment rhapsody */
@@ -751,8 +756,6 @@ static int get_buffer (AVCodecContext *context, AVFrame *av_frame)
 #endif /* ENABLE_DR1 */
 
 
-#include "ff_video_list.h"
-
 static const char *const skip_loop_filter_enum_names[] = {
   "default", /* AVDISCARD_DEFAULT */
   "none",    /* AVDISCARD_NONE */
@@ -829,7 +832,7 @@ static enum PixelFormat get_format(struct AVCodecContext *context, const enum Pi
     return avcodec_default_get_format(context, fmt);
   }
 
-  vaapi_accel_t *accel = (vaapi_accel_t*)this->accel_img->accel_data;
+  const struct vaapi_accel_funcs_s *accel = this->accel;
 
   for (i = 0; fmt[i] != PIX_FMT_NONE; i++) {
     if (fmt[i] != PIX_FMT_VAAPI_VLD)
@@ -840,7 +843,7 @@ static enum PixelFormat get_format(struct AVCodecContext *context, const enum Pi
       continue;
     }
 
-    this->vaapi_profile = accel->f->profile_from_imgfmt (this->accel_img, format);
+    this->vaapi_profile = accel->profile_from_imgfmt (this->accel_img, format);
 
     if (this->vaapi_profile >= 0) {
       int width  = context->width;
@@ -853,10 +856,10 @@ static enum PixelFormat get_format(struct AVCodecContext *context, const enum Pi
       }
       this->vaapi_width  = width;
       this->vaapi_height = height;
-      status = accel->f->vaapi_init (this->accel_img, this->vaapi_profile, width, height);
+      status = accel->vaapi_init (this->accel_img, this->vaapi_profile, width, height);
 
       if( status == VA_STATUS_SUCCESS ) {
-        ff_vaapi_context_t *va_context = accel->f->get_context(this->accel_img);
+        ff_vaapi_context_t *va_context = accel->get_context(this->accel_img);
 
         if(!va_context)
           break;
@@ -937,7 +940,7 @@ static void init_video_codec (ff_video_decoder_t *this, unsigned int codec_type)
 #ifdef ENABLE_VAAPI
   if( this->class->enable_vaapi ) {
     uint32_t format = vaapi_pixfmt2imgfmt(PIX_FMT_VAAPI_VLD, this->codec->id, -1);
-    if (format && this->accel->f->profile_from_imgfmt (this->accel_img, format) >= 0) {
+    if (format && this->accel->profile_from_imgfmt (this->accel_img, format) >= 0) {
       use_vaapi = 1;
     } else {
       xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
@@ -1289,9 +1292,9 @@ static void ff_convert_frame(ff_video_decoder_t *this, vo_frame_t *img, AVFrame
 
 #ifdef ENABLE_VAAPI
   if (this->context->pix_fmt == PIX_FMT_VAAPI_VLD) {
-    if (this->accel->f->guarded_render(this->accel_img)) {
+    if (this->accel->guarded_render(this->accel_img)) {
       ff_vaapi_surface_t *va_surface = (ff_vaapi_surface_t *)av_frame->data[0];
-      this->accel->f->render_vaapi_surface (img, va_surface);
+      this->accel->render_vaapi_surface (img, va_surface);
     }
     return;
   }
@@ -1793,49 +1796,43 @@ static void ff_handle_special_buffer (ff_video_decoder_t *this, buf_element_t *b
   }
 }
 
-static uint64_t ff_tag_pts(ff_video_decoder_t *this, uint64_t pts)
-{
-  return pts | this->pts_tag;
+static void ff_discontinuity (video_decoder_t *this_gen) {
+  ff_video_decoder_t *this = (ff_video_decoder_t *) this_gen;
+
+  lprintf ("ff_discontinuity\n");
+  this->pts = 0;
+  this->state = STATE_DISCONTINUITY;
+
+  /*
+   * there is currently no way to reset all the pts which are stored in the decoder.
+   * therefore, we add a unique tag (generated from pts_tag_counter) to pts (see
+   * ff_tag_pts()) and wait for it to appear on returned frames.
+   * until then, any retrieved pts value will be reset to 0 (see ff_untag_pts()).
+   * NOTE: there may be small negative pts values, eg -7200, or even -1 (reordered .mp4).
+   * hence the sign restore.
+   * NOTE: previous code had an "inactive" mode that let _any_ value pass - when there
+   * are no discontinuities. not very realistic. furthermore, many xine code parts
+   * assume that pts never use more than 48 bits or so for their 64bit math.
+   * so lets drop complexity, and _always_ tag 8 bits.
+   */
+  this->pts_tag_pass = (this->pts_tag_pass + 1) & 0xff;
 }
 
-static uint64_t ff_untag_pts(ff_video_decoder_t *this, uint64_t pts)
-{
-  if (this->pts_tag_mask == 0)
-    return pts; /* pts tagging inactive */
-
-  if (this->pts_tag != 0 && (pts & this->pts_tag_mask) != this->pts_tag)
-    return 0; /* reset pts if outdated while waiting for first pass (see below) */
-
-  return pts & ~this->pts_tag_mask;
+static int64_t ff_tag_pts (ff_video_decoder_t *this, int64_t pts) {
+  /* NOTE: m68k has sign preserving left shifts, x86 has not.
+   * both are correct with non overflowing 2's complement value.
+   * anyway, some compilers warn about undefined op there, so use * 256.
+   * NOTE: we still like notifications ;-) */
+  return (pts * 256) | this->pts_tag_pass;
 }
 
-static void ff_check_pts_tagging(ff_video_decoder_t *this, uint64_t pts)
-{
-  if (this->pts_tag_mask == 0)
-    return; /* pts tagging inactive */
-  if ((pts & this->pts_tag_mask) != this->pts_tag) {
-    this->pts_tag_stable_counter = 0;
-    return; /* pts still outdated */
-  }
-
-  /* the tag should be stable for 100 frames */
-  this->pts_tag_stable_counter++;
-
-  if (this->pts_tag != 0) {
-    if (this->pts_tag_stable_counter >= 100) {
-      /* first pass: reset pts_tag */
-      this->pts_tag = 0;
-      this->pts_tag_stable_counter = 0;
-    }
-  } else if (pts == 0)
-    return; /* cannot detect second pass */
-  else {
-    if (this->pts_tag_stable_counter >= 100) {
-      /* second pass: reset pts_tag_mask and pts_tag_counter */
-      this->pts_tag_mask = 0;
-      this->pts_tag_counter = 0;
-      this->pts_tag_stable_counter = 0;
-    }
+static int64_t ff_untag_pts (ff_video_decoder_t *this, int64_t pts) {
+  if ((uint8_t)(pts & 0xff) == this->pts_tag_pass) {
+    /* restore sign. */
+    return pts >> 8;
+  } else {
+    /* reset pts if outdated while waiting for first pass (see below). */
+    return 0;
   }
 }
 
@@ -1846,7 +1843,7 @@ static int decode_video_wrapper (ff_video_decoder_t *this,
 #if ENABLE_VAAPI
   int locked = 0;
   if (this->accel) {
-    locked = this->accel->f->lock_vaapi(this->accel_img);
+    locked = this->accel->lock_vaapi(this->accel_img);
   }
 #endif /* ENABLE_VAAPI */
 
@@ -1925,7 +1922,7 @@ static int decode_video_wrapper (ff_video_decoder_t *this,
 
 #if ENABLE_VAAPI
   if (locked) {
-    this->accel->f->unlock_vaapi(this->accel_img);
+    this->accel->unlock_vaapi(this->accel_img);
   }
 #endif /* ENABLE_VAAPI */
 
@@ -2063,7 +2060,6 @@ static void ff_handle_mpeg12_buffer (ff_video_decoder_t *this, buf_element_t *bu
 
       /* get back reordered pts */
       img->pts = ff_untag_pts (this, this->av_frame->reordered_opaque);
-      ff_check_pts_tagging (this, this->av_frame->reordered_opaque);
       this->av_frame->reordered_opaque = 0;
       this->context->reordered_opaque = 0;
 
@@ -2136,6 +2132,25 @@ static void ff_postprocess (ff_video_decoder_t *this, AVFrame *av_frame, vo_fram
 }
 #endif /* HAVE_POSTPROC */
 
+static int ff_video_step_get (ff_video_decoder_t *this) {
+  /* use externally provided video_step or fall back to stream's time_base otherwise */
+  int step = this->video_step;
+  if (step || !this->context->time_base.den)
+    return step;
+
+  /* good: 2 * 1001 / 48000. */
+  step = (int64_t)90000 * this->context->ticks_per_frame
+       * this->context->time_base.num / this->context->time_base.den;
+  if (step >= 90)
+    return step;
+
+  /* bad: 2 * 1 / 60000. seen this once from broken h.264 video usability info (VUI).
+   * VAAPI seems to apply a similar HACK.*/
+  step = (int64_t)90000000 * this->context->ticks_per_frame
+       * this->context->time_base.num / this->context->time_base.den;
+  return step;
+}
+
 static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
   uint8_t *chunk_buf = this->buf;
   AVRational avr00 = {0, 1};
@@ -2273,12 +2288,7 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
         }
       }
 
-      /* use externally provided video_step or fall back to stream's time_base otherwise */
-      video_step_to_use = (this->video_step || !this->context->time_base.den)
-                        ? this->video_step
-                        : (int)(90000ll
-                                * this->context->ticks_per_frame
-                                * this->context->time_base.num / this->context->time_base.den);
+      video_step_to_use = ff_video_step_get (this);
 
       /* aspect ratio provided by ffmpeg, override previous setting */
       if ((this->aspect_ratio_prio < 2) &&
@@ -2390,7 +2400,6 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
         }
 
         img->pts  = ff_untag_pts(this, this->av_frame->reordered_opaque);
-        ff_check_pts_tagging(this, this->av_frame->reordered_opaque); /* only check for valid frames */
         this->av_frame->reordered_opaque = 0;
 
         /* workaround for weird 120fps streams */
@@ -2547,10 +2556,7 @@ static void ff_flush_internal (ff_video_decoder_t *this, int display) {
     }
 
     /* All that jizz just to view the last 2 frames of a stream ;-) */
-    video_step_to_use = this->video_step || !this->context->time_base.den ?
-      this->video_step :
-      90000ll * this->context->ticks_per_frame * this->context->time_base.num /
-        this->context->time_base.den;
+    video_step_to_use = ff_video_step_get (this);
 
     if ((this->aspect_ratio_prio < 2) && av_cmp_q (this->context->sample_aspect_ratio, avr00)) {
       if (!this->bih.biWidth || !this->bih.biHeight) {
@@ -2618,7 +2624,6 @@ static void ff_flush_internal (ff_video_decoder_t *this, int display) {
     }
 
     img->pts = ff_untag_pts (this, this->av_frame2->reordered_opaque);
-    ff_check_pts_tagging (this, this->av_frame2->reordered_opaque);
 
     if (video_step_to_use == 750)
       video_step_to_use = 0;
@@ -2723,51 +2728,7 @@ static void ff_reset (video_decoder_t *this_gen) {
   if (this->is_mpeg12)
     mpeg_parser_reset(this->mpeg_parser);
 
-  this->pts_tag_mask = 0;
-  this->pts_tag = 0;
-  this->pts_tag_counter = 0;
-  this->pts_tag_stable_counter = 0;
-}
-
-static void ff_discontinuity (video_decoder_t *this_gen) {
-  ff_video_decoder_t *this = (ff_video_decoder_t *) this_gen;
-
-  lprintf ("ff_discontinuity\n");
-  this->pts = 0;
-  this->state = STATE_DISCONTINUITY;
-
-  /*
-   * there is currently no way to reset all the pts which are stored in the decoder.
-   * therefore, we add a unique tag (generated from pts_tag_counter) to pts (see
-   * ff_tag_pts()) and wait for it to appear on returned frames.
-   * until then, any retrieved pts value will be reset to 0 (see ff_untag_pts()).
-   * when we see the tag returned, pts_tag will be reset to 0. from now on, any
-   * untagged pts value is valid already.
-   * when tag 0 appears too, there are no tags left in the decoder so pts_tag_mask
-   * and pts_tag_counter will be reset to 0 too (see ff_check_pts_tagging()).
-   */
-  this->pts_tag_counter++;
-  this->pts_tag_mask = 0;
-  this->pts_tag = 0;
-  this->pts_tag_stable_counter = 0;
-  {
-    /* pts values typically don't use the uppermost bits. therefore we put the tag there */
-    int counter_mask = 1;
-    int counter = 2 * this->pts_tag_counter + 1; /* always set the uppermost bit in tag_mask */
-    uint64_t tag_mask = 0x8000000000000000ull;
-    while (this->pts_tag_counter >= counter_mask)
-    {
-      /*
-       * mirror the counter into the uppermost bits. this allows us to enlarge mask as
-       * necessary and while previous taggings can still be detected to be outdated.
-       */
-      if (counter & counter_mask)
-        this->pts_tag |= tag_mask;
-      this->pts_tag_mask |= tag_mask;
-      tag_mask >>= 1;
-      counter_mask <<= 1;
-    }
-  }
+  /* this->pts_tag_pass = 0; */
 }
 
 static void ff_dispose (video_decoder_t *this_gen) {
@@ -2850,9 +2811,11 @@ static video_decoder_t *ff_video_open_plugin (video_decoder_class_t *class_gen,
 
   lprintf ("open_plugin\n");
 
+  init_once_routine();
+
   /* check for codec support */
   video_type = BUF_VIDEO_BASE | (_x_get_video_streamtype(stream) << 16);
-  for (i = 0; i < sizeof(ff_video_lookup)/sizeof(ff_codec_t); i++) {
+  for (i = 0; i < ff_video_lookup_entries; i++) {
     if(ff_video_lookup[i].type == video_type) {
       pthread_mutex_lock(&ffmpeg_lock);
       codec = avcodec_find_decoder(ff_video_lookup[i].id);
@@ -2877,10 +2840,12 @@ static video_decoder_t *ff_video_open_plugin (video_decoder_class_t *class_gen,
    * Let it optimize away this on most systems where clear mem
    * interpretes as 0, 0f or NULL safely.
    */
+#ifndef HAVE_ZERO_SAFE_MEM
   this->size            = 0;
   this->decoder_ok      = 0;
   this->is_mpeg12       = 0;
   this->aspect_ratio    = 0;
+  this->pts_tag_pass    = 0;
 #ifdef HAVE_POSTPROC
   this->pp_quality      = 0;
   this->our_context     = NULL;
@@ -2895,6 +2860,7 @@ static video_decoder_t *ff_video_open_plugin (video_decoder_class_t *class_gen,
 #endif
 #if XFF_VIDEO == 3
   this->flush_packet_sent = 0;
+#endif
 #endif
 
   this->video_decoder.decode_data   = ff_decode_data;
@@ -2958,7 +2924,7 @@ static video_decoder_t *ff_video_open_plugin (video_decoder_class_t *class_gen,
                                                      VO_BOTH_FIELDS | VO_GET_FRAME_MAY_FAIL );
 
     if( this->accel_img ) {
-      this->accel = (vaapi_accel_t*)this->accel_img->accel_data;
+      this->accel = ((vaapi_accel_t*)this->accel_img->accel_data)->f;
       xprintf(this->class->xine, XINE_VERBOSITY_LOG, _("ffmpeg_video_dec: VAAPI Enabled in config.\n"));
     } else {
       this->class->enable_vaapi = 0;
@@ -3002,8 +2968,6 @@ void *init_video_plugin (xine_t *xine, const void *data) {
   this->decoder_class.dispose         = dispose_video_class;
   this->xine                          = xine;
 
-  pthread_once( &once_control, init_once_routine );
-
   /* Configuration for post processing quality - default to mid (3) for the
    * moment */
   config = xine->config;
@@ -3071,28 +3035,3 @@ void *init_video_plugin (xine_t *xine, const void *data) {
 
   return this;
 }
-
-static const uint32_t wmv8_video_types[] = {
-  BUF_VIDEO_WMV8,
-  0
-};
-
-static const uint32_t wmv9_video_types[] = {
-  BUF_VIDEO_WMV9,
-  0
-};
-
-const decoder_info_t dec_info_ffmpeg_video = {
-  supported_video_types,   /* supported types */
-  6                        /* priority        */
-};
-
-const decoder_info_t dec_info_ffmpeg_wmv8 = {
-  wmv8_video_types,        /* supported types */
-  0                        /* priority        */
-};
-
-const decoder_info_t dec_info_ffmpeg_wmv9 = {
-  wmv9_video_types,        /* supported types */
-  0                        /* priority        */
-};
