@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2018 the xine project
+ * Copyright (C) 2001-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -26,23 +26,38 @@
 
 #include <pthread.h>
 
-#include <xine/xine_internal.h>
+#ifdef HAVE_FFMPEG_AVUTIL_H
+#  include <avcodec.h>
+#else
+#  include <libavcodec/avcodec.h>
+#  include <libavutil/mem.h>
+#endif
+#ifdef HAVE_AVFORMAT
+#  include <libavformat/avformat.h> // av_register_all()
+#endif
 
-#include "ffmpeg_decoder.h"
 #include "ffmpeg_compat.h"
 
-#ifdef HAVE_AVFORMAT
-#include <libavformat/avformat.h> // av_register_all()
-#endif
+#include <xine.h>
+#include <xine/xine_internal.h>  // XINE_VERSION_CODE
+#include <xine/xine_plugin.h>
+#include <xine/buffer.h>
+
+#include "ffmpeg_decoder.h"
+
+#include "ff_audio_list.h"
+#include "ff_video_list.h"
+
+const size_t ff_video_lookup_entries = sizeof(ff_video_lookup) / sizeof(ff_video_lookup[0]);
+const size_t ff_audio_lookup_entries = sizeof(ff_audio_lookup) / sizeof(ff_audio_lookup[0]);
 
 /*
  * common initialisation
  */
 
-pthread_once_t once_control = PTHREAD_ONCE_INIT;
 pthread_mutex_t ffmpeg_lock;
 
-void init_once_routine(void) {
+static void _init_once_routine(void) {
   pthread_mutex_init(&ffmpeg_lock, NULL);
   XFF_AVCODEC_INIT();
   XFF_AVCODEC_REGISTER_ALL();
@@ -55,10 +70,60 @@ void init_once_routine(void) {
 #endif
 }
 
+void init_once_routine(void) {
+  static pthread_once_t once_control = PTHREAD_ONCE_INIT;
+
+  pthread_once( &once_control, _init_once_routine );
+}
+
 /*
  * exported plugin catalog entry
  */
 
+#ifdef HAVE_AVFORMAT
+static const input_info_t input_info_avio = {
+  .priority = -1,
+};
+
+static const input_info_t input_info_avformat = {
+  .priority = -2,
+};
+
+static const demuxer_info_t demux_info_avformat = {
+  .priority = -1,
+};
+#endif /* HAVE_AVFORMAT */
+
+static const decoder_info_t dec_info_ffmpeg_audio = {
+  .supported_types = supported_audio_types,
+  .priority = 7,
+};
+
+static const uint32_t wmv8_video_types[] = {
+  BUF_VIDEO_WMV8,
+  0
+};
+
+static const decoder_info_t dec_info_ffmpeg_wmv8 = {
+  .supported_types = wmv8_video_types,
+  .priority = 0,
+};
+
+static const uint32_t wmv9_video_types[] = {
+  BUF_VIDEO_WMV9,
+  0
+};
+
+static const decoder_info_t dec_info_ffmpeg_wmv9 = {
+  .supported_types = wmv9_video_types,
+  .priority = 0,
+};
+
+static const decoder_info_t dec_info_ffmpeg_video = {
+  .supported_types = supported_video_types,
+  .priority = 6,
+};
+
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_DECODER | PLUGIN_MUST_PRELOAD, 19, "ffmpegvideo", XINE_VERSION_CODE, &dec_info_ffmpeg_video, init_video_plugin },
