@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2018 the xine project
+ * Copyright (C) 2013-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -25,3 +25,5 @@
 void *vorbis_init_plugin (xine_t *xine, const void *data);
 void *speex_init_plugin (xine_t *xine, const void *data);
 void *theora_init_plugin (xine_t *xine, const void *data);
+void *anx_init_class (xine_t *xine, const void *data);
+void *ogg_init_class (xine_t *xine, const void *data);
