@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2017 the xine project
+ * Copyright (C) 2001-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -64,17 +64,18 @@ typedef struct {
   int                  seek_flag;  /* this is set when a seek just occurred */
 } demux_cdda_t;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_cdda_class_t;
 
 static int demux_cdda_send_chunk (demux_plugin_t *this_gen) {
   demux_cdda_t *this = (demux_cdda_t *) this_gen;
   buf_element_t *buf = NULL;
   uint32_t blocksize;
+  off_t len, pos;
+
+  pos = this->input->get_current_pos (this->input);
+  len = this->input->get_length (this->input);
 
   blocksize = this->input->get_blocksize(this->input);
-  if (!blocksize) blocksize = 2352;
+  if (!blocksize) blocksize = 3 * 2352;
   buf = this->input->read_block(this->input, this->audio_fifo,
     blocksize);
   if (!buf) {
@@ -83,10 +84,9 @@ static int demux_cdda_send_chunk (demux_plugin_t *this_gen) {
   }
 
   buf->type = BUF_AUDIO_LPCM_LE;
-  if( this->input->get_length (this->input) )
-    buf->extra_info->input_normpos = (int)( (double) this->input->get_current_pos (this->input) *
-                                     65535 / this->input->get_length (this->input) );
-  buf->pts = this->input->get_current_pos(this->input);
+  if (len > 0)
+    buf->extra_info->input_normpos = (int)((double)pos * 65535 / len);
+  buf->pts = pos;
   buf->pts *= 90000;
   buf->pts /= CD_BYTES_PER_SECOND;
   buf->extra_info->input_time = buf->pts / 90;
@@ -174,11 +174,15 @@ static int demux_cdda_get_stream_length (demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_cdda_get_capabilities(demux_plugin_t *this_gen){
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_cdda_get_optional_data(demux_plugin_t *this_gen,
                                         void *data, int data_type){
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -193,11 +197,20 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
       break;
 
     case METHOD_BY_CONTENT:
+      /* Avoid plugin loader probing us again after all others,
+         especially after that still unstable avformat. */
+      if (input && input->input_class && input->input_class->identifier
+        && !strcmp (input->input_class->identifier, "cdda"))
+        break;
+      /* fall through */
     default:
       return NULL;
   }
 
-  this         = calloc(1, sizeof(demux_cdda_t));
+  this = calloc(1, sizeof(demux_cdda_t));
+  if (!this)
+    return NULL;
+
   this->stream = stream;
   this->input  = input;
 
@@ -216,17 +229,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-void *demux_cdda_init_plugin (xine_t *xine, void *data) {
-  demux_cdda_class_t     *this;
+void *demux_cdda_init_plugin (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof(demux_cdda_class_t));
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("CD Digital Audio demux plugin");
-  this->demux_class.identifier      = "CDDA";
-  this->demux_class.mimetypes       = NULL;
-  this->demux_class.extensions      = "cdda:/";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_cdda_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("CD Digital Audio demux plugin"),
+    .identifier      = "CDDA",
+    .mimetypes       = NULL,
+    .extensions      = "cdda:/",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_cdda_class;
 }
