@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2017 the xine project
+ * Copyright (C) 2005-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -69,9 +69,6 @@ typedef struct {
   off_t                data_start;
 } demux_dts_t;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_dts_class_t;
 
 static const int dts_sample_rates[] =
 {
@@ -94,7 +91,6 @@ static int open_dts_file(demux_dts_t *this) {
     buf_element_t *buf = this->input->read_block(this->input,
 						 this->stream->audio_fifo,
 						 blocksize);
-    this->input->seek(this->input, 0, SEEK_SET);
 
     if (!buf)
       return 0;
@@ -103,10 +99,14 @@ static int open_dts_file(demux_dts_t *this) {
     xine_fast_memcpy(peak, buf->content, peak_size);
 
     buf->free_buffer(buf);
+
+    if (this->input->seek(this->input, 0, SEEK_SET) != 0)
+      return 0;
+
   } else {
     peak = alloca(peak_size = MAX_PREVIEW_SIZE);
 
-    if (_x_demux_read_header(this->input, peak, peak_size) != peak_size)
+    if (_x_demux_read_header (this->input, peak, peak_size) != (int)peak_size)
       return 0;
   }
 
@@ -131,7 +131,7 @@ static int open_dts_file(demux_dts_t *this) {
 
     /* Find the data chunk */
     offset = 20 + _X_LE_32(&peak[16]);
-    while (offset < peak_size-8) {
+    while (offset < (int)peak_size - 8) {
       unsigned int chunk_tag = _X_LE_32(&peak[offset]);
       unsigned int chunk_size = _X_LE_32(&peak[offset+4]);
 
@@ -154,7 +154,7 @@ static int open_dts_file(demux_dts_t *this) {
   int dts_version = -1;
 
   /* Look for a valid DTS syncword */
-  for (i=offset; i<peak_size-1; i++) {
+  for (i = offset; i < (int)peak_size - 1; i++) {
     /* 16 bits and big endian bitstream */
     if (syncword == 0x7ffe8001) {
 	dts_version = 0;
@@ -179,7 +179,7 @@ static int open_dts_file(demux_dts_t *this) {
   this->data_start = i-4;
   lprintf("found DTS syncword at offset %d\n", i-4);
 
-  if (i < peak_size-9) {
+  if (i < (int)peak_size - 9) {
     unsigned int nblks, fsize, sfreq;
     switch (dts_version)
     {
@@ -234,7 +234,8 @@ static int open_dts_file(demux_dts_t *this) {
     lprintf("sample rate: %d\n", this->sample_rate);
 
     /* Seek to start of DTS data */
-    this->input->seek(this->input, this->data_start, SEEK_SET);
+    if (this->input->seek(this->input, this->data_start, SEEK_SET) != this->data_start)
+      return 0;
 
     return 1;
   }
@@ -346,6 +347,8 @@ static int demux_dts_seek (demux_plugin_t *this_gen,
 
   demux_dts_t *this = (demux_dts_t *) this_gen;
 
+  (void)playing;
+
   this->seek_flag = 1;
   this->status = DEMUX_OK;
   _x_demux_flush_engine (this->stream);
@@ -383,11 +386,15 @@ static int demux_dts_get_status (demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_dts_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_dts_get_optional_data(demux_plugin_t *this_gen,
                                        void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -396,7 +403,10 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_dts_t   *this;
 
-  this         = calloc(1, sizeof(demux_dts_t));
+  this = calloc(1, sizeof(demux_dts_t));
+  if (!this)
+    return NULL;
+
   this->stream = stream;
   this->input  = input;
 
@@ -431,18 +441,20 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-void *demux_dts_init_plugin (xine_t *xine, void *data) {
-  demux_dts_class_t     *this;
+void *demux_dts_init_plugin (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof(demux_dts_class_t));
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("Raw DTS demux plugin");
-  this->demux_class.identifier      = "DTS";
-  this->demux_class.mimetypes       = NULL;
-  this->demux_class.extensions      = "dts";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_dts_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("Raw DTS demux plugin"),
+    .identifier      = "DTS",
+    .mimetypes       = NULL,
+    .extensions      = "dts",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_dts_class;
 }
 
