@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -23,10 +23,6 @@
 #ifndef _DEMUX_MATROSKA_H_
 #define _DEMUX_MATROSKA_H_
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <ctype.h>
 #include <stdio.h>
 #include <fcntl.h>
@@ -58,25 +54,10 @@ typedef struct {
 
 } matroska_index_t;
 
-typedef struct {
-
-  demux_class_t     demux_class;
-
-  /* class-wide, global variables here */
-
-  xine_t           *xine;
-
-#define LITERAL_UTF_8_LEN 5
-  char              literal_utf_8[8]; /* "utf-8" */
-
-} demux_matroska_class_t;
-
 typedef struct {
 
   demux_plugin_t       demux_plugin;
 
-  demux_matroska_class_t *class;
-
   xine_stream_t       *stream;
 
   input_plugin_t      *input;
