@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2016 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -62,10 +62,6 @@ typedef struct {
   uint64_t             pts;
 } demux_raw_dv_t ;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_raw_dv_class_t;
-
 
 static int demux_raw_dv_next (demux_raw_dv_t *this) {
   buf_element_t *buf, *abuf;
@@ -155,16 +151,17 @@ static void demux_raw_dv_send_headers (demux_plugin_t *this_gen) {
   scratch = (unsigned char *) malloc(NTSC_FRAME_SIZE);
   if (scratch == NULL )
     return;
-
   if (INPUT_IS_SEEKABLE(this->input)) {
-    this->input->seek(this->input, 0, SEEK_SET);
+    if (this->input->seek(this->input, 0, SEEK_SET) != 0)
+      goto out;
     if( this->input->read (this->input, scratch, NTSC_FRAME_SIZE) != NTSC_FRAME_SIZE )
-      return;
-    this->input->seek(this->input, 0, SEEK_SET);
+      goto out;
+    if (this->input->seek(this->input, 0, SEEK_SET) != 0)
+      goto out;
   }
   else {
     if( this->input->read (this->input, scratch, NTSC_FRAME_SIZE) != NTSC_FRAME_SIZE )
-      return;
+      goto out;
     if( !(scratch[3] & 0x80) )
       i = NTSC_FRAME_SIZE;
     else
@@ -173,7 +170,7 @@ static void demux_raw_dv_send_headers (demux_plugin_t *this_gen) {
     i -= NTSC_FRAME_SIZE;
     while (i > 0) {
       if( this->input->read (this->input, scratch2, 4) != 4 )
-        return;
+        goto out;
       i -= 4;
     }
   }
@@ -290,6 +287,8 @@ static void demux_raw_dv_send_headers (demux_plugin_t *this_gen) {
     _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_AUDIO, 1);
   }
 
+ out:
+  free(scratch);
 }
 
 static int demux_raw_dv_seek (demux_plugin_t *this_gen,
@@ -304,6 +303,8 @@ static int demux_raw_dv_seek (demux_plugin_t *this_gen,
     return this->status;
   }
 
+  (void)playing;
+
   if( !start_pos && start_time ) {
     /* Upcast start_time in case sizeof(off_t) > sizeof(int) */
     start_pos = ((off_t) start_time * 90 / this->duration) * this->frame_size;
@@ -332,11 +333,15 @@ static int demux_raw_dv_get_stream_length(demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_raw_dv_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_raw_dv_get_optional_data(demux_plugin_t *this_gen,
 					void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -344,8 +349,31 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
                                     input_plugin_t *input) {
 
   demux_raw_dv_t *this;
+  uint8_t buf[8];
+
+  switch (stream->content_detection_method) {
+
+    case METHOD_BY_CONTENT:
+      if (_x_demux_read_header(input, buf, 8) != 8)
+        return NULL;
+
+      /* DIF (DV) movie file */
+      if (memcmp(buf, "\x1F\x07\x00", 3) != 0 || !(buf[4] ^ 0x01))
+        return NULL;
+      break;
+
+    case METHOD_BY_MRL:
+    case METHOD_EXPLICIT:
+      break;
+
+    default:
+      return NULL;
+  }
+
+  this = calloc(1, sizeof(demux_raw_dv_t));
+  if (!this)
+    return NULL;
 
-  this         = calloc(1, sizeof(demux_raw_dv_t));
   this->stream = stream;
   this->input  = input;
 
@@ -361,33 +389,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   this->status = DEMUX_FINISHED;
 
-  switch (stream->content_detection_method) {
-
-  case METHOD_BY_CONTENT: {
-    uint8_t buf[8];
-
-    if (_x_demux_read_header(input, buf, 8) != 8) {
-      free (this);
-      return NULL;
-    }
-
-    /* DIF (DV) movie file */
-    if (memcmp(buf, "\x1F\x07\x00", 3) != 0 || !(buf[4] ^ 0x01)) {
-      free (this);
-      return NULL;
-    }
-  }
-  break;
-
-  case METHOD_BY_MRL:
-  case METHOD_EXPLICIT:
-  break;
-
-  default:
-    free (this);
-    return NULL;
-  }
-
   if (!INPUT_IS_SEEKABLE(this->input)) {
     /* "live" DV streams require more prebuffering */
     this->stream->metronom->set_option(this->stream->metronom, METRONOM_PREBUFFER, 90000);
@@ -396,18 +397,20 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-void *demux_rawdv_init_class (xine_t *xine, void *data) {
-  demux_raw_dv_class_t     *this;
+void *demux_rawdv_init_class (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof(demux_raw_dv_class_t));
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("Raw DV Video stream");
-  this->demux_class.identifier      = "raw_dv";
-  this->demux_class.mimetypes       = NULL;
-  this->demux_class.extensions      = "dv dif";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_rawdv_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("Raw DV Video stream"),
+    .identifier      = "raw_dv",
+    .mimetypes       = NULL,
+    .extensions      = "dv dif",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_rawdv_class;
 }
 
