@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2018 the xine project
  * May 2003 - Miguel Freitas
  * This plugin was sponsored by 1Control
  *
@@ -73,10 +73,6 @@ typedef struct {
   int                  scratch_used;
 } demux_slave_t ;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_slave_class_t;
-
 
 #define MAX_COMMAND_SIZE 20
 
@@ -90,16 +86,15 @@ static int demux_slave_next (demux_slave_t *this) {
   /* fill the scratch buffer */
   n = this->input->read(this->input, &this->scratch[this->scratch_used],
                         SCRATCH_SIZE - this->scratch_used);
-  if (n > 0)
-    this->scratch_used += n;
-  this->scratch[this->scratch_used] = '\0';
-
   if (n <= 0) {
     lprintf("connection closed\n");
     this->status = DEMUX_FINISHED;
     return 0;
   }
 
+  this->scratch_used += n;
+  this->scratch[this->scratch_used] = '\0';
+
   p = strchr(this->scratch,'\n');
   s = strchr(this->scratch,' ');
 
@@ -170,8 +165,13 @@ static int demux_slave_next (demux_slave_t *this) {
       n = size;
     if( n )
       memcpy(buf->content, p, n);
-    if( n < size )
-      this->input->read(this->input, &buf->content[n], size-n);
+    if( n < size ) {
+      if (this->input->read(this->input, &buf->content[n], size-n) != (size-n)) {
+        buf->free_buffer(buf);
+        this->status = DEMUX_FINISHED;
+        return 0;
+      }
+    }
 
     p += n;
     n = this->scratch_used - (p-this->scratch);
@@ -227,8 +227,12 @@ static int demux_slave_next (demux_slave_t *this) {
       n = size;
     if( n )
       memcpy(this->decoder_info_ptr[i], p, n);
-    if( n < size )
-      this->input->read(this->input, (char *)this->decoder_info_ptr[i]+n, size-n);
+    if( n < size ) {
+      if (this->input->read(this->input, (char *)this->decoder_info_ptr[i]+n, size-n) != (size-n)) {
+        this->status = DEMUX_FINISHED;
+        return 0;
+      }
+    }
 
     p += n;
     n = this->scratch_used - (p-this->scratch);
@@ -291,6 +295,10 @@ static void demux_slave_send_headers (demux_plugin_t *this_gen) {
 static int demux_slave_seek (demux_plugin_t *this_gen, off_t start_pos, int start_time, int playing) {
   demux_slave_t *this = (demux_slave_t *) this_gen;
 
+  (void)this_gen;
+  (void)start_pos;
+  (void)start_time;
+  (void)playing;
   return this->status;
 }
 
@@ -300,26 +308,29 @@ static void demux_slave_dispose (demux_plugin_t *this_gen) {
   xine_list_iterator_t ite;
 
   /* free all decoder information */
-  while ((ite = xine_list_front(this->dec_infos)) != NULL) {
-    data = xine_list_get_value(this->dec_infos, ite);
+  ite = NULL;
+  while ((data = xine_list_next_value (this->dec_infos, &ite)))
     free(data);
-    xine_list_remove (this->dec_infos, ite);
-  }
   xine_list_delete(this->dec_infos);
 
   free (this);
 }
 
 static int demux_slave_get_stream_length(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return 0 ; /*FIXME: implement */
 }
 
 static uint32_t demux_slave_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_slave_get_optional_data(demux_plugin_t *this_gen,
 					void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -327,33 +338,35 @@ static int demux_slave_get_optional_data(demux_plugin_t *this_gen,
 static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *stream,
                                     input_plugin_t *input) {
 
+  static const char slave_id_str[] = "master xine v1\n";
+  const size_t      slave_id_str_len = strlen(slave_id_str);
   demux_slave_t *this;
-  static char slave_id_str[] = "master xine v1\n";
-
-  this         = calloc(1, sizeof(demux_slave_t));
+  char           scratch[sizeof(slave_id_str)];
 
   switch (stream->content_detection_method) {
 
-  case METHOD_BY_CONTENT: {
-
-    if (_x_demux_read_header(input, this->scratch, SCRATCH_SIZE) > 0) {
-      if (!strncmp(this->scratch,slave_id_str,strlen(slave_id_str)))
-        break;
-    }
-
-    free (this);
-    return NULL;
-  }
+  case METHOD_BY_CONTENT:
+    if (_x_demux_read_header (input, scratch, slave_id_str_len) != (int)slave_id_str_len)
+      return NULL;
+    if (memcmp(scratch, slave_id_str, slave_id_str_len))
+      return NULL;
+    break;
 
   case METHOD_BY_MRL:
   case METHOD_EXPLICIT:
-  break;
+    break;
 
   default:
-    free (this);
     return NULL;
   }
 
+  if (input->seek (input, slave_id_str_len, SEEK_SET) != (int)slave_id_str_len)
+    return NULL;
+
+  this = calloc(1, sizeof(demux_slave_t));
+  if (!this)
+    return NULL;
+
   this->stream = stream;
   this->input  = input;
   this->dec_infos = xine_list_new();
@@ -370,7 +383,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   this->status = DEMUX_FINISHED;
 
-  this->input->read(this->input, this->scratch,strlen(slave_id_str));
   this->scratch_used = 0;
 
   memset(this->decoder_info, 0, sizeof(this->decoder_info));
@@ -379,30 +391,32 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static void *init_plugin (xine_t *xine, void *data) {
-  demux_slave_class_t     *this;
+static void *init_plugin (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof(demux_slave_class_t));
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = "";
-  this->demux_class.identifier      = "slave";
-  this->demux_class.mimetypes       = NULL;
-  this->demux_class.extensions      = "slave://";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_slave_class = {
+    .open_plugin     = open_plugin,
+    .description     = "",
+    .identifier      = "slave",
+    .mimetypes       = NULL,
+    .extensions      = "slave://",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_slave_class;
 }
 
 /*
  * exported plugin catalog entry
  */
 static const demuxer_info_t demux_info_slave = {
-  10                       /* priority */
+  .priority = 10,
 };
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_DEMUX, 27, "slave", XINE_VERSION_CODE, &demux_info_slave, init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
