@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2017 the xine project
+ * Copyright (C) 2006-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -72,20 +72,22 @@ typedef struct {
   } header;
 } demux_tta_t;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_tta_class_t;
 
-static int open_tta_file(demux_tta_t *this) {
+static int probe_tta_file(input_plugin_t *input) {
   uint32_t peek;
-  uint32_t framelen;
 
-  if (_x_demux_read_header(this->input, &peek, 4) != 4)
+  if (_x_demux_read_header(input, &peek, 4) != 4)
       return 0;
 
   if ( !_x_is_fourcc(&peek, "TTA1") )
     return 0;
 
+  return 1;
+}
+
+static int open_tta_file(demux_tta_t *this) {
+  uint32_t framelen;
+
   if ( this->input->read(this->input, this->header.buffer, sizeof(this->header)) != sizeof(this->header) )
     return 0;
 
@@ -99,10 +101,16 @@ static int open_tta_file(demux_tta_t *this) {
   }
 
   this->seektable = xine_xcalloc(this->totalframes, sizeof(uint32_t));
-  this->input->read(this->input, (uint8_t*)this->seektable, sizeof(uint32_t)*this->totalframes);
+  if (!this->seektable)
+    return 0;
+
+  if (this->input->read(this->input, (uint8_t*)this->seektable, sizeof(uint32_t)*this->totalframes)
+      != (int)sizeof(uint32_t)*this->totalframes)
+    return 0;
 
   /* Skip the CRC32 */
-  this->input->seek(this->input, 4, SEEK_CUR);
+  if (this->input->seek(this->input, 4, SEEK_CUR) < 0)
+    return 0;
 
   /* Store the offset after the header for seeking */
   this->datastart = this->input->get_current_pos(this->input);
@@ -121,17 +129,19 @@ static int demux_tta_send_chunk(demux_plugin_t *this_gen) {
 
   bytes_to_read = le2me_32(this->seektable[this->currentframe]);
 
-  _x_demux_read_send_data(this->audio_fifo,
-                          this->input,
-                          bytes_to_read,
-                          (int64_t)(FRAME_TIME * this->currentframe * 90000),
-                          BUF_AUDIO_TTA,
-                          /*decoder_flags*/ 0,
-                          (int) ((double) this->currentframe * 65535.0 / this->totalframes),
-                          (int)(FRAME_TIME * this->currentframe * 1000),
-                          (int)(le2me_32(this->header.tta.data_length) * 1000.0 /
-                                le2me_32(this->header.tta.samplerate)),
-                          this->currentframe);
+  if (_x_demux_read_send_data(this->audio_fifo,
+                              this->input,
+                              bytes_to_read,
+                              (int64_t)(FRAME_TIME * this->currentframe * 90000),
+                              BUF_AUDIO_TTA,
+                              /*decoder_flags*/ 0,
+                              (int) ((double) this->currentframe * 65535.0 / this->totalframes),
+                              (int)(FRAME_TIME * this->currentframe * 1000),
+                              (int)(le2me_32(this->header.tta.data_length) * 1000.0 /
+                                    le2me_32(this->header.tta.samplerate)),
+                              this->currentframe) < 0) {
+    this->status = DEMUX_FINISHED;
+  }
 
   this->currentframe++;
 
@@ -148,8 +158,6 @@ static void demux_tta_send_headers(demux_plugin_t *this_gen) {
 
   memset(&wave, 0, sizeof(wave));
 
-  header = malloc(total_size);
-
   this->audio_fifo  = this->stream->audio_fifo;
 
   this->status = DEMUX_OK;
@@ -169,6 +177,10 @@ static void demux_tta_send_headers(demux_plugin_t *this_gen) {
   /* create header */
   wave.cbSize = total_size - sizeof(xine_waveformatex);
 
+  header = malloc(total_size);
+  if (!header)
+    return;
+
   memcpy(header, &wave, sizeof(wave));
   memcpy(header+sizeof(xine_waveformatex), this->header.buffer, sizeof(this->header));
   memcpy(header+sizeof(xine_waveformatex)+sizeof(this->header), this->seektable, sizeof(uint32_t)*this->totalframes);
@@ -185,7 +197,7 @@ static void demux_tta_send_headers(demux_plugin_t *this_gen) {
       buf->type = BUF_AUDIO_TTA;
 
       /* Copy min(bytes_left, max_size) bytes */
-      buf->size = bytes_left < buf->max_size ? bytes_left : buf->max_size;
+      buf->size = (int)bytes_left < buf->max_size ? (int)bytes_left : buf->max_size;
       memcpy(buf->content, header+(total_size-bytes_left), buf->size);
 
       bytes_left -= buf->size;
@@ -253,7 +265,7 @@ static int demux_tta_seek (demux_plugin_t *this_gen,
 static void demux_tta_dispose (demux_plugin_t *this_gen) {
   demux_tta_t *this = (demux_tta_t *) this_gen;
 
-  free(this->seektable);
+  _x_freep(&this->seektable);
   free(this);
 }
 
@@ -269,11 +281,15 @@ static int demux_tta_get_stream_length (demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_tta_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_tta_get_optional_data(demux_plugin_t *this_gen,
                                        void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -282,7 +298,21 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_tta_t    *this;
 
-  this         = calloc(1, sizeof(demux_tta_t));
+  switch (stream->content_detection_method) {
+    case METHOD_BY_MRL:
+    case METHOD_BY_CONTENT:
+    case METHOD_EXPLICIT:
+      if (!probe_tta_file(input))
+        return NULL;
+      break;
+    default:
+      return NULL;
+  }
+
+  this = calloc(1, sizeof(demux_tta_t));
+  if (!this)
+    return NULL;
+
   this->stream = stream;
   this->input  = input;
 
@@ -300,36 +330,27 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   this->seektable = NULL;
 
-  switch (stream->content_detection_method) {
-
-  case METHOD_BY_MRL:
-  case METHOD_BY_CONTENT:
-  case METHOD_EXPLICIT:
-    if (!open_tta_file(this)) {
-      free (this);
-      return NULL;
-    }
-    break;
-
-  default:
-    free (this);
+  if (!open_tta_file(this)) {
+    demux_tta_dispose(&this->demux_plugin);
     return NULL;
   }
 
   return &this->demux_plugin;
 }
 
-void *demux_tta_init_plugin (xine_t *xine, void *data) {
-  demux_tta_class_t     *this;
+void *demux_tta_init_plugin (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof(demux_tta_class_t));
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("True Audio demux plugin");
-  this->demux_class.identifier      = "True Audio";
-  this->demux_class.mimetypes       = NULL;
-  this->demux_class.extensions      = "tta";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_tta_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("True Audio demux plugin"),
+    .identifier      = "True Audio",
+    .mimetypes       = NULL,
+    .extensions      = "tta",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_tta_class;
 }
