@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -318,6 +318,7 @@ struct matroska_track_s {
 #define MATROSKA_CODEC_ID_V_VP8          "V_VP8"
 #define MATROSKA_CODEC_ID_V_VP9          "V_VP9"
 #define MATROSKA_CODEC_ID_V_HEVC         "V_MPEGH/ISO/HEVC"
+#define MATROSKA_CODEC_ID_V_AV1          "V_AV1"
 
 #define MATROSKA_CODEC_ID_A_MPEG1_L1     "A_MPEG/L1"
 #define MATROSKA_CODEC_ID_A_MPEG1_L2     "A_MPEG/L2"
@@ -348,6 +349,7 @@ struct matroska_track_s {
 #define MATROSKA_CODEC_ID_S_UTF8         "S_UTF8"        /* deprecated */
 #define MATROSKA_CODEC_ID_S_SSA          "S_SSA"         /* deprecated */
 #define MATROSKA_CODEC_ID_S_ASS          "S_ASS"         /* deprecated */
+#define MATROSKA_CODEC_ID_S_DVBSUB       "S_DVBSUB"
 #define MATROSKA_CODEC_ID_S_VOBSUB       "S_VOBSUB"
 #define MATROSKA_CODEC_ID_S_HDMV_PGS     "S_HDMV/PGS"
 #define MATROSKA_CODEC_ID_S_HDMV_TEXTST  "S_HDMV/TEXTST"
