@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -47,6 +47,11 @@
 
 #include "../combined/ffmpeg/ffmpeg_compat.h"
 
+/* av_opt_set_int (), _after_ ffmpeg_compat.h */
+#if LIBAVCODEC_VERSION_INT >= XFF_INT_VERSION(56,56,101)
+#  include <libavutil/opt.h>
+#endif
+
 #if XFF_ENCVIDEO == 1
 /* buffer size for encoded mpeg1 stream; will hold one intra frame
  * at 640x480 typical sizes are <50 kB. 512 kB should be plenty */
@@ -84,13 +89,16 @@ static int dxr3_lavc_close(dxr3_driver_t *drv) {
 int dxr3_lavc_init(dxr3_driver_t *drv, plugin_node_t *plugin)
 {
   lavc_data_t* this;
-  XFF_AVCODEC_INIT();
-
-  avcodec_register_all();
   lprintf("lavc init , version %x\n", avcodec_version());
   this = calloc(1, sizeof(lavc_data_t));
   if (!this) return 0;
 
+  (void)plugin;
+
+  XFF_AVCODEC_INIT();
+
+  XFF_AVCODEC_REGISTER_ALL();
+
   this->encoder_data.type             = ENC_LAVC;
   this->encoder_data.on_update_format = lavc_on_update_format;
   this->encoder_data.on_frame_copy    = NULL;
@@ -207,7 +215,10 @@ static int lavc_on_update_format(dxr3_driver_t *drv, dxr3_frame_t *frame)
   this->context->height = frame->oheight;
 
   this->context->gop_size = 0; /*intra frames only */
-#if defined(LIBAVCODEC_VERSION_MAJOR) && LIBAVCODEC_VERSION_MAJOR < 58
+/* TJ. this version is known to have that but maybe its not the first. */
+#if LIBAVCODEC_VERSION_INT >= XFF_INT_VERSION(56,56,101)
+  av_opt_set_int (codec, "motion_est", 0 /* "zero" */, 0);
+#else
   this->context->me_method = ME_ZERO; /*motion estimation type*/
 #endif
 
@@ -246,8 +257,8 @@ static int lavc_on_display_frame(dxr3_driver_t *drv, dxr3_frame_t *frame)
 {
 #if XFF_ENCVIDEO == 1
   int size;
-#else /* 2 */
-  AVPacket pkt = { 0 };
+#else /* 2, 3 */
+  AVPacket pkt = {.data = NULL};
   int ret, got_output;
 #endif
   lavc_data_t* this = (lavc_data_t *)drv->enc;
@@ -255,7 +266,7 @@ static int lavc_on_display_frame(dxr3_driver_t *drv, dxr3_frame_t *frame)
 
   if (frame->vo_frame.bad_frame) return 1;
     /* ignore old frames */
-  if ((frame->vo_frame.pitches[0] != this->context->width) || (frame->oheight != this->context->height)) {
+  if ((frame->vo_frame.pitches[0] != this->context->width) || ((int)frame->oheight != this->context->height)) {
 	frame->vo_frame.free(&frame->vo_frame);
     lprintf("LAVC ignoring frame !!!\n");
     return 1;
@@ -267,15 +278,23 @@ static int lavc_on_display_frame(dxr3_driver_t *drv, dxr3_frame_t *frame)
   /* do the encoding */
 #if XFF_ENCVIDEO == 1
   size = avcodec_encode_video(this->context, this->ffmpeg_buffer, DEFAULT_BUFFER_SIZE, this->picture);
-#else /* 2 */
+#elif XFF_ENCVIDEO == 2
   ret = avcodec_encode_video2(this->context, &pkt, this->picture, &got_output);
+#else /* 3 */
+  got_output = 0;
+  ret = avcodec_send_frame (this->context, this->picture);
+  if ((ret >= 0) || (ret == AVERROR (EAGAIN))) {
+    ret = avcodec_receive_packet (this->context, &pkt);
+    got_output = (ret == 0);
+    ret = (ret == AVERROR (EAGAIN)) ? 0 : ret;
+  }
 #endif
 
   frame->vo_frame.free(&frame->vo_frame);
 
 #if XFF_ENCVIDEO == 1
   if (size < 0)
-#else /* 2 */
+#else /* 2, 3 */
   if (ret < 0)
 #endif
   {
@@ -283,19 +302,19 @@ static int lavc_on_display_frame(dxr3_driver_t *drv, dxr3_frame_t *frame)
         "dxr3_mpeg_encoder: encoding failed\n");
       return 0;
   }
-#if XFF_ENCVIDEO == 2
+#if XFF_ENCVIDEO >= 2
   else if (!got_output)
       return 1;
 #endif
 
 #if XFF_ENCVIDEO == 1
   written = write(drv->fd_video, this->ffmpeg_buffer, size);
-#else
+#else /* 2, 3 */
   written = write(drv->fd_video, pkt.data, pkt.size);
 #endif
 
   if (written < 0) {
-#if XFF_ENCVIDEO == 2
+#if XFF_ENCVIDEO >= 2
       av_packet_unref(&pkt);
 #endif
       xprintf(drv->class->xine, XINE_VERBOSITY_LOG,
@@ -306,7 +325,7 @@ static int lavc_on_display_frame(dxr3_driver_t *drv, dxr3_frame_t *frame)
   if (written != size)
       xprintf(drv->class->xine, XINE_VERBOSITY_LOG,
         "dxr3_mpeg_encoder: Could only write %zd of %d mpeg bytes.\n", written, size);
-#else /* 2 */
+#else /* 2, 3 */
   if (written != pkt.size)
       xprintf(drv->class->xine, XINE_VERBOSITY_LOG,
         "dxr3_mpeg_encoder: Could only write %zd of %d mpeg bytes.\n", written, pkt.size);
