@@ -601,6 +601,8 @@ int vm_get_current_menu(vm_t *vm, int *menuid) {
   int pgcn;
   pgcn = (vm->state).pgcN;
   pgcit = get_PGCIT(vm);
+  if (!pgcit)
+    return 0;
   *menuid = pgcit->pgci_srp[pgcn - 1].entry_id & 0xf ;
   return 1;
 }
@@ -861,43 +863,46 @@ int vm_get_video_scale_permission(vm_t *vm) {
 
 video_attr_t vm_get_video_attr(vm_t *vm) {
   switch ((vm->state).domain) {
-  case VTS_DOMAIN:
-    return vm->vtsi->vtsi_mat->vts_video_attr;
   case VTSM_DOMAIN:
     return vm->vtsi->vtsi_mat->vtsm_video_attr;
   case VMGM_DOMAIN:
   case FP_DOMAIN:
     return vm->vmgi->vmgi_mat->vmgm_video_attr;
   default:
-    abort();
+    assert(!"vm_get_video_attr(): unknown domain");
+    /* fall through */
+  case VTS_DOMAIN:
+    return vm->vtsi->vtsi_mat->vts_video_attr;
   }
 }
 
 audio_attr_t vm_get_audio_attr(vm_t *vm, int streamN) {
   switch ((vm->state).domain) {
-  case VTS_DOMAIN:
-    return vm->vtsi->vtsi_mat->vts_audio_attr[streamN];
   case VTSM_DOMAIN:
     return vm->vtsi->vtsi_mat->vtsm_audio_attr;
   case VMGM_DOMAIN:
   case FP_DOMAIN:
     return vm->vmgi->vmgi_mat->vmgm_audio_attr;
   default:
-    abort();
+    assert(!"vm_get_audio_attr(): invalid domain");
+    /* fall through */
+  case VTS_DOMAIN:
+    return vm->vtsi->vtsi_mat->vts_audio_attr[streamN];
   }
 }
 
 subp_attr_t vm_get_subp_attr(vm_t *vm, int streamN) {
   switch ((vm->state).domain) {
-  case VTS_DOMAIN:
-    return vm->vtsi->vtsi_mat->vts_subp_attr[streamN];
   case VTSM_DOMAIN:
     return vm->vtsi->vtsi_mat->vtsm_subp_attr;
   case VMGM_DOMAIN:
   case FP_DOMAIN:
     return vm->vmgi->vmgi_mat->vmgm_subp_attr;
   default:
-    abort();
+    assert(!"vm_get_subp_attr(): invalid domain");
+    /* fall through */
+  case VTS_DOMAIN:
+    return vm->vtsi->vtsi_mat->vts_subp_attr[streamN];
   }
 }
 
@@ -1629,6 +1634,8 @@ static int set_PGCN(vm_t *vm, int pgcN) {
   
   pgcit = get_PGCIT(vm);
   assert(pgcit != NULL);  /* ?? Make this return -1 instead */
+  if (!pgcit)
+    return 0;
 
   if(pgcN < 1 || pgcN > pgcit->nr_of_pgci_srp) {
 #ifdef TRACE
@@ -1730,6 +1737,8 @@ static int get_ID(vm_t *vm, int id) {
   /* Relies on state to get the correct pgcit. */
   pgcit = get_PGCIT(vm);
   assert(pgcit != NULL);
+  if (!pgcit)
+    return 0;
 #ifdef TRACE
   fprintf(MSG_OUT, "libdvdnav: ** Searching for menu (0x%x) entry PGC\n", id);
 #endif
@@ -1782,7 +1791,9 @@ static int get_PGCN(vm_t *vm) {
 
 static pgcit_t* get_MENU_PGCIT(vm_t *vm, ifo_handle_t *h, uint16_t lang) {
   int i;
-  
+
+  (void)vm;
+
   if(h == NULL || h->pgci_ut == NULL) {
     fprintf(MSG_OUT, "libdvdnav: *** pgci_ut handle is NULL ***\n");
     return NULL; /*  error? */
@@ -1826,7 +1837,8 @@ static pgcit_t* get_PGCIT(vm_t *vm) {
     pgcit = get_MENU_PGCIT(vm, vm->vmgi, (vm->state).registers.SPRM[0]);
     break;
   default:
-    abort();
+    pgcit = NULL;
+    assert(!"get_PGCIT(): invalid domain");
   }
   
   return pgcit;
