@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -41,6 +41,7 @@
 #include <xine/xine_internal.h>
 #include <xine/audio_out.h>
 #include <xine/buffer.h>
+#include "group_w32.h"
 
 #include "qtx/qtxsdk/components.h"
 #include "wine/win32.h"
@@ -134,10 +135,6 @@ typedef int (__cdecl* LPFUNC8)(SoundConverter sc,
 typedef int (__cdecl* LPFUNC9)(SoundConverter sc) ;
 
 
-typedef struct {
-  audio_decoder_class_t   decoder_class;
-} qta_class_t;
-
 typedef struct qta_decoder_s {
   audio_decoder_t     audio_decoder;
 
@@ -230,7 +227,7 @@ static void qta_init_driver (qta_decoder_t *this, buf_element_t *buf) {
 
   this->qtml_dll = LoadLibraryA("qtmlClient.dll");
 
-  if (this->qtml_dll == NULL ) {
+  if (this->qtml_dll == (HINSTANCE)NULL) {
     xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "qt_audio: failed to load dll\n" );
     pthread_mutex_unlock(&win32_codec_mutex);
     _x_message(this->stream, XINE_MSG_LIBRARY_LOAD_ERROR,
@@ -425,7 +422,7 @@ static void qta_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 
   if (buf->decoder_flags & BUF_FLAG_STDHEADER) {
 
-    if (buf->size >= sizeof(xine_waveformatex))
+    if (buf->size >= (int)sizeof(xine_waveformatex))
       memcpy (&this->wave, buf->content, sizeof (xine_waveformatex));
 
     this->wave.nChannels      = buf->decoder_info[3];
@@ -518,6 +515,7 @@ static void qta_reset (audio_decoder_t *this_gen) {
 
 
 static void qta_discontinuity (audio_decoder_t *this_gen) {
+  (void)this_gen;
 }
 
 static void qta_dispose (audio_decoder_t *this_gen) {
@@ -551,55 +549,39 @@ static void qta_dispose (audio_decoder_t *this_gen) {
 static audio_decoder_t *qta_open_plugin (audio_decoder_class_t *class_gen,
 					 xine_stream_t *stream) {
 
-  qta_decoder_t *this ;
+  qta_decoder_t *this = calloc (1, sizeof (*this));
+  if (!this)
+    return NULL;
 
-  this = (qta_decoder_t *) calloc(1, sizeof(qta_decoder_t));
-
-  this->audio_decoder.decode_data         = qta_decode_data;
-  this->audio_decoder.reset               = qta_reset;
-  this->audio_decoder.discontinuity       = qta_discontinuity;
-  this->audio_decoder.dispose             = qta_dispose;
-  this->stream                            = stream;
-
-  this->output_open     = 0;
-
-  return (audio_decoder_t *) this;
+  this->audio_decoder.decode_data   = qta_decode_data;
+  this->audio_decoder.reset         = qta_reset;
+  this->audio_decoder.discontinuity = qta_discontinuity;
+  this->audio_decoder.dispose       = qta_dispose;
+  this->stream                      = stream;
+#ifndef HAVE_ZERO_SAFE_MEM
+  this->output_open = 0;
+#endif
+  (void)class_gen;
+  return &this->audio_decoder;
 }
 
 /*
  * qta plugin class
  */
-static void *qta_init_class (xine_t *xine, void *data) {
-
-  qta_class_t     *this;
-  config_values_t *cfg;
-
-  cfg = xine->config;
-  if ((win32_def_path = get_win32_codecs_path(cfg)) == NULL) return NULL;
-
+void *qta_init_class (xine_t *xine, const void *data) {
+  static const audio_decoder_class_t this = {
+    .open_plugin     = qta_open_plugin,
+    .identifier      = "qta",
+    .description     = N_("quicktime audio decoder plugin"),
+    .dispose         = NULL
+  };
+  (void)data;
+  if ((win32_def_path = get_win32_codecs_path (xine->config)) == NULL)
+    return NULL;
   pthread_once (&once_control, init_routine);
-
-  this = (qta_class_t *) calloc(1, sizeof(qta_class_t));
-
-  this->decoder_class.open_plugin     = qta_open_plugin;
-  this->decoder_class.identifier      = "qta";
-  this->decoder_class.description     = N_("quicktime audio decoder plugin");
-  this->decoder_class.dispose         = default_audio_decoder_class_dispose;
-
-  return this;
+  return (audio_decoder_class_t *)&this;
 }
 
-static const uint32_t audio_types[] = {
-  BUF_AUDIO_QDESIGN1,
-  BUF_AUDIO_QDESIGN2,
-  BUF_AUDIO_QCLP,
-  0
-};
-
-static const decoder_info_t qta_dec_info = {
-  audio_types,         /* supported types */
-  1                    /* priority        */
-};
 
 /*
  *
@@ -608,19 +590,10 @@ static const decoder_info_t qta_dec_info = {
  *
  */
 
-typedef struct {
-  video_decoder_class_t   decoder_class;
-
-  char                   *qt_codec_path;
-
-} qtv_class_t;
-
 
 typedef struct qtv_decoder_s {
   video_decoder_t  video_decoder;
 
-  qtv_class_t     *cls;
-
   xine_stream_t   *stream;
 
   HINSTANCE        qtml_dll;
@@ -749,7 +722,7 @@ static void qtv_init_driver (qtv_decoder_t *this, buf_element_t *buf) {
 
   this->qtml_dll = LoadLibraryA("qtmlClient.dll");
 
-  if (this->qtml_dll == NULL ) {
+  if (this->qtml_dll == (HINSTANCE)NULL) {
     xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "qt_video: failed to load dll\n" );
     pthread_mutex_unlock(&win32_codec_mutex);
     _x_message(this->stream, XINE_MSG_LIBRARY_LOAD_ERROR,
@@ -1013,7 +986,7 @@ static void qtv_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
 static void qtv_flush (video_decoder_t *this_gen) {
   /* qtv_decoder_t *this = (qtv_decoder_t *) this_gen; */
-
+  (void)this_gen;
   lprintf ("video: flush\n");
 }
 
@@ -1025,7 +998,7 @@ static void qtv_reset (video_decoder_t *this_gen) {
 
 static void qtv_discontinuity (video_decoder_t *this_gen) {
   /* qtv_decoder_t *this = (qtv_decoder_t *) this_gen; */
-
+  (void)this_gen;
 }
 
 static void qtv_dispose (video_decoder_t *this_gen) {
@@ -1047,19 +1020,17 @@ static void qtv_dispose (video_decoder_t *this_gen) {
 static video_decoder_t *qtv_open_plugin (video_decoder_class_t *class_gen,
 					 xine_stream_t *stream) {
 
-  qtv_class_t   *cls = (qtv_class_t *) class_gen;
-  qtv_decoder_t *this ;
-
-  this = (qtv_decoder_t *) calloc(1, sizeof(qtv_decoder_t));
-
-  this->video_decoder.decode_data         = qtv_decode_data;
-  this->video_decoder.flush               = qtv_flush;
-  this->video_decoder.reset               = qtv_reset;
-  this->video_decoder.discontinuity       = qtv_discontinuity;
-  this->video_decoder.dispose             = qtv_dispose;
-  this->stream                            = stream;
-  this->cls                               = cls;
+  qtv_decoder_t *this = calloc (1, sizeof (*this));
+  if (!this)
+    return NULL;
 
+  this->video_decoder.decode_data   = qtv_decode_data;
+  this->video_decoder.flush         = qtv_flush;
+  this->video_decoder.reset         = qtv_reset;
+  this->video_decoder.discontinuity = qtv_discontinuity;
+  this->video_decoder.dispose       = qtv_dispose;
+  this->stream                      = stream;
+  (void)class_gen;
   return &this->video_decoder;
 }
 
@@ -1080,41 +1051,18 @@ static void codec_path_cb (void *data, xine_cfg_entry_t *cfg) {
 }
 #endif
 
-static void *qtv_init_class (xine_t *xine, void *data) {
-
-  qtv_class_t        *this;
-  config_values_t    *cfg = xine->config;
-
-  if ((win32_def_path = get_win32_codecs_path(cfg)) == NULL) return NULL;
-
+void *qtv_init_class (xine_t *xine, const void *data) {
+  static const video_decoder_class_t this = {
+    .open_plugin     = qtv_open_plugin,
+    .identifier      = "qtvdec",
+    .description     = N_("quicktime binary-only codec based video decoder plugin"),
+    .dispose         = NULL
+  };
   lprintf ("%s...\n", __XINE_FUNCTION__);
-
+  (void)data;
+  if ((win32_def_path = get_win32_codecs_path (xine->config)) == NULL)
+    return NULL;
   pthread_once (&once_control, init_routine);
-
-  this = (qtv_class_t *) calloc(1, sizeof(qtv_class_t));
-
-  this->decoder_class.open_plugin     = qtv_open_plugin;
-  this->decoder_class.identifier      = "qtvdec";
-  this->decoder_class.description     = N_("quicktime binary-only codec based video decoder plugin");
-  this->decoder_class.dispose         = default_video_decoder_class_dispose;
-
-  return this;
+  return (video_decoder_class_t *)&this;
 }
 
-/*
- * exported plugin catalog entry
- */
-
-static const uint32_t qtv_supported_types[] = { BUF_VIDEO_SORENSON_V3, 0 };
-
-static const decoder_info_t qtv_dec_info = {
-  qtv_supported_types,     /* supported types */
-  1                        /* priority        */
-};
-
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_VIDEO_DECODER | PLUGIN_MUST_PRELOAD, 19, "qtv", XINE_VERSION_CODE, &qtv_dec_info, qtv_init_class },
-  { PLUGIN_AUDIO_DECODER | PLUGIN_MUST_PRELOAD, 16, "qta", XINE_VERSION_CODE, &qta_dec_info, qta_init_class },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
