@@ -120,10 +120,12 @@ void (*composite_bars_packed4444_scanline)( uint8_t *output,
 void (*packed444_to_nonpremultiplied_packed4444_scanline)( uint8_t *output,
                                                            uint8_t *input,
                                                            int width, int alpha );
+#if 0
 void (*aspect_adjust_packed4444_scanline)( uint8_t *output,
                                            uint8_t *input,
                                            int width,
                                            double pixel_aspect );
+#endif
 void (*packed444_to_packed422_scanline)( uint8_t *output,
                                          uint8_t *input,
                                          int width );
@@ -345,8 +347,8 @@ static unsigned int diff_factor_packed422_scanline_mmx( uint8_t *cur, uint8_t *o
 
 #if defined(ARCH_X86)
 
-static const sse_t dqwYMask = { uq: { 0x00ff00ff00ff00ffULL, 0x00ff00ff00ff00ffULL }};
-static const sse_t dqwCMask = { uq: { 0xff00ff00ff00ff00ULL, 0xff00ff00ff00ff00ULL }};
+static const sse_t dqwYMask = { .uq = { 0x00ff00ff00ff00ffULL, 0x00ff00ff00ff00ffULL }};
+static const sse_t dqwCMask = { .uq = { 0xff00ff00ff00ff00ULL, 0xff00ff00ff00ff00ULL }};
 
 static unsigned int diff_factor_packed422_scanline_sse2_aligned( uint8_t *cur, uint8_t *old, int width )
 {
@@ -1734,7 +1736,7 @@ static void composite_alphamask_to_packed4444_scanline_c( uint8_t *output,
 
         if( a == 0xff ) {
             *((uint32_t *) output) = opaque;
-        } else if( (input[ 0 ] == 0x00) ) {
+        } else if( input[ 0 ] == 0x00 ) {
             *((uint32_t *) output) = (multiply_alpha( a, textcr ) << 24)
                                        | (multiply_alpha( a, textcb ) << 16)
                                        | (multiply_alpha( a, textluma ) << 8) | a;
@@ -1785,7 +1787,7 @@ static void composite_alphamask_to_packed4444_scanline_mmxext( uint8_t *output,
 
         if( a == 0xff ) {
             *((uint32_t *) output) = opaque;
-        } else if( (input[ 0 ] == 0x00) ) {
+        } else if( input[ 0 ] == 0x00 ) {
             /* We just need to multiply our colour by the alpha value. */
 
             /* mm2 = [ a ][ a ][ a ][ a ] */
@@ -2352,6 +2354,7 @@ static void packed444_to_nonpremultiplied_packed4444_scanline_c( uint8_t *output
     }
 }
 
+#if 0
 static void aspect_adjust_packed4444_scanline_c( uint8_t *output,
                                                  uint8_t *input,
                                                  int width,
@@ -2397,7 +2400,7 @@ static void aspect_adjust_packed4444_scanline_c( uint8_t *output,
         w++;
     }
 }
-
+#endif
 /**
  * Sub-pixel data bar renderer.  There are 128 bars.
  */
@@ -2608,7 +2611,9 @@ void setup_speedy_calls( uint32_t accel, int verbose )
     subpix_blit_vertical_packed422_scanline = subpix_blit_vertical_packed422_scanline_c;
     composite_bars_packed4444_scanline = composite_bars_packed4444_scanline_c;
     packed444_to_nonpremultiplied_packed4444_scanline = packed444_to_nonpremultiplied_packed4444_scanline_c;
+#if 0
     aspect_adjust_packed4444_scanline = aspect_adjust_packed4444_scanline_c;
+#endif
     packed444_to_packed422_scanline = packed444_to_packed422_scanline_c;
     packed422_to_packed444_scanline = packed422_to_packed444_scanline_c;
     packed422_to_packed444_rec601_scanline = packed422_to_packed444_rec601_scanline_c;
