@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2018 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * Copyright (C) James Courtier-Dutton James@superbug.demon.co.uk - July 2001
  *
@@ -98,13 +98,11 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
 #ifdef LOG_DEBUG
     printf("libspudec: SPU CLUT\n");
 #endif
-    if (buf->content[0]) { /* cheap endianess detection */
-      xine_fast_memcpy(this->state.clut, buf->content, sizeof(uint32_t)*16);
-    } else {
+    memcpy(this->state.clut, buf->content, sizeof(uint32_t)*16);
+    if (!buf->content[0]) { /* cheap endianess detection */
       int i;
-      uint32_t *clut = (uint32_t*) buf->content;
       for (i = 0; i < 16; i++)
-        this->state.clut[i] = bswap_32(clut[i]);
+        this->state.clut[i] = bswap_32(this->state.clut[i]);
     }
     this->state.need_clut = 0;
     return;
@@ -291,7 +289,7 @@ static void spudec_set_button (spu_decoder_t *this_gen, int32_t button, int32_t
     xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
 	    "libspudec:xine_decoder.c:spudec_event_listener:HIDE ????\n");
     printf("We dropped out here for some reason");
-    _x_abort();
+    _x_assert(show > 0);
     overlay_event->object.handle = this->menu_handle;
     overlay_event->event_type = OVERLAY_EVENT_HIDE;
   }
@@ -351,7 +349,7 @@ static spu_decoder_t *open_plugin (spu_decoder_class_t *class_gen, xine_stream_t
   }
 
 /* FIXME:Do we really need a default clut? */
-  xine_fast_memcpy(this->state.clut, default_clut, sizeof(this->state.clut));
+  memcpy(this->state.clut, default_clut, sizeof(this->state.clut));
   this->state.need_clut = 1;
   this->state.vobsub = 0;
 
