@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2018 the xine project
+ * Copyright (C) 2000-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -98,15 +98,6 @@ typedef struct {
 
 } demux_sputext_t;
 
-typedef struct demux_sputext_class_s {
-
-  demux_class_t      demux_class;
-
-  xine_t            *xine;
-  int                max_timeout;  /* default timeout of hidding subtitles */
-
-} demux_sputext_class_t;
-
 /*
  * Demuxer code start
  */
@@ -309,7 +300,8 @@ static subtitle_t *sub_read_line_microdvd(demux_sputext_t *this, subtitle_t *cur
 static subtitle_t *sub_read_line_subviewer(demux_sputext_t *this, subtitle_t *current) {
 
   char line[LINE_LEN + 1];
-  int a1,a2,a3,a4,b1,b2,b3,b4;
+  int a1,a2,b1,b2;
+  float a3,b3;
   char *p=NULL, *q=NULL;
   int len;
 
@@ -317,12 +309,11 @@ static subtitle_t *sub_read_line_subviewer(demux_sputext_t *this, subtitle_t *cu
 
   while (1) {
     if (!read_line_from_input(this, line, LINE_LEN)) return NULL;
-    if (sscanf (line, "%d:%d:%d.%d,%d:%d:%d.%d",&a1,&a2,&a3,&a4,&b1,&b2,&b3,&b4) < 8) {
-      if (sscanf (line, "%d:%d:%d,%d,%d:%d:%d,%d",&a1,&a2,&a3,&a4,&b1,&b2,&b3,&b4) < 8)
+    if (sscanf (line, "%d:%d:%f,%d:%d:%f",&a1,&a2,&a3,&b1,&b2,&b3) < 6) {
         continue;
     }
-    current->start = a1*360000+a2*6000+a3*100+a4;
-    current->end   = b1*360000+b2*6000+b3*100+b4;
+    current->start = a1*360000+a2*6000+a3*100.0;
+    current->end   = b1*360000+b2*6000+b3*100.0;
 
     if (!read_line_from_input(this, line, LINE_LEN))
       return NULL;
@@ -344,17 +335,21 @@ static subtitle_t *sub_read_line_subviewer(demux_sputext_t *this, subtitle_t *cu
 
 static subtitle_t *sub_read_line_subrip(demux_sputext_t *this,subtitle_t *current) {
   char line[LINE_LEN + 1];
-  int a1,a2,a3,a4,b1,b2,b3,b4;
+  int a1,a2,b1,b2;
+  float a3,b3;
   int i,end_sub;
 
   memset(current,0,sizeof(subtitle_t));
   do {
+    char *p;
     if(!read_line_from_input(this,line,LINE_LEN))
       return NULL;
-    i = sscanf(line,"%d:%d:%d%*[,.]%d --> %d:%d:%d%*[,.]%d",&a1,&a2,&a3,&a4,&b1,&b2,&b3,&b4);
-  } while(i < 8);
-  current->start = a1*360000+a2*6000+a3*100+a4/10;
-  current->end   = b1*360000+b2*6000+b3*100+b4/10;
+    while ((p = strchr(line, ',')))
+	*p = '.';
+    i = sscanf(line,"%d:%d:%f --> %d:%d:%f",&a1,&a2,&a3,&b1,&b2,&b3);
+  } while(i < 6);
+  current->start = a1*360000+a2*6000+a3*100.0;
+  current->end   = b1*360000+b2*6000+b3*100.0;
   i=0;
   end_sub=0;
   do {
@@ -422,8 +417,7 @@ static subtitle_t *sub_read_line_vplayer(demux_sputext_t *this,subtitle_t *curre
       if( !read_line_from_input(this, line, LINE_LEN) ) return NULL;
     } else {
       /* ... get the current line from buffer. */
-      strncpy( line, this->next_line, LINE_LEN);
-      line[LINE_LEN] = '\0'; /* I'm scared. This makes me feel better. */
+      strlcpy( line, this->next_line, sizeof(line));
       this->next_line[0] = '\0'; /* mark the buffer as empty. */
     }
     /* Initialize buffer with next line */
@@ -471,7 +465,8 @@ static subtitle_t *sub_read_line_rt(demux_sputext_t *this,subtitle_t *current) {
    * WARNING: full XML parses can be required for proper parsing
    */
   char line[LINE_LEN + 1];
-  int a1,a2,a3,a4,b1,b2,b3,b4;
+  int a1,a2,b1,b2;
+  float a3,b3;
   char *p=NULL,*next=NULL;
   int i,len,plen;
 
@@ -483,19 +478,13 @@ static subtitle_t *sub_read_line_rt(demux_sputext_t *this,subtitle_t *current) {
      * TODO: it seems that format of time is not easily determined, it may be 1:12, 1:12.0 or 0:1:12.0
      * to describe the same moment in time. Maybe there are even more formats in use.
      */
-    if ((len=sscanf (line, "<Time Begin=\"%d:%d:%d.%d\" End=\"%d:%d:%d.%d\"",&a1,&a2,&a3,&a4,&b1,&b2,&b3,&b4)) < 8)
+    if ((len=sscanf (line, "<Time Begin=\"%d:%d:%f\" End=\"%d:%d:%f\"",&a1,&a2,&a3,&b1,&b2,&b3)) < 6)
 
-      plen=a1=a2=a3=a4=b1=b2=b3=b4=0;
-    if (
-	((len=sscanf (line, "<%*[tT]ime %*[bB]egin=\"%d:%d\" %*[Ee]nd=\"%d:%d\"%*[^<]<clear/>%n",&a2,&a3,&b2,&b3,&plen)) < 4) &&
-	((len=sscanf (line, "<%*[tT]ime %*[bB]egin=\"%d:%d\" %*[Ee]nd=\"%d:%d.%d\"%*[^<]<clear/>%n",&a2,&a3,&b2,&b3,&b4,&plen)) < 5) &&
-	/*	((len=sscanf (line, "<%*[tT]ime %*[bB]egin=\"%d:%d.%d\" %*[Ee]nd=\"%d:%d\"%*[^<]<clear/>%n",&a2,&a3,&a4,&b2,&b3,&plen)) < 5) && */
-	((len=sscanf (line, "<%*[tT]ime %*[bB]egin=\"%d:%d.%d\" %*[Ee]nd=\"%d:%d.%d\"%*[^<]<clear/>%n",&a2,&a3,&a4,&b2,&b3,&b4,&plen)) < 6) &&
-	((len=sscanf (line, "<%*[tT]ime %*[bB]egin=\"%d:%d:%d.%d\" %*[Ee]nd=\"%d:%d:%d.%d\"%*[^<]<clear/>%n",&a1,&a2,&a3,&a4,&b1,&b2,&b3,&b4,&plen)) < 8)
-	)
+      plen=a1=a2=a3=b1=b2=b3=0;
+    if ((len=sscanf (line, "<%*[tT]ime %*[bB]egin=\"%d:%f\" %*[Ee]nd=\"%d:%f\"%n",&a2,&a3,&b2,&b3,&plen)) < 4)
       continue;
-    current->start = a1*360000+a2*6000+a3*100+a4/10;
-    current->end   = b1*360000+b2*6000+b3*100+b4/10;
+    current->start = a1*360000+a2*6000+a3*100.0;
+    current->end   = b1*360000+b2*6000+b3*100.0;
     p=line;	p+=plen;i=0;
     /* TODO: I don't know what kind of convention is here for marking multiline subs, maybe <br/> like in xml? */
     next = strstr(line,"<clear/>")+8;i=0;
@@ -519,24 +508,25 @@ static subtitle_t *sub_read_line_ssa(demux_sputext_t *this,subtitle_t *current)
   static int max_comma = 32; /* let's use 32 for the case that the */
   /*  amount of commas increase with newer SSA versions */
 
-  int hour1, min1, sec1, hunsec1, hour2, min2, sec2, hunsec2, nothing;
+  int hour1, min1, hour2, min2, nothing;
+  float sec1, sec2;
   int num;
   char line[LINE_LEN + 1], line3[LINE_LEN + 1], *line2;
   char *tmp;
 
   do {
     if (!read_line_from_input(this, line, LINE_LEN)) return NULL;
-  } while (sscanf (line, "Dialogue: Marked=%d,%d:%d:%d.%d,%d:%d:%d.%d,"
+  } while (sscanf (line, "Dialogue: Marked=%d,%d:%d:%f,%d:%d:%f,"
 		   "%[^\n\r]", &nothing,
-		   &hour1, &min1, &sec1, &hunsec1,
-		   &hour2, &min2, &sec2, &hunsec2,
-		   line3) < 9
+		   &hour1, &min1, &sec1,
+		   &hour2, &min2, &sec2,
+		   line3) < 7
 	   &&
-	   sscanf (line, "Dialogue: %d,%d:%d:%d.%d,%d:%d:%d.%d,"
+	   sscanf (line, "Dialogue: %d,%d:%d:%f,%d:%d:%f,"
 		   "%[^\n\r]", &nothing,
-		   &hour1, &min1, &sec1, &hunsec1,
-		   &hour2, &min2, &sec2, &hunsec2,
-		   line3) < 9	    );
+		   &hour1, &min1, &sec1,
+		   &hour2, &min2, &sec2,
+		   line3) < 7	    );
 
   line2=strchr(line3, ',');
   if (!line2)
@@ -556,8 +546,8 @@ static subtitle_t *sub_read_line_ssa(demux_sputext_t *this,subtitle_t *current)
   if(*line2 == ',') line2++;
 
   current->lines=0;num=0;
-  current->start = 360000*hour1 + 6000*min1 + 100*sec1 + hunsec1;
-  current->end   = 360000*hour2 + 6000*min2 + 100*sec2 + hunsec2;
+  current->start = 360000*hour1 + 6000*min1 + 100.0*sec1;
+  current->end   = 360000*hour2 + 6000*min2 + 100.0*sec2;
 
   while (((tmp=strstr(line2, "\\n")) != NULL) || ((tmp=strstr(line2, "\\N")) != NULL) ){
     current->text[num] = strndup(line2, tmp-line2);
@@ -741,17 +731,17 @@ static subtitle_t *sub_read_line_jacobsub(demux_sputext_t *this, subtitle_t *cur
 				if (sscanf
 				    (&line1[delta], "%*d:%*d:%d",
 				     &seconds)) {
-				    sscanf(&line1[delta], "%*d:%*d:%*d.%d",
+                                    sscanf(&line1[delta], "%*d:%*d:%*d.%u",
 					   &units);
 				} else {
 				    hours = 0;
-				    sscanf(&line1[delta], "%d:%d.%d",
+                                    sscanf(&line1[delta], "%d:%d.%u",
 					   &minutes, &seconds, &units);
 				    minutes *= inverter;
 				}
 			    } else {
 				hours = minutes = 0;
-				sscanf(&line1[delta], "%d.%d", &seconds,
+                                sscanf(&line1[delta], "%d.%u", &seconds,
 				       &units);
 				seconds *= inverter;
 			    }
@@ -877,9 +867,7 @@ static subtitle_t *sub_read_line_jacobsub(demux_sputext_t *this, subtitle_t *cur
 		    if (!read_line_from_input(this, directive, LINE_LEN))
 			return NULL;
 		    trail_space(directive);
-		    strncat(line2, directive,
-			    ((LINE_LEN > 511) ? LINE_LEN-1 : 511)
-			    - strlen(line2));
+		    strlcat(line2, directive, ((LINE_LEN > 511) ? LINE_LEN-1 : 511));
 		    break;
 		}
                 /* fall through */
@@ -1193,8 +1181,13 @@ static subtitle_t *sub_read_file (demux_sputext_t *this) {
   this->num=0;n_max=32;
   first = calloc(n_max, sizeof(subtitle_t));
   if(!first) return NULL;
-  timeout = ((demux_sputext_class_t *)
-             (this->demux_plugin.demux_class))->max_timeout;
+
+  {
+    cfg_entry_t *entry;
+    entry = this->stream->xine->config->lookup_entry (this->stream->xine->config,
+                                                      "subtitles.separate.timeout");
+    timeout = entry ? entry->num_value : 4;
+  }
   if (this->uses_time) timeout *= 100;
   else timeout *= 10;
 
@@ -1281,8 +1274,7 @@ static int demux_sputext_next (demux_sputext_t *this_gen) {
   *val++ = (this->uses_time) ? sub->end * 10 : sub->end;
   str = (char *)val;
   for (line = 0; line < sub->lines; line++, str+=strlen(str)+1) {
-    strncpy(str, sub->text[line], SUB_BUFSIZE-1);
-    str[SUB_BUFSIZE-1] = '\0';
+    strlcpy(str, sub->text[line], SUB_BUFSIZE);
   }
 
   if (this->encoding) {
@@ -1478,57 +1470,35 @@ static demux_plugin_t *open_demux_plugin (demux_class_t *class_gen, xine_stream_
   return NULL;
 }
 
-static void config_timeout_cb(void *this_gen, xine_cfg_entry_t *entry) {
-  demux_sputext_class_t *this = (demux_sputext_class_t *)this_gen;
-
-  this->max_timeout = entry->num_value;
-}
-
-static void sputext_demux_class_dispose(demux_class_t *this_gen)
-{
-  demux_sputext_class_t *this = (demux_sputext_class_t *)this_gen;
-
-  this->xine->config->unregister_callback(this->xine->config, "subtitles.separate.timeout");
-
-  free(this);
-}
-
 void *init_sputext_demux_class (xine_t *xine, const void *data) {
 
-  demux_sputext_class_t *this ;
-
-  lprintf("initializing\n");
+  static const demux_class_t demux_class = {
+    .open_plugin     = open_demux_plugin,
+    .description     = N_("sputext demuxer plugin"),
+    .identifier      = "sputext",
+    /* do not report this mimetype, it might confuse browsers. */
+    /* "text/plain: asc txt sub srt: VIDEO subtitles;" */
+    .mimetypes       = NULL,
+    .extensions      = "asc txt sub srt smi ssa ass",
+    .dispose         = NULL,
+  };
+  config_values_t *config = xine->config;
 
   (void)xine;
   (void)data;
 
-  this = calloc(1, sizeof (demux_sputext_class_t));
-  if (!this)
-    return NULL;
-
-  this->demux_class.open_plugin     = open_demux_plugin;
-  this->demux_class.description     = N_("sputext demuxer plugin");
-  this->demux_class.identifier      = "sputext";
-  /* do not report this mimetype, it might confuse browsers. */
-  /* "text/plain: asc txt sub srt: VIDEO subtitles;" */
-  this->demux_class.mimetypes       = NULL;
-  this->demux_class.extensions      = "asc txt sub srt smi ssa ass";
-  this->demux_class.dispose         = sputext_demux_class_dispose;
-
-  this->xine = xine;
-
   /*
    * Some subtitling formats, namely AQT and Subrip09, define the end of a
    * subtitle as the beginning of the following. From end-user view it's
    * better define timeout of hidding. Setting to zero means "no timeout".
    */
-  this->max_timeout = xine->config->register_num(xine->config,
-                         "subtitles.separate.timeout", 4,
-			 _("default duration of subtitle display in seconds"),
-			 _("Some subtitle formats do not explicitly give a duration for each subtitle. "
-			   "For these, you can set a default duration here. Setting to zero will result "
-			   "in the subtitle being shown until the next one takes over."),
-			 20, config_timeout_cb, this);
+  config->register_num(config,
+                       "subtitles.separate.timeout", 4,
+                       _("default duration of subtitle display in seconds"),
+                       _("Some subtitle formats do not explicitly give a duration for each subtitle. "
+                         "For these, you can set a default duration here. Setting to zero will result "
+                         "in the subtitle being shown until the next one takes over."),
+                       20, NULL, NULL);
 
-  return this;
+  return (void *)&demux_class;
 }
