@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2019 the xine project
+ * Copyright (C) 2003-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -25,6 +25,10 @@
  * looks like ordinary absolute discontinuities. We need to watch the control
  * messages coming through a side channel, and inject apropriate xine engine calls
  * manually. In reverse, we listen to xine events, and send back vdr keys.
+ * "Trick play" is turned on and off by server. When on, xine shall just play all
+ * frames as if they had perfectly consecutive time stamps. We still need to register
+ * first discontinuity early because server will wait for it, and video decoder may
+ * delay it -> freeze.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -67,7 +71,8 @@
 #include <xine/vdr.h>
 #include "combined_vdr.h"
 
-
+#define VDR_DISC_START (('V' << 24) | ('D' << 16) | ('R' << 8) | 1)
+#define VDR_DISC_STOP  (('V' << 24) | ('D' << 16) | ('R' << 8) | 0)
 
 #define VDR_MAX_NUM_WINDOWS 16
 #define VDR_ABS_FIFO_DIR "/tmp/vdr-xine"
@@ -91,10 +96,13 @@ typedef struct {
   metronom_t         *stream_metronom;
   vdr_input_plugin_t *input;
   pthread_mutex_t     mutex;
-  int                 disc_num_audio;
-  int                 disc_num_video;
-  int                 audio_seek;
-  int                 video_seek;
+  struct {
+    int               disc_num;
+    int               seek;
+    int               on;
+  } audio, video;
+  /* -1 = unset, 0 = off, 1 = on, 2 = first disc sent. */
+  int                 trick_new_mode, trick_mode;
 }
 vdr_metronom_t;
 
@@ -132,7 +140,6 @@ struct vdr_input_plugin_s
   char               *mrl;
 
   off_t               curpos;
-  char                seek_buf[ BUF_SIZE ];
 
   enum funcs          cur_func;
   off_t               cur_size;
@@ -151,11 +158,6 @@ struct vdr_input_plugin_s
   int                 last_volume;
   vdr_frame_size_changed_data_t frame_size;
 
-  uint8_t             trick_speed_mode;
-  uint8_t             trick_speed_mode_blocked;
-  pthread_mutex_t     trick_speed_mode_lock;
-  pthread_cond_t      trick_speed_mode_cond;
-
   pthread_t           rpc_thread;
   int                 rpc_thread_created;
   int                 rpc_thread_shutdown;
@@ -181,16 +183,31 @@ struct vdr_input_plugin_s
 #define OFFS_RING_LD 7
 #define OFFS_RING_NUM (1 << OFFS_RING_LD)
 #define OFFS_RING_MASK (OFFS_RING_NUM - 1)
-  vdr_vpts_offset_t   vpts_offsets[OFFS_RING_NUM];
-  int                 vpts_offset_read;
-  int                 vpts_offset_write;
-  pthread_mutex_t     vpts_offset_queue_lock;
-  pthread_cond_t      vpts_offset_queue_changed_cond;
+  struct {
+    vdr_vpts_offset_t items[OFFS_RING_NUM];
+    int               read;
+    int               write;
+    pthread_mutex_t   lock;
+    pthread_cond_t    changed;
+  } vpts_offs_queue;
 
   int                         video_window_active;
   vdr_set_video_window_data_t video_window_event_data;
+
+  uint8_t             seek_buf[BUF_SIZE];
 };
 
+static void trick_speed_send_event (vdr_input_plugin_t *this, int mode) {
+  xine_event_t event;
+
+  xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
+    "input_vdr: trick play mode now %d.\n", mode);
+  _x_demux_seek (this->stream, 0, 0, 0);
+  event.type = XINE_EVENT_VDR_TRICKSPEEDMODE;
+  event.data = NULL;
+  event.data_length = mode;
+  xine_event_send (this->stream, &event);
+}
 
 static int vdr_write(int f, void *b, int n)
 {
@@ -290,9 +307,8 @@ static void external_stream_play(vdr_input_plugin_t *this, char *file_name)
   }
 }
 
-static off_t vdr_read_abort(xine_stream_t *stream, int fd, char *buf, off_t todo)
-{
-  off_t ret;
+static ssize_t vdr_read_abort (xine_stream_t *stream, int fd, uint8_t *buf, size_t todo) {
+  ssize_t ret;
 
   while (1)
   {
@@ -303,7 +319,7 @@ static off_t vdr_read_abort(xine_stream_t *stream, int fd, char *buf, off_t todo
      * check cancellation.
      */
     pthread_testcancel();
-    ret = _x_read_abort(stream, fd, buf, todo);
+    ret = _x_read_abort (stream, fd, (char *)buf, todo);
     pthread_testcancel();
 
     if (ret < 0
@@ -323,7 +339,7 @@ static off_t vdr_read_abort(xine_stream_t *stream, int fd, char *buf, off_t todo
   data_##kind##_t *data = &data_union.kind; \
   { \
     log; \
-    n = vdr_read_abort(this->stream, this->fh_control, (char *)data + sizeof (data->header), sizeof (*data) - sizeof (data->header)); \
+    n = vdr_read_abort (this->stream, this->fh_control, (uint8_t *)data + sizeof (data->header), sizeof (*data) - sizeof (data->header)); \
     if (n != sizeof (*data) - sizeof (data->header)) \
       return -1; \
     \
@@ -368,45 +384,54 @@ static void adjust_zoom(vdr_input_plugin_t *this)
 
 
 static void vdr_vpts_offset_queue_init (vdr_input_plugin_t *this) {
-  this->vpts_offset_read = 0;
-  this->vpts_offset_write = 1;
-  this->vpts_offsets[0].offset = this->metronom.stream_metronom->get_option (this->metronom.stream_metronom, METRONOM_VPTS_OFFSET);
-  this->vpts_offsets[0].vpts = xine_get_current_vpts (this->stream);
+  pthread_mutex_init (&this->vpts_offs_queue.lock, NULL);
+  pthread_cond_init (&this->vpts_offs_queue.changed, NULL);
+  this->metronom.stream_metronom = this->stream->metronom;
+  this->vpts_offs_queue.read = 0;
+  this->vpts_offs_queue.write = 1;
+  this->vpts_offs_queue.items[0].offset = this->metronom.stream_metronom->get_option (this->metronom.stream_metronom,
+    METRONOM_VPTS_OFFSET);
+  this->vpts_offs_queue.items[0].vpts = xine_get_current_vpts (this->stream);
+}
+
+static void vdr_vpts_offset_queue_deinit (vdr_input_plugin_t *this) {
+  pthread_cond_destroy (&this->vpts_offs_queue.changed);
+  pthread_mutex_destroy (&this->vpts_offs_queue.lock);
 }
 
 static void vdr_vpts_offset_queue_process (vdr_input_plugin_t *this, int64_t vpts) {
-  int i = this->vpts_offset_read;
+  int i = this->vpts_offs_queue.read;
   while (1) {
     int j = (i + 1) & OFFS_RING_MASK;
-    if (j == this->vpts_offset_write)
+    if (j == this->vpts_offs_queue.write)
       break;
-    if (this->vpts_offsets[j].vpts > vpts)
+    if (this->vpts_offs_queue.items[j].vpts > vpts)
       break;
     i = j;
   }
-  this->vpts_offset_read = i;
+  this->vpts_offs_queue.read = i;
 }
 
 static void vdr_vpts_offset_queue_add_int (vdr_input_plugin_t *this, int64_t pts) {
   int64_t offset = this->metronom.stream_metronom->get_option (this->metronom.stream_metronom, METRONOM_VPTS_OFFSET);
   int64_t vpts = pts + offset;
-  this->vpts_offsets[this->vpts_offset_write].offset = offset;
-  this->vpts_offsets[this->vpts_offset_write].vpts = vpts;
-  this->vpts_offset_write = (this->vpts_offset_write + 1) & OFFS_RING_MASK;
+  this->vpts_offs_queue.items[this->vpts_offs_queue.write].offset = offset;
+  this->vpts_offs_queue.items[this->vpts_offs_queue.write].vpts = vpts;
+  this->vpts_offs_queue.write = (this->vpts_offs_queue.write + 1) & OFFS_RING_MASK;
   /* queue full, make some room */
-  if (this->vpts_offset_write == this->vpts_offset_read)
+  if (this->vpts_offs_queue.write == this->vpts_offs_queue.read)
     vdr_vpts_offset_queue_process (this, xine_get_current_vpts (this->stream));
 }
 
 static int vdr_vpts_offset_queue_ask (vdr_input_plugin_t *this, int64_t *pts) {
   int64_t vpts = xine_get_current_vpts (this->stream);
   vdr_vpts_offset_queue_process (this, vpts);
-  *pts = vpts - this->vpts_offsets[this->vpts_offset_read].offset;
-  return ((this->vpts_offset_write - this->vpts_offset_read) & OFFS_RING_MASK) > 1;
+  *pts = vpts - this->vpts_offs_queue.items[this->vpts_offs_queue.read].offset;
+  return ((this->vpts_offs_queue.write - this->vpts_offs_queue.read) & OFFS_RING_MASK) > 1;
 }
 
 static void vdr_vpts_offset_queue_purge (vdr_input_plugin_t *this) {
-  this->vpts_offset_read = (this->vpts_offset_write - 1) & OFFS_RING_MASK;
+  this->vpts_offs_queue.read = (this->vpts_offs_queue.write - 1) & OFFS_RING_MASK;
 }
 
 
@@ -423,12 +448,11 @@ static void vdr_start_buffers (vdr_input_plugin_t *this) {
 }
 
 
-static off_t vdr_execute_rpc_command(vdr_input_plugin_t *this)
-{
+static ssize_t vdr_execute_rpc_command (vdr_input_plugin_t *this) {
   data_union_t data_union;
-  off_t n;
+  ssize_t n;
 
-  n = vdr_read_abort(this->stream, this->fh_control, (char *)&data_union, sizeof (data_union.header));
+  n = vdr_read_abort (this->stream, this->fh_control, (uint8_t *)&data_union, sizeof (data_union.header));
   if (n != sizeof (data_union.header))
     return -1;
 
@@ -611,7 +635,7 @@ break;
         this->osd_buffer_size = this->cur_size;
       }
 
-      n = vdr_read_abort (this->stream, this->fh_control, (char *)this->osd_buffer, this->cur_size);
+      n = vdr_read_abort (this->stream, this->fh_control, this->osd_buffer, this->cur_size);
       if (n != this->cur_size)
         return -1;
 
@@ -678,7 +702,7 @@ break;
       if (((data->num + 1) * sizeof (uint32_t)) != (unsigned int)this->cur_size)
         return -1;
 
-      n = vdr_read_abort (this->stream, this->fh_control, (char *)&vdr_color[ data->index ], this->cur_size);
+      n = vdr_read_abort (this->stream, this->fh_control, (uint8_t *)&vdr_color[ data->index ], this->cur_size);
       if (n != this->cur_size)
         return -1;
 
@@ -740,7 +764,7 @@ break;
           return -1;
         }
 
-        n = vdr_read_abort (this->stream, this->fh_control, file_name, file_name_len);
+        n = vdr_read_abort (this->stream, this->fh_control, (uint8_t *)file_name, file_name_len);
         if (n != file_name_len)
           return -1;
 
@@ -801,12 +825,12 @@ break;
 
           /* XXX: why is this needed? */
           pthread_mutex_lock (&this->metronom.mutex);
-          this->metronom.audio_seek = 1;
-          this->metronom.video_seek = 1;
+          this->metronom.audio.seek = 1;
+          this->metronom.video.seek = 1;
           pthread_mutex_unlock (&this->metronom.mutex);
-          pthread_mutex_lock (&this->vpts_offset_queue_lock);
+          pthread_mutex_lock (&this->vpts_offs_queue.lock);
           this->last_disc_type = DISC_STREAMSTART;
-          pthread_mutex_unlock (&this->vpts_offset_queue_lock);
+          pthread_mutex_unlock (&this->vpts_offs_queue.lock);
 
           _x_stream_info_reset(this->stream, XINE_STREAM_INFO_AUDIO_BITRATE);
 /* fprintf(stderr, "=== CLEAR(%d.4)\n", data->n); */
@@ -821,8 +845,12 @@ break;
 */
         }
 /* fprintf(stderr, "--- CLEAR(%d%c)\n", data->n, data->s ? 'b' : 'a'); */
-        if (orig_speed <= XINE_FINE_SPEED_NORMAL / 3)
+        if (orig_speed <= XINE_FINE_SPEED_NORMAL / 3) {
           xine_set_param (this->stream, XINE_PARAM_FINE_SPEED, orig_speed);
+          if (orig_speed == 0)
+            /* make sure decoders are responsive. */
+            xine_set_param (this->stream, XINE_PARAM_FINE_SPEED, XINE_LIVE_PAUSE_ON);
+        }
       }
     }
     break;
@@ -905,28 +933,15 @@ break;
       xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
         "input_vdr: trick_speed_mode (%d)\n", (int)data->on);
 
-      if (this->trick_speed_mode != data->on)
-      {
-        pthread_mutex_lock(&this->trick_speed_mode_lock);
-
-        while (this->trick_speed_mode_blocked)
-          pthread_cond_wait(&this->trick_speed_mode_cond, &this->trick_speed_mode_lock);
-
-        this->trick_speed_mode = data->on;
-
-        pthread_mutex_unlock(&this->trick_speed_mode_lock);
-
-        _x_demux_seek(this->stream, 0, 0, 0);
-
-        {
-          xine_event_t event;
-
-          event.type = XINE_EVENT_VDR_TRICKSPEEDMODE;
-          event.data = NULL;
-          event.data_length = 0; /* this->trick_speed_mode; */
-/*          fprintf(stderr, "************************: %p, %d\n", event.data, event.data_length); */
-          xine_event_send(this->stream, &event);
-        }
+      pthread_mutex_lock (&this->metronom.mutex);
+      if (this->metronom.audio.disc_num != this->metronom.video.disc_num) {
+        this->metronom.trick_new_mode = data->on;
+        pthread_mutex_unlock (&this->metronom.mutex);
+      } else {
+        this->metronom.trick_mode = data->on;
+        this->metronom.trick_new_mode = -1;
+        pthread_mutex_unlock (&this->metronom.mutex);
+        trick_speed_send_event (this, this->metronom.trick_mode);
       }
     }
     break;
@@ -1116,8 +1131,12 @@ break;
 
       lprintf("... got SETSPEED %d\n", data->speed);
 
-      if (data->speed != xine_get_param(this->stream, XINE_PARAM_FINE_SPEED))
-        xine_set_param(this->stream, XINE_PARAM_FINE_SPEED, data->speed);
+      if (data->speed != xine_get_param (this->stream, XINE_PARAM_FINE_SPEED)) {
+        xine_set_param (this->stream, XINE_PARAM_FINE_SPEED, data->speed);
+        if (data->speed == 0)
+          /* make sure decoders are responsive. */
+          xine_set_param (this->stream, XINE_PARAM_FINE_SPEED, XINE_LIVE_PAUSE_ON);
+      }
     }
     break;
 
@@ -1249,7 +1268,7 @@ break;
         result_get_pts.header.func = data->header.func;
         result_get_pts.header.len = sizeof (result_get_pts);
 
-        pthread_mutex_lock(&this->vpts_offset_queue_lock);
+        pthread_mutex_lock(&this->vpts_offs_queue.lock);
 
         if (this->last_disc_type == DISC_STREAMSTART
           && data->ms_timeout > 0)
@@ -1272,7 +1291,7 @@ break;
           }
 
           while (this->last_disc_type == DISC_STREAMSTART) {
-            if (0 != pthread_cond_timedwait (&this->vpts_offset_queue_changed_cond, &this->vpts_offset_queue_lock, &abstime))
+            if (0 != pthread_cond_timedwait (&this->vpts_offs_queue.changed, &this->vpts_offs_queue.lock, &abstime))
               break;
           }
         }
@@ -1289,7 +1308,7 @@ break;
 /* fprintf(stderr, "vpts: %12ld, stc: %12ld, offset: %12ld\n", vpts, result_get_pts.pts, offset); */
         }
 
-        pthread_mutex_unlock(&this->vpts_offset_queue_lock);
+        pthread_mutex_unlock(&this->vpts_offs_queue.lock);
 
         if (sizeof (result_get_pts) != vdr_write(this->fh_result, &result_get_pts, sizeof (result_get_pts)))
           return -1;
@@ -1319,7 +1338,7 @@ break;
       READ_DATA_OR_FAIL(video_size, lprintf("got VIDEO SIZE\n"));
 
       {
-        int format;
+        int format, width, height, ratio;
 
         result_video_size_t result_video_size;
         result_video_size.header.func = data->header.func;
@@ -1331,15 +1350,18 @@ break;
         result_video_size.height = -1;
         result_video_size.ratio  = 0;
 
-        xine_get_current_frame_s(this->stream, &result_video_size.width, &result_video_size.height, &result_video_size.ratio, &format, NULL, NULL);
+        xine_get_current_frame_s(this->stream, &width, &height, &ratio, &format, NULL, NULL);
+        result_video_size.width = width;
+        result_video_size.height = height;
+        result_video_size.ratio = ratio;
 
-        if (result_video_size.ratio == XINE_VO_ASPECT_SQUARE)
+        if (ratio == XINE_VO_ASPECT_SQUARE)
           result_video_size.ratio = 10000;
-        else if (result_video_size.ratio == XINE_VO_ASPECT_4_3)
+        else if (ratio == XINE_VO_ASPECT_4_3)
           result_video_size.ratio = 13333;
-        else if (result_video_size.ratio == XINE_VO_ASPECT_ANAMORPHIC)
+        else if (ratio == XINE_VO_ASPECT_ANAMORPHIC)
           result_video_size.ratio = 17778;
-        else if (result_video_size.ratio == XINE_VO_ASPECT_DVB)
+        else if (ratio == XINE_VO_ASPECT_DVB)
           result_video_size.ratio = 21100;
 
         if (0 != this->frame_size.x
@@ -1881,9 +1903,6 @@ static void vdr_plugin_dispose(input_plugin_t *this_gen)
   pthread_cond_destroy(&this->rpc_thread_shutdown_cond);
   pthread_mutex_destroy(&this->rpc_thread_shutdown_lock);
 
-  pthread_mutex_destroy(&this->trick_speed_mode_lock);
-  pthread_cond_destroy(&this->trick_speed_mode_cond);
-
   pthread_mutex_destroy(&this->find_sync_point_lock);
   pthread_mutex_destroy(&this->adjust_zoom_lock);
 
@@ -1918,12 +1937,12 @@ static void vdr_plugin_dispose(input_plugin_t *this_gen)
 
   free(this->mrl);
 
+  /* unset metronom */
   this->stream->metronom = this->metronom.stream_metronom;
   this->metronom.stream_metronom = NULL;
 
-  vdr_vpts_offset_queue_purge(this);
-  pthread_cond_destroy(&this->vpts_offset_queue_changed_cond);
-  pthread_mutex_destroy(&this->vpts_offset_queue_lock);
+  vdr_vpts_offset_queue_purge (this);
+  vdr_vpts_offset_queue_deinit (this);
 
   pthread_mutex_destroy (&this->metronom.mutex);
 
@@ -2184,17 +2203,17 @@ static int vdr_plugin_open_socket_mrl(input_plugin_t *this_gen)
 }
 
 static void vdr_vpts_offset_queue_add (vdr_input_plugin_t *this, int type, int64_t disc_off) {
-  pthread_mutex_lock (&this->vpts_offset_queue_lock);
+  pthread_mutex_lock (&this->vpts_offs_queue.lock);
   if ((type == DISC_ABSOLUTE) || (type == DISC_STREAMSTART))
     vdr_vpts_offset_queue_add_int (this, disc_off);
   else
     vdr_vpts_offset_queue_purge(this);
   this->last_disc_type = type;
   if (type != DISC_STREAMSTART)
-    pthread_cond_broadcast (&this->vpts_offset_queue_changed_cond);
-  pthread_mutex_unlock (&this->vpts_offset_queue_lock);
+    pthread_cond_broadcast (&this->vpts_offs_queue.changed);
+  pthread_mutex_unlock (&this->vpts_offs_queue.lock);
 
-  if (!this->trick_speed_mode)
+  if (!this->metronom.trick_mode)
   {
     xine_event_t event;
 
@@ -2400,139 +2419,183 @@ static void event_handler(void *user_data, const xine_event_t *event)
 
 
 static void vdr_metronom_handle_audio_discontinuity (metronom_t *self, int type, int64_t disc_off) {
-  vdr_metronom_t *this = (vdr_metronom_t *)self;
-  int paired, relay_type;
+  vdr_metronom_t *metr = (vdr_metronom_t *)self;
+  int diff, num, trick_mode, trick_new_mode, add, relay_type = type;
 
-  /* TJ. vdr does not tell its exact trick speed. for now, defer discontinuity handling to
-   * decoder output like before :-/ */
-  pthread_mutex_lock (&this->input->trick_speed_mode_lock);
-  if (this->input->trick_speed_mode) {
-    pthread_mutex_lock (&this->mutex);
-    this->disc_num_audio += 1;
-    paired = (this->disc_num_audio == this->disc_num_video);
-    pthread_mutex_unlock (&this->mutex);
-    this->input->trick_speed_mode_blocked = !paired;
-    if (!this->input->trick_speed_mode_blocked)
-      pthread_cond_broadcast (&this->input->trick_speed_mode_cond);
-    pthread_mutex_unlock (&this->input->trick_speed_mode_lock);
+  /* just 1 finite time lock session. */
+  pthread_mutex_lock (&metr->mutex);
+  /* just relay all we dont need (in particular, DISC_GAPLESS). */
+  if ((type != DISC_STREAMSTART) &&
+      (type != DISC_RELATIVE) &&
+      (type != DISC_ABSOLUTE) &&
+      (type != DISC_STREAMSEEK)) {
+    pthread_mutex_unlock (&metr->mutex);
+    metr->stream_metronom->handle_audio_discontinuity (metr->stream_metronom, type, disc_off);
+    return;
+  }
+  /* make sure we dont respond too early. */
+  if (!metr->audio.on) {
+    if ((type != DISC_STREAMSEEK) || (disc_off != VDR_DISC_START)) {
+      pthread_mutex_unlock (&metr->mutex);
+      metr->stream_metronom->handle_audio_discontinuity (metr->stream_metronom, type, disc_off);
+      return;
+    }
+    metr->audio.on = 1;
+    pthread_mutex_unlock (&metr->mutex);
+    xprintf (metr->input->stream->xine, XINE_VERBOSITY_DEBUG,
+      "input_vdr: audio discontinuity handling now on.\n");
     return;
   }
-  this->input->trick_speed_mode_blocked = 1;
-  pthread_mutex_unlock (&this->input->trick_speed_mode_lock);
-
-  pthread_mutex_lock (&this->mutex);
   /* Demux knows nothing about vdr seek. It just sees a pts jump, and sends a plain absolute
    * discontinuity. That will make metronom try a seamless append, leaving a 1..2s gap there.
    * Thus, after a seek, manually upgrade first "absolute" to "streamseek" here. */
-  relay_type = type;
   if (type == DISC_STREAMSTART) {
-    this->audio_seek = 1;
-  } else if ((type == DISC_ABSOLUTE) && this->audio_seek) {
-    this->audio_seek = 0;
+    metr->audio.seek = 1;
+  } else if ((type == DISC_ABSOLUTE) && metr->audio.seek) {
+    metr->audio.seek = 0;
     relay_type = DISC_STREAMSEEK;
   }
-  this->disc_num_audio += 1;
-  paired = (this->disc_num_audio == this->disc_num_video);
-  if (this->disc_num_audio > this->disc_num_video) {
-    /* we are ahaed: relay, and return. */
-    pthread_mutex_unlock (&this->mutex);
-    this->stream_metronom->handle_audio_discontinuity (this->stream_metronom, relay_type, disc_off);
-    return;
+
+  trick_mode = metr->trick_mode;
+  trick_new_mode = metr->trick_new_mode;
+  metr->audio.disc_num += 1;
+  num = metr->audio.disc_num;
+  add = diff = metr->audio.disc_num - metr->video.disc_num;
+
+  if (trick_mode && (type == DISC_ABSOLUTE) && (diff <= 0)) {
+    if (trick_mode == 1)
+      metr->trick_mode = 2;
+    else
+      add = 1;
   }
-  /* we are behind: complete this pair. */
-  pthread_mutex_unlock (&this->mutex);
-  this->stream_metronom->handle_audio_discontinuity (this->stream_metronom, relay_type, disc_off);
-  vdr_vpts_offset_queue_add (this->input, type, disc_off);
-  if (paired) {
-    pthread_mutex_lock (&this->input->trick_speed_mode_lock);
-    this->input->trick_speed_mode_blocked = 0;
-    pthread_cond_broadcast (&this->input->trick_speed_mode_cond);
-    pthread_mutex_unlock (&this->input->trick_speed_mode_lock);
+
+  if ((diff == 0) && (trick_new_mode >= 0)) {
+    metr->trick_mode = trick_new_mode;
+    metr->trick_new_mode = -1;
+  } else {
+    trick_new_mode = -1;
   }
+  pthread_mutex_unlock (&metr->mutex);
+
+  /* report */
+  xprintf (metr->input->stream->xine, XINE_VERBOSITY_DEBUG,
+    "input_vdr: %s audio discontinuity #%d, type is %d, disc off %" PRId64 ".\n",
+    trick_mode ? "trick play" : "", num, type, disc_off);
+  /* relay */
+  if (!trick_mode)
+    metr->stream_metronom->handle_audio_discontinuity (metr->stream_metronom, relay_type, disc_off);
+  /* if we are behind: complete this pair. */
+  if (add <= 0)
+    vdr_vpts_offset_queue_add (metr->input, type, disc_off);
+  /* new trick mode */
+  if (trick_new_mode >= 0)
+    trick_speed_send_event (metr->input, trick_new_mode);
 }
 
 static void vdr_metronom_handle_video_discontinuity (metronom_t *self, int type, int64_t disc_off) {
-  vdr_metronom_t *this = (vdr_metronom_t *)self;
-  int paired, relay_type;
+  vdr_metronom_t *metr = (vdr_metronom_t *)self;
+  int diff, num, trick_mode, trick_new_mode, add, relay_type = type;
 
-  /* TJ. vdr does not tell its exact trick speed. for now, defer discontinuity handling to
-   * decoder output like before :-/ */
-  pthread_mutex_lock (&this->input->trick_speed_mode_lock);
-  if (this->input->trick_speed_mode) {
-    pthread_mutex_lock (&this->mutex);
-    this->disc_num_video += 1;
-    paired = (this->disc_num_audio == this->disc_num_video);
-    pthread_mutex_unlock (&this->mutex);
-    this->input->trick_speed_mode_blocked = !paired;
-    if (!this->input->trick_speed_mode_blocked)
-      pthread_cond_broadcast (&this->input->trick_speed_mode_cond);
-    pthread_mutex_unlock (&this->input->trick_speed_mode_lock);
+  /* just 1 finite time lock session. */
+  pthread_mutex_lock (&metr->mutex);
+  /* just relay all we dont need (in particular, DISC_GAPLESS). */
+  if ((type != DISC_STREAMSTART) &&
+      (type != DISC_RELATIVE) &&
+      (type != DISC_ABSOLUTE) &&
+      (type != DISC_STREAMSEEK)) {
+    pthread_mutex_unlock (&metr->mutex);
+    metr->stream_metronom->handle_video_discontinuity (metr->stream_metronom, type, disc_off);
+    return;
+  }
+  /* make sure we dont respond too early. */
+  if (!metr->video.on) {
+    if ((type != DISC_STREAMSEEK) || (disc_off != VDR_DISC_START)) {
+      pthread_mutex_unlock (&metr->mutex);
+      metr->stream_metronom->handle_video_discontinuity (metr->stream_metronom, type, disc_off);
+      return;
+    }
+    metr->video.on = 1;
+    pthread_mutex_unlock (&metr->mutex);
+    xprintf (metr->input->stream->xine, XINE_VERBOSITY_DEBUG,
+      "input_vdr: video discontinuity handling now on.\n");
     return;
   }
-  this->input->trick_speed_mode_blocked = 1;
-  pthread_mutex_unlock (&this->input->trick_speed_mode_lock);
-
-  pthread_mutex_lock (&this->mutex);
   /* Demux knows nothing about vdr seek. It just sees a pts jump, and sends a plain absolute
    * discontinuity. That will make metronom try a seamless append, leaving a 1..2s gap there.
    * Thus, after a seek, manually upgrade first "absolute" to "streamseek" here. */
-  relay_type = type;
   if (type == DISC_STREAMSTART) {
-    this->video_seek = 1;
-  } else if ((type == DISC_ABSOLUTE) && this->video_seek) {
-    this->video_seek = 0;
+    metr->video.seek = 1;
+  } else if ((type == DISC_ABSOLUTE) && metr->video.seek) {
+    metr->video.seek = 0;
     relay_type = DISC_STREAMSEEK;
   }
-  this->disc_num_video += 1;
-  paired = (this->disc_num_audio == this->disc_num_video);
-  if (this->disc_num_video > this->disc_num_audio) {
-    /* we are ahaed: relay, and return. */
-    pthread_mutex_unlock (&this->mutex);
-    this->stream_metronom->handle_video_discontinuity (this->stream_metronom, relay_type, disc_off);
-    return;
+
+  trick_mode = metr->trick_mode;
+  trick_new_mode = metr->trick_new_mode;
+  metr->video.disc_num += 1;
+  num = metr->video.disc_num;
+  add = diff = metr->video.disc_num - metr->audio.disc_num;
+
+  if (trick_mode && (type == DISC_ABSOLUTE) && (diff <= 0)) {
+    if (trick_mode == 1)
+      metr->trick_mode = 2;
+    else
+      add = 1;
   }
-  /* we are behind: complete this pair. */
-  pthread_mutex_unlock (&this->mutex);
-  this->stream_metronom->handle_video_discontinuity (this->stream_metronom, relay_type, disc_off);
-  vdr_vpts_offset_queue_add (this->input, type, disc_off);
-  if (paired) {
-    pthread_mutex_lock (&this->input->trick_speed_mode_lock);
-    this->input->trick_speed_mode_blocked = 0;
-    pthread_cond_broadcast (&this->input->trick_speed_mode_cond);
-    pthread_mutex_unlock (&this->input->trick_speed_mode_lock);
+
+  if ((diff == 0) && (trick_new_mode >= 0)) {
+    metr->trick_mode = trick_new_mode;
+    metr->trick_new_mode = -1;
+  } else {
+    trick_new_mode = -1;
   }
+  pthread_mutex_unlock (&metr->mutex);
+
+  /* report */
+  xprintf (metr->input->stream->xine, XINE_VERBOSITY_DEBUG,
+    "input_vdr: %s video discontinuity #%d, type is %d, disc off %" PRId64 ".\n",
+    trick_mode ? "trick play" : "", num, type, disc_off);
+  /* relay */
+  if (!trick_mode)
+    metr->stream_metronom->handle_video_discontinuity (metr->stream_metronom, relay_type, disc_off);
+  /* if we are behind: complete this pair. */
+  if (add <= 0)
+    vdr_vpts_offset_queue_add (metr->input, type, disc_off);
+  /* new trick mode */
+  if (trick_new_mode >= 0)
+    trick_speed_send_event (metr->input, trick_new_mode);
 }
 
 static void vdr_metronom_got_video_frame(metronom_t *self, vo_frame_t *frame)
 {
-  vdr_metronom_t *this = (vdr_metronom_t *)self;
+  vdr_metronom_t *metr = (vdr_metronom_t *)self;
 
   if (frame->pts) {
 
-    pthread_mutex_lock (&this->input->trick_speed_mode_lock);
-    if (!this->input->trick_speed_mode) {
+    pthread_mutex_lock (&metr->mutex);
+    if (!metr->trick_mode) {
 
-      pthread_mutex_unlock (&this->input->trick_speed_mode_lock);
-      this->stream_metronom->got_video_frame (this->stream_metronom, frame);
+      pthread_mutex_unlock (&metr->mutex);
+      metr->stream_metronom->got_video_frame (metr->stream_metronom, frame);
 
     } else {
 
       frame->progressive_frame = -1; /* force progressive */
 
-      this->stream_metronom->set_option (this->stream_metronom, METRONOM_VDR_TRICK_PTS, frame->pts);
+      metr->stream_metronom->set_option (metr->stream_metronom, METRONOM_VDR_TRICK_PTS, frame->pts);
 
-      this->stream_metronom->got_video_frame (this->stream_metronom, frame);
-      vdr_vpts_offset_queue_add (this->input, DISC_ABSOLUTE, frame->pts);
+      metr->stream_metronom->got_video_frame (metr->stream_metronom, frame);
+      vdr_vpts_offset_queue_add (metr->input, DISC_ABSOLUTE, frame->pts);
 
-      pthread_mutex_unlock (&this->input->trick_speed_mode_lock);
+      pthread_mutex_unlock (&metr->mutex);
 
       /* fprintf (stderr, "vpts: %12ld, pts: %12ld, offset: %12ld\n",
-        frame->vpts, frame->pts, this->stream_metronom->get_option(this->stream_metronom, METRONOM_VPTS_OFFSET)); */
+        frame->vpts, frame->pts, metr->stream_metronom->get_option (metr->stream_metronom, METRONOM_VPTS_OFFSET)); */
     }
 
   } else {
 
-    this->stream_metronom->got_video_frame (this->stream_metronom, frame);
+    metr->stream_metronom->got_video_frame (metr->stream_metronom, frame);
 
   }
 }
@@ -2576,7 +2639,8 @@ static void vdr_metronom_set_master(metronom_t *self, metronom_t *master)
 static void vdr_metronom_exit(metronom_t *self)
 {
   (void)self;
-  _x_abort();
+  int this_shall_never_be_called = 1;
+  _x_assert (this_shall_never_be_called == 0);
 }
 
 
@@ -2615,7 +2679,6 @@ static input_plugin_t *vdr_class_get_instance(input_class_t *cls_gen, xine_strea
   this->osd_buffer               = NULL;
   this->osd_buffer_size          = 0;
   this->osd_unscaled_blending    = 0;
-  this->trick_speed_mode         = 0;
   this->audio_channels           = 0;
   this->frame_size.x             = 0;
   this->frame_size.y             = 0;
@@ -2628,21 +2691,25 @@ static input_plugin_t *vdr_class_get_instance(input_class_t *cls_gen, xine_strea
   this->image4_3_zoom_y          = 0;
   this->image16_9_zoom_x         = 0;
   this->image16_9_zoom_y         = 0;
-  this->metronom.audio_seek      = 0;
-  this->metronom.video_seek      = 0;
-  this->metronom.disc_num_audio  = 0;
-  this->metronom.disc_num_video  = 0;
-  this->vpts_offset_read         = 0;
-  this->vpts_offset_write        = 0;
+  this->metronom.audio.on        = 0;
+  this->metronom.audio.seek      = 0;
+  this->metronom.audio.disc_num  = 0;
+  this->metronom.video.on        = 0;
+  this->metronom.video.seek      = 0;
+  this->metronom.video.disc_num  = 0;
+  this->metronom.trick_mode      = 0;
 #endif
 
   this->stream     = stream;
+
   this->mrl        = mrl;
   this->fh         = -1;
   this->fh_control = -1;
   this->fh_result  = -1;
   this->fh_event   = -1;
 
+  this->metronom.trick_new_mode  = -1;
+
   this->input_plugin.open              = vdr_plugin_open;
   this->input_plugin.get_capabilities  = vdr_plugin_get_capabilities;
   this->input_plugin.read              = vdr_plugin_read;
@@ -2676,16 +2743,16 @@ static input_plugin_t *vdr_class_get_instance(input_class_t *cls_gen, xine_strea
   pthread_mutex_init (&this->rpc_thread_shutdown_lock, NULL);
   pthread_cond_init (&this->rpc_thread_shutdown_cond, NULL);
 
-  pthread_mutex_init (&this->trick_speed_mode_lock, NULL);
-  pthread_cond_init (&this->trick_speed_mode_cond, NULL);
-
   pthread_mutex_init (&this->find_sync_point_lock, NULL);
   pthread_mutex_init (&this->adjust_zoom_lock, NULL);
 
+  vdr_vpts_offset_queue_init (this);
+
   this->event_queue = xine_event_new_queue(this->stream);
   if (this->event_queue)
     xine_event_create_listener_thread(this->event_queue, event_handler, this);
 
+  /* init metronom */
   this->metronom.input = this;
   this->metronom.metronom.set_audio_rate             = vdr_metronom_set_audio_rate;
   this->metronom.metronom.got_video_frame            = vdr_metronom_got_video_frame;
@@ -2697,14 +2764,11 @@ static input_plugin_t *vdr_class_get_instance(input_class_t *cls_gen, xine_strea
   this->metronom.metronom.get_option                 = vdr_metronom_get_option;
   this->metronom.metronom.set_master                 = vdr_metronom_set_master;
   this->metronom.metronom.exit                       = vdr_metronom_exit;
-  this->metronom.stream_metronom                     = stream->metronom;
   pthread_mutex_init (&this->metronom.mutex, NULL);
-  stream->metronom                                   = &this->metronom.metronom;
-
-  pthread_mutex_init (&this->vpts_offset_queue_lock, NULL);
-  pthread_cond_init (&this->vpts_offset_queue_changed_cond, NULL);
-
-  vdr_vpts_offset_queue_init (this);
+  /* set metronom */
+  stream->metronom = &this->metronom.metronom;
+  /* send start discontinuities */
+  _x_demux_control_newpts (stream, VDR_DISC_START, BUF_FLAG_SEEK);
 
   return &this->input_plugin;
 }
