@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2017 the xine project
+ * Copyright (C) 2004-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -205,14 +205,9 @@
   IFF_REPLACE_BYTE(xindex_ptr, xyuv_y, xyuv_u, xyuv_v, yuv_palette_l, *xold_data, *xnew_data, colorindexx_l ); \
 }
 
-typedef struct {
-  video_decoder_class_t   decoder_class;
-} bitplane_class_t;
-
 typedef struct bitplane_decoder_s {
   video_decoder_t   video_decoder;  /* parent video decoder structure */
 
-  bitplane_class_t *class;
   xine_stream_t    *stream;
 
   /* these are traditional variables in a video decoder object    */
@@ -376,6 +371,8 @@ static void bitplane_decode_ham (uint8_t *ham_buffer,
   int maskbits                          = 8 - hambits;
   int mask                              = ( 1 << hambits ) - 1;
 
+  (void)bytes_per_pixel;
+
   for(; ham_buffer_work < ham_buffer_end; j = *ham_buffer_work++) {
     i                                   = (j & mask);
     switch ( j >> hambits ) {
@@ -430,7 +427,7 @@ static void bitplane_sdelta_opt_3 (bitplane_decoder_t *this) {
   uint32_t yuv_index                    = 0;
 
   /* Repeat for each plane */
-  for(palette_index = 0; palette_index < this->num_bitplanes; palette_index++) {
+  for (palette_index = 0; (int32_t)palette_index < this->num_bitplanes; palette_index++) {
 
     planeptr                            = (uint16_t *)(&this->buf_uk[(palette_index * rowsize * 2)]);
     /* data starts at beginn of delta-Buffer + offset of the first */
@@ -550,7 +547,7 @@ static void bitplane_set_dlta_short (bitplane_decoder_t *this) {
   uint32_t yuv_index                    = 0;
 
   /* Repeat for each plane */
-  for(palette_index = 0; palette_index < this->num_bitplanes; palette_index++) {
+  for (palette_index = 0; (int32_t)palette_index < this->num_bitplanes; palette_index++) {
 
     planeptr                            = (uint16_t *)(&this->buf_uk[(palette_index * rowsize * 2)]);
     /* data starts at beginn of delta-Buffer + offset of the first */
@@ -635,7 +632,7 @@ static void bitplane_dlta_5 (bitplane_decoder_t *this) {
   uint8_t  count                        = 0;
 
   /* Repeat for each plane */
-  for(palette_index = 0; palette_index < this->num_bitplanes; palette_index++) {
+  for (palette_index = 0; (int32_t)palette_index < this->num_bitplanes; palette_index++) {
 
     planeptr                            = &this->buf_uk[(palette_index * rowsize)];
     /* data starts at beginn of delta-Buffer + offset of the first */
@@ -730,7 +727,7 @@ static void bitplane_dlta_7_short (bitplane_decoder_t *this) {
   uint8_t  count                        = 0;
 
   /* Repeat for each plane */
-  for(palette_index = 0; palette_index < this->num_bitplanes; palette_index++) {
+  for (palette_index = 0; (int32_t)palette_index < this->num_bitplanes; palette_index++) {
 
     planeptr                            = &this->buf_uk[(palette_index * rowsize * 2)];
     /* find opcode and data offset (up to 8 pointers, one for every bitplane */
@@ -825,7 +822,7 @@ static void bitplane_dlta_7_long  (bitplane_decoder_t *this) {
   uint8_t  count                        = 0;
 
   /* Repeat for each plane */
-  for(palette_index = 0; palette_index < this->num_bitplanes; palette_index++) {
+  for (palette_index = 0; (int32_t)palette_index < this->num_bitplanes; palette_index++) {
     planeptr                            = &this->buf_uk[(palette_index * rowsize * 4)];
     /* find opcode and data offset (up to 8 pointers, one for every bitplane */
     opcode_offset                       = _X_BE_32(&deltadata[palette_index]);
@@ -917,7 +914,7 @@ static void bitplane_dlta_8_short (bitplane_decoder_t *this) {
   uint16_t count                        = 0;
 
   /* Repeat for each plane */
-  for(palette_index = 0; palette_index < this->num_bitplanes; palette_index++) {
+  for (palette_index = 0; (int32_t)palette_index < this->num_bitplanes; palette_index++) {
 
     planeptr                            = (uint16_t *)(&this->buf_uk[(palette_index * rowsize * 2)]);
     /* data starts at beginn of delta-Buffer + offset of the first */
@@ -1014,7 +1011,7 @@ static void bitplane_dlta_8_long (bitplane_decoder_t *this) {
   uint32_t count                        = 0;
 
   /* Repeat for each plane */
-  for(palette_index = 0; palette_index < this->num_bitplanes; palette_index++) {
+  for (palette_index = 0; (int32_t)palette_index < this->num_bitplanes; palette_index++) {
 
     planeptr                            = (uint32_t *)(&this->buf_uk[(palette_index * rowsize * 4)]);
     /* data starts at beginn of delta-Buffer + offset of the first */
@@ -1111,18 +1108,19 @@ static void bitplane_decode_data (video_decoder_t *this_gen,
 
   if ((buf->decoder_flags & BUF_FLAG_SPECIAL) &&
       (buf->decoder_info[1] == BUF_SPECIAL_PALETTE)) {
+    unsigned int u;
     palette                             = (palette_entry_t *)buf->decoder_info_ptr[2];
 
-    for (i = 0; i < buf->decoder_info[2]; i++) {
-      this->yuv_palette[i * 4 + 0]      =
-        COMPUTE_Y(palette[i].r, palette[i].g, palette[i].b);
-      this->yuv_palette[i * 4 + 1]      =
-        COMPUTE_U(palette[i].r, palette[i].g, palette[i].b);
-      this->yuv_palette[i * 4 + 2]      =
-        COMPUTE_V(palette[i].r, palette[i].g, palette[i].b);
-      this->rgb_palette[i * 4 + 0]      = palette[i].r;
-      this->rgb_palette[i * 4 + 1]      = palette[i].g;
-      this->rgb_palette[i * 4 + 2]      = palette[i].b;
+    for (u = 0; u < buf->decoder_info[2]; u++) {
+      this->yuv_palette[u * 4 + 0]      =
+        COMPUTE_Y(palette[u].r, palette[u].g, palette[u].b);
+      this->yuv_palette[u * 4 + 1]      =
+        COMPUTE_U(palette[u].r, palette[u].g, palette[u].b);
+      this->yuv_palette[u * 4 + 2]      =
+        COMPUTE_V(palette[u].r, palette[u].g, palette[u].b);
+      this->rgb_palette[u * 4 + 0]      = palette[u].r;
+      this->rgb_palette[u * 4 + 1]      = palette[u].g;
+      this->rgb_palette[u * 4 + 2]      = palette[u].b;
     }
 
     /* EHB Pictures not allways contain all 64 colors, sometimes only    */
@@ -1237,7 +1235,13 @@ static void bitplane_decode_data (video_decoder_t *this_gen,
       img = this->stream->video_out->get_frame (this->stream->video_out,
                                         this->width, this->height,
                                         this->ratio, XINE_IMGFMT_YUY2,
-                                        VO_BOTH_FIELDS);
+                                        VO_BOTH_FIELDS | VO_GET_FRAME_MAY_FAIL);
+      if (!img) {
+        xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+                LOG_MODULE ": get_frame(%dx%d) failed\n", this->width, this->height);
+        this->size = 0;
+        return;
+      }
 
       img->duration                     = this->video_step;
       img->pts                          = buf->pts;
@@ -1457,6 +1461,7 @@ static void bitplane_decode_data (video_decoder_t *this_gen,
  * sure when or if this is used or even if it needs to do anything.
  */
 static void bitplane_flush (video_decoder_t *this_gen) {
+  (void)this_gen;
 }
 
 /*
@@ -1469,6 +1474,7 @@ static void bitplane_reset (video_decoder_t *this_gen) {
 }
 
 static void bitplane_discontinuity (video_decoder_t *this_gen) {
+  (void)this_gen;
 }
 
 /*
@@ -1493,7 +1499,13 @@ static void bitplane_dispose (video_decoder_t *this_gen) {
 
 static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stream_t *stream) {
 
-  bitplane_decoder_t  *this             = (bitplane_decoder_t *) calloc(1, sizeof(bitplane_decoder_t));
+  bitplane_decoder_t *this;
+
+  (void)class_gen;
+
+  this = (bitplane_decoder_t *) calloc(1, sizeof(bitplane_decoder_t));
+  if (!this)
+    return NULL;
 
   this->video_decoder.decode_data       = bitplane_decode_data;
   this->video_decoder.flush             = bitplane_flush;
@@ -1503,7 +1515,6 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
   this->size                            = 0;
 
   this->stream                          = stream;
-  this->class                           = (bitplane_class_t *) class_gen;
 
   this->decoder_ok                      = 0;
   this->buf                             = NULL;
@@ -1514,15 +1525,17 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
   return &this->video_decoder;
 }
 
-void *decode_bitplane_init_class (xine_t *xine, void *data) {
+void *decode_bitplane_init_class (xine_t *xine, const void *data) {
 
-  bitplane_class_t *this                = (bitplane_class_t *) calloc(1, sizeof(bitplane_class_t));
+  (void)xine;
+  (void)data;
 
-  this->decoder_class.open_plugin       = open_plugin;
-  this->decoder_class.identifier        = "bitplane";
-  this->decoder_class.description       = N_("Raw bitplane video decoder plugin");
-  this->decoder_class.dispose           = default_video_decoder_class_dispose;
-
-  return this;
+  static const video_decoder_class_t decode_video_bitplane_class = {
+    .open_plugin       = open_plugin,
+    .identifier        = "bitplane",
+    .description       = N_("Raw bitplane video decoder plugin"),
+    .dispose           = NULL,
+  };
+  return (void *)&decode_video_bitplane_class;
 }
 
