@@ -1071,6 +1071,9 @@ static inline void slice_xvmc_intra_DCT (picture_t * picture, int cc,
 #define bit_buf (picture->bitstream_buf)
 #define bits (picture->bitstream_bits)  
 #define bit_ptr (picture->bitstream_ptr)
+    (void)dest;
+    (void)stride;
+
     NEEDBITS (bit_buf, bits, bit_ptr);
     /* Get the intra DC coefficient and inverse quantize it */
 
@@ -1108,6 +1111,9 @@ static inline void slice_xvmc_intra_DCT (picture_t * picture, int cc,
 static inline void slice_xvmc_non_intra_DCT (picture_t * picture, uint8_t * dest,
 					int stride)
 {
+    (void)dest;
+    (void)stride;
+
   mpeg2_zero_block(picture->mc->blockptr);
 
     if (picture->mpeg1)
@@ -1130,6 +1136,8 @@ static void motion_mp1 (picture_t * picture, motion_t * motion,
 #define bit_ptr (picture->bitstream_ptr)
     int motion_x, motion_y;
 
+    (void)table;
+
     NEEDBITS (bit_buf, bits, bit_ptr);
     motion_x = (motion->pmv[0][0] +
 		(get_xvmc_motion_delta (picture,
@@ -1159,6 +1167,8 @@ static void motion_fr_frame (picture_t * picture, motion_t * motion,
 #define bit_ptr (picture->bitstream_ptr)
     int motion_x, motion_y;
 
+    (void)table;
+
     NEEDBITS (bit_buf, bits, bit_ptr);
     motion_x = motion->pmv[0][0] + get_xvmc_motion_delta (picture,
 						     motion->f_code[0]);
@@ -1186,6 +1196,8 @@ static void motion_fr_field (picture_t * picture, motion_t * motion,
     int motion_x, motion_y, field;
     //    unsigned int pos_x, pos_y, xy_half;
 
+    (void)table;
+
     NEEDBITS (bit_buf, bits, bit_ptr);
     field = UBITS (bit_buf, 1);
     picture->XvMC_mv_field_sel[0][dir] = field;
@@ -1232,6 +1244,8 @@ static void motion_fr_dmv (picture_t * picture, motion_t * motion,
 #define bit_ptr (picture->bitstream_ptr)
     int motion_x, motion_y;
 
+    (void)table;
+
     // TODO field select ?? possible need to be 0
     picture->XvMC_mv_field_sel[0][0] = picture->XvMC_mv_field_sel[1][0] = 0;
 
@@ -1299,6 +1313,8 @@ static void motion_fi_field (picture_t * picture, motion_t * motion,
     int motion_x, motion_y;
     /*uint8_t ** ref_field;*/
 
+    (void)table;
+
     NEEDBITS (bit_buf, bits, bit_ptr);
     /*ref_field = motion->ref2[UBITS (bit_buf, 1)];*/
 
@@ -1332,6 +1348,8 @@ static void motion_fi_16x8 (picture_t * picture, motion_t * motion,
     int motion_x, motion_y;
     /*uint8_t ** ref_field;*/
 
+    (void)table;
+
     NEEDBITS (bit_buf, bits, bit_ptr);
     /*ref_field = motion->ref2[UBITS (bit_buf, 1)];*/
 
@@ -1384,6 +1402,8 @@ static void motion_fi_dmv (picture_t * picture, motion_t * motion,
 #define bit_ptr (picture->bitstream_ptr)
     int motion_x, motion_y;
 
+    (void)table;
+
     NEEDBITS (bit_buf, bits, bit_ptr);
     motion_x = motion->pmv[0][0] + get_xvmc_motion_delta (picture,
 						     motion->f_code[0]);
@@ -1561,7 +1581,7 @@ static inline int slice_xvmc_init (picture_t * picture, int code)
 	picture->dest[0] += picture->pitches[0];
 	picture->dest[1] += picture->pitches[1];
 	picture->dest[2] += picture->pitches[2];
-	/* follow thru */
+	/* fall through */
     case TOP_FIELD:
 	picture->pitches[0] <<= 1;
 	picture->pitches[1] <<= 1;
@@ -1874,7 +1894,7 @@ void mpeg2_xvmc_slice (mpeg2dec_accel_t *accel, picture_t * picture, int code, u
 	    } else switch (UBITS (bit_buf, 11)) {
 	    case 8:		/* macroblock_escape */
 		mba_inc += 33;
-		/* pass through */
+		/* fall through */
 	    case 15:	/* macroblock_stuffing (MPEG1 only) */
 		DUMPBITS (bit_buf, bits, 11);
 		NEEDBITS (bit_buf, bits, bit_ptr);
