@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2017 the xine project
+ * Copyright (C) 2013-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -30,20 +30,21 @@
 #include <vpx/vpx_decoder.h>
 #include <vpx/vp8dx.h>
 
+#define LOG_MODULE "libvpx_video_decoder"
+#define LOG_VERBOSE
+
+/*
+#define LOG
+*/
+
 #include <xine/xine_internal.h>
 #include <xine/video_out.h>
 #include <xine/buffer.h>
 #include <xine/xineutils.h>
 
-typedef struct {
-  video_decoder_class_t decoder_class;
-  uint32_t              buffer_type;
-} vpx_class_t;
-
 typedef struct vpx_decoder_s {
   video_decoder_t   video_decoder;  /* parent video decoder structure */
 
-  vpx_class_t      *class;
   xine_stream_t    *stream;
 
   int64_t              pts;
@@ -105,22 +106,25 @@ static void vpx_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 {
   vpx_decoder_t *this = (vpx_decoder_t *) this_gen;
 
-  if (buf->decoder_flags & BUF_FLAG_PREVIEW) {
-    return;
-  }
+  if ((!this->decoder_ok) |
+      (buf->decoder_flags & (BUF_FLAG_PREVIEW | BUF_FLAG_COLOR_MATRIX | BUF_FLAG_STDHEADER | BUF_FLAG_SPECIAL))) {
 
-  /* optional demux override (matroska/webm) */
-  if (buf->decoder_flags & BUF_FLAG_COLOR_MATRIX) {
-    VO_SET_FLAGS_CM (buf->decoder_info[4], this->frame_flags);
-  }
+    if (buf->decoder_flags & (BUF_FLAG_PREVIEW | BUF_FLAG_SPECIAL)) {
+      return;
+    }
 
-  if (buf->decoder_flags & BUF_FLAG_STDHEADER) {
-    vpx_handle_header(this, buf);
-    return;
-  }
+    /* optional demux override (matroska/webm) */
+    if (buf->decoder_flags & BUF_FLAG_COLOR_MATRIX) {
+      VO_SET_FLAGS_CM (buf->decoder_info[4], this->frame_flags);
+    }
 
-  if (!this->decoder_ok || buf->decoder_flags & BUF_FLAG_SPECIAL) {
-    return;
+    if (buf->decoder_flags & BUF_FLAG_STDHEADER) {
+      vpx_handle_header(this, buf);
+      return;
+    }
+
+    if (!this->decoder_ok)
+      return;
   }
 
   /* collect data */
@@ -156,8 +160,8 @@ static void vpx_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
   if (err != VPX_CODEC_OK) {
     const char *error  = vpx_codec_error(ctx);
     const char *detail = vpx_codec_error_detail(ctx);
-    xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-            LOG_MODULE": Failed to decode frame: %s (%s)\n",
+    xprintf(this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": "
+            "Failed to decode frame: %s (%s)\n",
             error, detail ? detail : "");
     free(p_pts);
     return;
@@ -175,14 +179,26 @@ static void vpx_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 
   if (vpx_img->fmt != VPX_IMG_FMT_I420) {
     xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-            LOG_MODULE": Unsupported colour space %d\n", vpx_img->fmt);
+            LOG_MODULE ": Unsupported colour space %d\n", vpx_img->fmt);
     return;
   }
 
   img = this->stream->video_out->get_frame (this->stream->video_out,
                                             this->width, this->height,
                                             this->ratio, XINE_IMGFMT_YV12,
-                                            this->frame_flags | VO_BOTH_FIELDS);
+                                            this->frame_flags | VO_BOTH_FIELDS | VO_GET_FRAME_MAY_FAIL);
+  if (!img) {
+    xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
+            LOG_MODULE ": get_frame(%dx%d) failed\n", this->width, this->height);
+    return;
+  }
+
+  /* crop if allocated frame is smaller than requested */
+  int width = this->width, height = this->height;
+  if (width > img->width)
+    width = img->width;
+  if (height > img->height)
+    height = img->height;
 
   yv12_to_yv12(
                /* Y */
@@ -195,7 +211,7 @@ static void vpx_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
                vpx_img->planes[2], vpx_img->stride[2],
                img->base[2], img->pitches[2],
                /* width x height */
-               this->width, this->height);
+               width, height);
 
   img->pts       = pts;
   img->bad_frame = 0;
@@ -207,6 +223,7 @@ static void vpx_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 
 static void vpx_flush (video_decoder_t *this_gen)
 {
+  (void)this_gen;
 }
 
 static void vpx_reset (video_decoder_t *this_gen)
@@ -229,6 +246,7 @@ static void vpx_reset (video_decoder_t *this_gen)
 
 static void vpx_discontinuity (video_decoder_t *this_gen)
 {
+  (void)this_gen;
 }
 
 /*
@@ -261,14 +279,18 @@ static void vpx_dispose (video_decoder_t *this_gen)
 
 static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stream_t *stream)
 {
-  vpx_class_t    *cls = (vpx_class_t *)class_gen;
   vpx_decoder_t  *this;
 
   const struct vpx_codec_iface *iface;
-  struct vpx_codec_dec_cfg deccfg = { 0 };
+  struct vpx_codec_dec_cfg deccfg = {
+    .threads = xine_cpu_count(),
+    .w = 0,
+    .h = 0
+  };
   int vp_version;
 
-  switch (cls->buffer_type) {
+  uint32_t video_type = BUF_VIDEO_BASE | (_x_get_video_streamtype(stream) << 16);
+  switch (video_type) {
     case BUF_VIDEO_VP8:
       iface = &vpx_codec_vp8_dx_algo;
       vp_version = 8;
@@ -283,8 +305,14 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
       return NULL;
   }
 
+  (void)class_gen;
+
+  xprintf(stream->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": "
+          "using %d CPU cores\n", deccfg.threads);
 
   this = (vpx_decoder_t *) calloc(1, sizeof(vpx_decoder_t));
+  if (!this)
+    return NULL;
 
   this->video_decoder.decode_data         = vpx_decode_data;
   this->video_decoder.flush               = vpx_flush;
@@ -295,7 +323,6 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
   this->size                              = 0;
 
   this->stream                            = stream;
-  this->class                             = (vpx_class_t *) class_gen;
 
   this->decoder_ok    = 0;
   this->buf           = NULL;
@@ -305,14 +332,15 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
   this->frame_flags   = 0;
   VO_SET_FLAGS_CM (4, this->frame_flags); /* undefined, mpeg range */
 
-  xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
-          LOG_MODULE "VP%d: using libvpx version %s\n",
+  xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": "
+          "VP%d: using libvpx version %s\n",
           vp_version, vpx_codec_version_str());
 
+  /* FIXME: ver should be VPX_DECODER_ABI_VERSION not 0. */
   if (vpx_codec_dec_init(&this->ctx, iface, &deccfg, 0) != VPX_CODEC_OK) {
     const char *err = vpx_codec_error(&this->ctx);
-    xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-            LOG_MODULE": Failed to initialize VP%d decoder: %s\n",
+    xprintf(this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": "
+            "Failed to initialize VP%d decoder: %s\n",
             vp_version, err);
     free(this);
     return NULL;
@@ -321,67 +349,40 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
   return &this->video_decoder;
 }
 
-static void *init_plugin (xine_t *xine, uint32_t buffer_type, const char *identifier)
+static void *init_plugin_vpx (xine_t *xine, const void *data)
 {
-  vpx_class_t *this;
+  (void)xine;
+  (void)data;
 
-  this = (vpx_class_t *) calloc(1, sizeof(vpx_class_t));
+  static const video_decoder_class_t decode_video_vpx_class = {
+    .open_plugin     = open_plugin,
+    .identifier      = "libvpx",
+    .description     = N_("WebM (VP8/VP9) video decoder plugin"),
+    .dispose         = NULL,
+  };
 
-  this->decoder_class.open_plugin     = open_plugin;
-  this->decoder_class.identifier      = identifier;
-  this->decoder_class.description     = N_("WebM (VP8/VP9) video decoder plugin");
-  this->decoder_class.dispose         = default_video_decoder_class_dispose;
-
-  this->buffer_type = buffer_type;
-
-  return this;
+  return (void *)&decode_video_vpx_class;
 }
 
-static void *init_plugin_vp8 (xine_t *xine, void *data)
-{
-  return init_plugin(xine, BUF_VIDEO_VP8, "libvpx-vp8");
-}
-
-#ifdef HAVE_VPX_VP9_DECODER
-static void *init_plugin_vp9 (xine_t *xine, void *data)
-{
-  return init_plugin(xine, BUF_VIDEO_VP9, "libvpx-vp9");
-}
-#endif
-
 /*
  * exported plugin catalog entry
  */
 
-static const uint32_t video_types_vp8[] = {
+static const uint32_t video_types_vpx[] = {
   BUF_VIDEO_VP8,
-  0
-};
-
 #ifdef HAVE_VPX_VP9_DECODER
-static const uint32_t video_types_vp9[] = {
   BUF_VIDEO_VP9,
+#endif
   0
 };
-#endif
 
-static const decoder_info_t dec_info_video_vp8 = {
-  video_types_vp8,     /* supported types */
-  1                    /* priority        */
+static const decoder_info_t dec_info_video_vpx = {
+  .supported_types = video_types_vpx,
+  .priority        = 1,
 };
 
-#ifdef HAVE_VPX_VP9_DECODER
-static const decoder_info_t dec_info_video_vp9 = {
-  video_types_vp9,     /* supported types */
-  1                    /* priority        */
-};
-#endif
-
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_VIDEO_DECODER, 19, "libvpx-vp8", XINE_VERSION_CODE, &dec_info_video_vp8, init_plugin_vp8 },
-#ifdef HAVE_VPX_VP9_DECODER
-  { PLUGIN_VIDEO_DECODER, 19, "libvpx-vp9", XINE_VERSION_CODE, &dec_info_video_vp9, init_plugin_vp9 },
-#endif
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_VIDEO_DECODER, 19, "libvpx", XINE_VERSION_CODE, &dec_info_video_vpx, init_plugin_vpx },
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
