@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2019 the xine project and Claudio Ciccani
+ * Copyright (C) 2000-2020 the xine project and Claudio Ciccani
  *
  * This file is part of xine, a free video player.
  *
@@ -208,6 +208,8 @@ static uint32_t directfb_get_capabilities (vo_driver_t *this_gen) {
 
 static void directfb_frame_field (vo_frame_t *vo_img, int which_field) {
   /* not needed */
+  (void)vo_img;
+  (void)which_field;
 }
 
 static void directfb_frame_dispose (vo_frame_t *vo_img) {
@@ -254,16 +256,18 @@ static void directfb_update_frame_format (vo_driver_t *this_gen,
   directfb_frame_t      *frame  = (directfb_frame_t *) frame_gen;
   DFBSurfacePixelFormat  format = (fmt == XINE_IMGFMT_YUY2) ? DSPF_YUY2 : DSPF_YV12;
 
+  (void)flags;
+
   if (frame->surface == NULL   ||
-      frame->width   != width  ||
-      frame->height  != height ||
+      frame->width   != (int)width  ||
+      frame->height  != (int)height ||
       frame->format  != format)
   {
     DFBSurfaceDescription dsc;
     DFBResult             ret;
 
     lprintf ("frame %p: format changed to %dx%d %s.\n",
-             frame, width, height, (format == DSPF_YUY2) ? "YUY2" : "YV12");
+             (void*)frame, width, height, (format == DSPF_YUY2) ? "YUY2" : "YV12");
 
     if (frame->surface) {
       if (frame->locked)
@@ -581,6 +585,8 @@ static void directfb_overlay_blend (vo_driver_t *this_gen,
 static void directfb_overlay_end (vo_driver_t *this_gen, vo_frame_t *frame_gen) {
   directfb_driver_t *this = (directfb_driver_t *) this_gen;
 
+  (void)frame_gen;
+
   if (this->ovl_changed) {
 #ifdef DIRECTFB_X11
     if (this->xoverlay) {
@@ -1751,11 +1757,14 @@ static void directfb_frame_output_cb (void *user_data, int video_width, int vide
                                       double *dest_pixel_aspect, int *win_x, int *win_y) {
   directfb_driver_t *this = (directfb_driver_t *) user_data;
 
+  (void)video_width;
+  (void)video_height;
+
   *dest_x            = 0;
   *dest_y            = 0;
   *dest_width        = this->screen_width;
   *dest_height       = this->screen_height;
-  *dest_pixel_aspect = video_pixel_aspect ? :
+  *dest_pixel_aspect = (video_pixel_aspect > 0.01) ? video_pixel_aspect :
                        (double)this->screen_width/(double)this->screen_height;
   *win_x             = 0;
   *win_y             = 0;
@@ -1819,7 +1828,7 @@ static vo_driver_t *open_plugin_fb (video_driver_class_t *class_gen, const void
                              _("video layer id (auto: -1)"),
                              _("Select the video output layer by its id."),
                              20, NULL, 0);
-  if (id == -1) {
+  if (id == (DFBDisplayLayerID)-1) {
     IDirectFBScreen *screen;
 
     /* retrieve an interface to the current screen */
@@ -1923,6 +1932,8 @@ static void *init_class_fb (xine_t *xine, const void *visual_gen) {
   directfb_class_t *this;
   const char       *error;
 
+  (void)visual_gen;
+
   /* check DirectFB version */
   error = DirectFBCheckVersion( DIRECTFB_MAJOR_VERSION,
                                 DIRECTFB_MINOR_VERSION,
