@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2018 the xine project
  * Copyright (C) 2004 the Unichrome project
  *
  * This file is part of xine, a free video player.
@@ -31,10 +31,6 @@
 extern "C" {
 #endif
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 typedef struct xine_macroblock_s {
   short  *blockptr;          /* pointer to current dct block */
   short  *blockbaseptr;      /* pointer to base of dct block array in blocks */
