@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2019 the xine project
+ * Copyright (C) 2000-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -48,7 +48,6 @@ static void *audio_decoder_loop (void *stream_gen) {
   xine_stream_private_t *stream = (xine_stream_private_t *)stream_gen;
   xine_private_t *xine = (xine_private_t *)stream->s.xine;
   xine_ticket_t   *running_ticket = xine->port_ticket;
-  buf_element_t   *buf = NULL;
   buf_element_t   *headers_first = NULL, **headers_add = &headers_first, *headers_replay = NULL;
   int              running = 1;
   int              prof_audio_decode = -1;
@@ -80,6 +79,8 @@ static void *audio_decoder_loop (void *stream_gen) {
   running_ticket->acquire (running_ticket, 0);
 
   while (running) {
+    int handled, ignore;
+    buf_element_t *buf;
 
     lprintf ("audio_loop: waiting for package...\n");
 
@@ -98,7 +99,12 @@ static void *audio_decoder_loop (void *stream_gen) {
 
         if ((buf->type & 0xffff0000) == BUF_AUDIO_UNKNOWN)
           break;
-        if (_x_stream_info_get (&stream->s, XINE_STREAM_INFO_IGNORE_AUDIO))
+        xine_rwlock_rdlock (&stream->info_lock);
+        handled = stream->stream_info[XINE_STREAM_INFO_AUDIO_HANDLED];
+        ignore  = stream->stream_info[XINE_STREAM_INFO_IGNORE_AUDIO];
+        xine_rwlock_unlock (&stream->info_lock);
+        (void)handled; /* dont optimize away the read. */
+        if (ignore)
           break;
         xine_profiler_start_count (prof_audio_decode);
 
@@ -170,8 +176,10 @@ static void *audio_decoder_loop (void *stream_gen) {
                 }
                 stream->audio_decoder_streamtype = streamtype;
                 stream->audio_decoder_plugin = _x_get_audio_decoder (&stream->s, streamtype);
-                _x_stream_info_set (&stream->s, XINE_STREAM_INFO_AUDIO_HANDLED,
-                  (stream->audio_decoder_plugin != NULL));
+                handled = (stream->audio_decoder_plugin != NULL);
+                xine_rwlock_wrlock (&stream->info_lock);
+                stream->stream_info[XINE_STREAM_INFO_AUDIO_HANDLED] = handled;
+                xine_rwlock_unlock (&stream->info_lock);
                 /* audio_br_reset */
                 audio_br_lasttime = 0;
                 audio_br_lastsize = 0;
@@ -190,11 +198,7 @@ static void *audio_decoder_loop (void *stream_gen) {
                 }
               }
 
-              /* finally - decode data */
-              if (stream->audio_decoder_plugin)
-                stream->audio_decoder_plugin->decode_data (stream->audio_decoder_plugin, buf);
-
-              /* audio_br_add */
+              /* audio_br_add. some decoders reset buf->pts, do this first. */
               if (buf->pts) {
                 int64_t d = buf->pts - audio_br_lasttime;
                 if (d > 0) {
@@ -209,13 +213,15 @@ static void *audio_decoder_loop (void *stream_gen) {
                         audio_br_bytes >>= 1;
                         audio_br_time  >>= 1;
                       }
-                      br = (uint64_t)audio_br_bytes * 90000 * 8 / audio_br_time;
+                      br = xine_uint_mul_div (audio_br_bytes, 90000 * 8, audio_br_time);
                       bdiff = br - audio_br_value;
                       if (bdiff < 0)
                         bdiff = -bdiff;
                       if (bdiff > (br >> 6)) {
                         audio_br_value = br;
-                        _x_stream_info_set (&stream->s, XINE_STREAM_INFO_AUDIO_BITRATE, br);
+                        xine_rwlock_wrlock (&stream->info_lock);
+                        stream->stream_info[XINE_STREAM_INFO_AUDIO_BITRATE] = br;
+                        xine_rwlock_unlock (&stream->info_lock);
                       }
                     }
                   }
@@ -228,12 +234,21 @@ static void *audio_decoder_loop (void *stream_gen) {
               }
               audio_br_lastsize += buf->size;
 
-              if (buf->type != buftype_unknown &&
-                !_x_stream_info_get (&stream->s, XINE_STREAM_INFO_AUDIO_HANDLED)) {
+              /* finally - decode data */
+              if (stream->audio_decoder_plugin)
+                stream->audio_decoder_plugin->decode_data (stream->audio_decoder_plugin, buf);
+
+              /* no need to lock again. it may have been reset from this thread inside
+               * audio_decoder_plugin->decode_data (), if at all.
+               * XXX: should we try a different decoder then? */
+              handled = stream->stream_info[XINE_STREAM_INFO_AUDIO_HANDLED];
+              if (!handled && (buf->type != buftype_unknown)) {
+                const char *aname = _x_buf_audio_name (buf->type);
+
                 xine_log (stream->s.xine, XINE_LOG_MSG,
-                  _("audio_decoder: no plugin available to handle '%s'\n"), _x_buf_audio_name (buf->type));
+                  _("audio_decoder: no plugin available to handle '%s'\n"), aname);
                 if (!_x_meta_info_get (&stream->s, XINE_META_INFO_AUDIOCODEC))
-                  _x_meta_info_set_utf8 (&stream->s, XINE_META_INFO_AUDIOCODEC, _x_buf_audio_name (buf->type));
+                  _x_meta_info_set_utf8 (&stream->s, XINE_META_INFO_AUDIOCODEC, aname);
                 buftype_unknown = buf->type;
                 /* fatal error - dispose plugin */
                 if (stream->audio_decoder_plugin) {
@@ -257,16 +272,16 @@ static void *audio_decoder_loop (void *stream_gen) {
           int t;
 
           case BUFTYPE_SUB (BUF_CONTROL_HEADERS_DONE):
-            pthread_mutex_lock (&stream->counter_lock);
-            stream->header_count_audio++;
+            pthread_mutex_lock (&stream->counter.lock);
+            stream->counter.headers_audio++;
             if (stream->video_thread_created) {
               /* avoid useless wakes on an incomplete pair */
-              if (stream->header_count_audio <= stream->header_count_video)
-                pthread_cond_broadcast (&stream->counter_changed);
+              if (stream->counter.headers_audio <= stream->counter.headers_video)
+                pthread_cond_broadcast (&stream->counter.changed);
             } else {
-              pthread_cond_broadcast (&stream->counter_changed);
+              pthread_cond_broadcast (&stream->counter.changed);
             }
-            pthread_mutex_unlock (&stream->counter_lock);
+            pthread_mutex_unlock (&stream->counter.lock);
             break;
 
           case BUFTYPE_SUB (BUF_CONTROL_START):
@@ -317,25 +332,25 @@ static void *audio_decoder_loop (void *stream_gen) {
             }
             running_ticket->release (running_ticket, 0);
             /* wait for video to reach this marker, if necessary */
-            pthread_mutex_lock (&stream->counter_lock);
-            stream->finished_count_audio++;
-            lprintf ("reached end marker # %d\n", stream->finished_count_audio);
+            pthread_mutex_lock (&stream->counter.lock);
+            stream->counter.finisheds_audio++;
+            lprintf ("reached end marker # %d\n", stream->counter.finisheds_audio);
             if (stream->video_thread_created) {
-              if (stream->finished_count_audio > stream->finished_count_video) {
+              if (stream->counter.finisheds_audio > stream->counter.finisheds_video) {
                 do {
                   struct timespec ts = {0, 0};
                   xine_gettime (&ts);
                   ts.tv_sec += 1;
                   /* use timedwait to workaround buggy pthread broadcast implementations */
-                  pthread_cond_timedwait (&stream->counter_changed, &stream->counter_lock, &ts);
-                } while (stream->finished_count_audio > stream->finished_count_video);
-              } else if (stream->finished_count_audio == stream->finished_count_video) {
-                pthread_cond_broadcast (&stream->counter_changed);
+                  pthread_cond_timedwait (&stream->counter.changed, &stream->counter.lock, &ts);
+                } while (stream->counter.finisheds_audio > stream->counter.finisheds_video);
+              } else if (stream->counter.finisheds_audio == stream->counter.finisheds_video) {
+                pthread_cond_broadcast (&stream->counter.changed);
               }
             } else {
-              pthread_cond_broadcast (&stream->counter_changed);
+              pthread_cond_broadcast (&stream->counter.changed);
             }
-            pthread_mutex_unlock (&stream->counter_lock);
+            pthread_mutex_unlock (&stream->counter.lock);
             stream->s.audio_channel_auto = -1;
             running_ticket->acquire (running_ticket, 0);
             break;
