@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2018 the xine project
+ * Copyright (C) 2000-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -855,6 +855,67 @@ static cfg_entry_t *config_lookup_entry_safe (config_values_t *this, const char
   return this->lookup_entry (this, key);
 }
 
+static char *config_lookup_string(config_values_t *this, const char *key) {
+  cfg_entry_t *entry;
+  char *str_value = NULL;
+
+  entry = config_lookup_entry_safe(this, key);
+  if (!entry) {
+    if (this->xine)
+      xprintf (this->xine, XINE_VERBOSITY_LOG, "configfile: WARNING: entry %s not found\n", key);
+    goto out;
+  }
+
+  if (entry->type != XINE_CONFIG_TYPE_STRING) {
+    if (this->xine)
+      xprintf (this->xine, XINE_VERBOSITY_LOG, "configfile: WARNING: %s is not string entry\n", key);
+    goto out;
+  }
+
+  if (entry->str_value) {
+    str_value = strdup(entry->str_value);
+  }
+
+out:
+  pthread_mutex_unlock(&this->config_lock);
+  return str_value;
+}
+
+static void config_free_string(config_values_t *this, char **str)
+{
+  (void)this;
+  _x_freep(str);
+}
+
+static int config_lookup_num(config_values_t *this, const char *key, int def_value) {
+  cfg_entry_t *entry;
+  int value = def_value;
+
+  entry = config_lookup_entry_safe(this, key);
+  if (!entry) {
+    if (this->xine)
+      xprintf (this->xine, XINE_VERBOSITY_LOG, "configfile: WARNING: entry %s not found\n", key);
+    goto out;
+  }
+
+  switch (entry->type) {
+    case XINE_CONFIG_TYPE_RANGE:
+    case XINE_CONFIG_TYPE_ENUM:
+    case XINE_CONFIG_TYPE_NUM:
+    case XINE_CONFIG_TYPE_BOOL:
+      value = entry->num_value;
+      break;
+    default:
+      if (this->xine)
+        xprintf (this->xine, XINE_VERBOSITY_LOG, "configfile: WARNING: %s is not numeric entry\n", key);
+      break;
+  }
+
+out:
+  pthread_mutex_unlock(&this->config_lock);
+  return value;
+}
+
 static void config_reset_value(cfg_entry_t *entry) {
   /* NULL is a frequent case. */
   if (entry->str_value)   {free (entry->str_value);   entry->str_value = NULL;}
@@ -933,9 +994,11 @@ static char *config_register_string (config_values_t *this,
 
   cfg_entry_t *entry;
 
-  if (!this || !key || !def_value) {
-    printf ("config_register_string: error: config=%p, key=%s, def_value=%s.\n",
-    (void *)this, key ? key : "NULL", def_value ? def_value : "NULL");
+  if (!key || !def_value) {
+    if (this->xine)
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+               "config_register_string: error: config=%p, key=%s, def_value=%s.\n",
+               (void *)this, key ? key : "NULL", def_value ? def_value : "NULL");
     return NULL;
   }
 
@@ -967,9 +1030,11 @@ static char *config_register_filename (config_values_t *this,
 
   cfg_entry_t *entry;
 
-  if (!this || !key || !def_value) {
-    printf ("config_register_filename: error: config=%p, key=%s, def_value=%s.\n",
-    (void *)this, key ? key : "NULL", def_value ? def_value : "NULL");
+  if (!key || !def_value) {
+    if (this->xine)
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+               "config_register_filename: error: config=%p, key=%s, def_value=%s.\n",
+               (void *)this, key ? key : "NULL", def_value ? def_value : "NULL");
     return NULL;
   }
 
@@ -1005,9 +1070,11 @@ static int config_register_num (config_values_t *this,
 
   cfg_entry_t *entry;
 
-  if (!this || !key) {
-    printf ("config_register_num: error: config=%p, key=%s.\n",
-    (void *)this, key ? key : "NULL");
+  if (!key) {
+    if (this->xine)
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+               "config_register_num: error: config=%p, key=%s.\n",
+               (void *)this, key ? key : "NULL");
     return 0;
   }
 
@@ -1038,9 +1105,11 @@ static int config_register_bool (config_values_t *this,
 
   cfg_entry_t *entry;
 
-  if (!this || !key) {
-    printf ("config_register_bool: error: config=%p, key=%s.\n",
-    (void *)this, key ? key : "NULL");
+  if (!key) {
+    if (this->xine)
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+               "config_register_bool: error: config=%p, key=%s.\n",
+               (void *)this, key ? key : "NULL");
     return 0;
   }
 
@@ -1072,9 +1141,11 @@ static int config_register_range (config_values_t *this,
 
   cfg_entry_t *entry;
 
-  if (!this || !key) {
-    printf ("config_register_range: error: config=%p, key=%s.\n",
-    (void *)this, key ? key : "NULL");
+  if (!key) {
+    if (this->xine)
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+               "config_register_range: error: config=%p, key=%s.\n",
+               (void *)this, key ? key : "NULL");
     return 0;
   }
 
@@ -1149,9 +1220,11 @@ static int config_register_enum (config_values_t *this,
 
   cfg_entry_t *entry;
 
-  if (!this || !key || !values) {
-    printf ("config_register_enum: error: config=%p, key=%s, values=%p.\n",
-    (void *)this, key ? key : "NULL", (void *)values);
+  if (!key || !values) {
+    if (this->xine)
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+               "config_register_enum: error: config=%p, key=%s, values=%p.\n",
+               (void *)this, key ? key : "NULL", (void *)values);
     return 0;
   }
 
@@ -1175,6 +1248,36 @@ static int config_register_enum (config_values_t *this,
       entry->num_value = 0;
     if (entry->num_value >= (int)value_count)
       entry->num_value = value_count;
+  } else if (entry->type == XINE_CONFIG_TYPE_ENUM) {
+    /* xine-ui does this to update the list. */
+    if (entry->enum_values && values) {
+      const char **old, **new;
+      for (old = (const char **)entry->enum_values, new = (const char **)values; *old && *new; old++, new++) {
+        if (strcmp (*old, *new))
+          break;
+      }
+      if (*old || *new) {
+        uint32_t value_count;
+        char **nv = str_array_dup ((const char **)values, &value_count);
+        if (nv) {
+          int found = -1;
+          if ((entry->num_value >=0) && (entry->num_value < entry->range_max)) {
+            for (new = (const char **)values; *new; new++) {
+              if (!strcmp (entry->enum_values[entry->num_value], *new)) {
+                found = new - (const char **)values;
+                break;
+              }
+            }
+          }
+          str_array_free (entry->enum_values);
+          entry->enum_values = nv;
+          entry->num_default = (def_value >= 0) && (def_value < (int)value_count) ? def_value : 0;
+          entry->num_value = found < 0 ? 0 : found;
+          entry->range_min = 0;
+          entry->range_max = value_count;
+        }
+      }
+    }
   }
 
   pthread_mutex_unlock (&this->config_lock);
@@ -1231,8 +1334,10 @@ static void config_update_num (config_values_t *this, const char *key, int value
   }
   if ((entry->type == XINE_CONFIG_TYPE_UNKNOWN)
       || (entry->type == XINE_CONFIG_TYPE_STRING)) {
-    printf ("configfile: error - tried to update non-num type %d (key %s, value %d)\n",
-	    entry->type, entry->key, value);
+    if (this->xine)
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+               "configfile: error - tried to update non-num type %d (key %s, value %d)\n",
+               entry->type, entry->key, value);
     pthread_mutex_unlock (&this->config_lock);
     return;
   }
@@ -1317,7 +1422,9 @@ static void config_update_string (config_values_t *this, const char *key, const
   lprintf ("updating %s to %s\n", key, value);
   entry = config_lookup_entry_safe (this, key);
   if (!entry) {
-    printf ("configfile: error - tried to update unknown key %s (to %s)\n", key, value);
+    if (this->xine)
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+               "configfile: error - tried to update unknown key %s (to %s)\n", key, value);
     pthread_mutex_unlock (&this->config_lock);
     return;
   }
@@ -1797,7 +1904,8 @@ static void put_int (uint8_t **dest, int value) {
 
 static void put_string (uint8_t **dest, const char *value, uint32_t value_len) {
   put_int (dest, value_len);
-  memcpy (*dest, value, value_len);
+  if (value_len > 0)
+    memcpy (*dest, value, value_len);
   *dest += value_len;
 }
 
@@ -2080,6 +2188,9 @@ config_values_t *_x_config_init (void) {
   this->unset_new_entry_callback  = config_unset_new_entry_callback;
   this->get_serialized_entry      = config_get_serialized_entry;
   this->unregister_callbacks      = config_unregister_callbacks;
+  this->lookup_num                = config_lookup_num;
+  this->lookup_string             = config_lookup_string;
+  this->free_string               = config_free_string;
 
   return this;
 }
