@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2019 the xine project
+ * Copyright (C) 2000-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -26,11 +26,13 @@
 
 #include <pthread.h>
 #include <sys/time.h>
+#include <errno.h>
 
 #include <xine/xine_internal.h>
+#include <xine/sorted_array.h>
 #include "xine_private.h"
 
-#define MAX_REUSE_EVENTS 8
+#define MAX_REUSE_EVENTS 16
 #define MAX_REUSE_DATA 256
 
 typedef struct xine_event_queue_private_s xine_event_queue_private_t;
@@ -43,10 +45,22 @@ typedef struct {
 struct xine_event_queue_private_s {
   xine_event_queue_t  q;
   xine_list_t        *free_events;
+  xine_sarray_t      *select;
   int                 refs;
+  int                 flush;
+  struct timeval      lasttime;
   int                 num_all;
   int                 num_alloc;
   int                 num_skip;
+#if (HAVE_ATOMIC_VARS > 0)
+  /* give the "get" functions a quick fall through
+   * for the most common "no events" case.
+   * this is basically noise shaping. */
+  xine_refs_t         pending;
+#else
+  /* skip that mutex fallback. we already have a mutex. */
+#endif
+  pthread_t           handler;
   struct {
     xine_event_private_t e;
     uint8_t data[MAX_REUSE_DATA];
@@ -54,17 +68,27 @@ struct xine_event_queue_private_s {
 };
 
 xine_event_t *xine_event_get  (xine_event_queue_t *queue) {
+  xine_event_queue_private_t *q = (xine_event_queue_private_t *)queue;
   xine_event_t *event;
   xine_list_iterator_t ite;
 
-  if (!queue)
+  if (!q)
     return NULL;
-  pthread_mutex_lock (&queue->lock);
+#if (HAVE_ATOMIC_VARS > 0)
+  if (xine_refs_get (&q->pending) <= 1)
+    return NULL;
+#endif
+  pthread_mutex_lock (&q->q.lock);
   ite = NULL;
-  event = xine_list_next_value (queue->events, &ite);
-  if (ite)
-    xine_list_remove (queue->events, ite);
-  pthread_mutex_unlock (&queue->lock);
+  event = xine_list_next_value (q->q.events, &ite);
+  if (ite) {
+    xine_list_remove (q->q.events, ite);
+#if (HAVE_ATOMIC_VARS > 0)
+    if (xine_list_size (q->q.events) == 0)
+      xine_refs_add (&q->pending, -1);
+#endif
+  }
+  pthread_mutex_unlock (&q->q.lock);
 
   return event;
 }
@@ -74,26 +98,38 @@ xine_event_t *xine_event_next (xine_event_queue_t *queue, xine_event_t *prev_eve
 
   if (!q)
     return NULL;
-  pthread_mutex_lock (&q->q.lock);
   if (prev_event) {
     if (PTR_IN_RANGE (prev_event, &q->revents, sizeof (q->revents))) {
+      pthread_mutex_lock (&q->q.lock);
       q->refs -= 1;
       xine_list_push_back (q->free_events, prev_event);
     } else {
       free (prev_event);
+      pthread_mutex_lock (&q->q.lock);
     }
+  } else {
+#if (HAVE_ATOMIC_VARS > 0)
+    if (xine_refs_get (&q->pending) <= 1)
+      return NULL;
+#endif
+    pthread_mutex_lock (&q->q.lock);
   }
   {
     xine_list_iterator_t ite = NULL;
     xine_event_t *event = xine_list_next_value (q->q.events, &ite);
-    if (ite)
+    if (ite) {
       xine_list_remove (q->q.events, ite);
+#if (HAVE_ATOMIC_VARS > 0)
+      if (xine_list_size (q->q.events) == 0)
+        xine_refs_add (&q->pending, -1);
+#endif
+    }
     pthread_mutex_unlock (&q->q.lock);
     return event;
   }
 }
 
-static xine_event_t *xine_event_wait_locked (xine_event_queue_t *queue) {
+static xine_event_t *xine_event_wait_locked (xine_event_queue_private_t *q) {
   xine_event_t  *event, *first_event;
   xine_list_iterator_t ite, first_ite;
   int wait;
@@ -101,10 +137,10 @@ static xine_event_t *xine_event_wait_locked (xine_event_queue_t *queue) {
   /* wait until there is at least 1 event */
   while (1) {
     first_ite = NULL;
-    first_event = xine_list_next_value (queue->events, &first_ite);
+    first_event = xine_list_next_value (q->q.events, &first_ite);
     if (first_ite)
       break;
-    pthread_cond_wait (&queue->new_event, &queue->lock);
+    pthread_cond_wait (&q->q.new_event, &q->q.lock);
   }
 
   /* calm down bursting progress events pt 2:
@@ -117,7 +153,7 @@ static xine_event_t *xine_event_wait_locked (xine_event_queue_t *queue) {
       wait = 0;
       break;
     }
-    event = xine_list_next_value (queue->events, &ite);
+    event = xine_list_next_value (q->q.events, &ite);
   } while (ite);
   if (wait) {
     struct timespec ts = {0, 0};
@@ -127,46 +163,54 @@ static xine_event_t *xine_event_wait_locked (xine_event_queue_t *queue) {
       ts.tv_nsec -= 1000000000;
       ts.tv_sec += 1;
     }
-    pthread_cond_timedwait (&queue->new_event, &queue->lock, &ts);
+    pthread_cond_timedwait (&q->q.new_event, &q->q.lock, &ts);
     /* paranoia ? */
     while (1) {
       first_ite = NULL;
-      first_event = xine_list_next_value (queue->events, &first_ite);
+      first_event = xine_list_next_value (q->q.events, &first_ite);
       if (first_ite)
         break;
-      pthread_cond_wait (&queue->new_event, &queue->lock);
+      pthread_cond_wait (&q->q.new_event, &q->q.lock);
     }
   }
 
-  xine_list_remove (queue->events, first_ite);
+  xine_list_remove (q->q.events, first_ite);
+#if (HAVE_ATOMIC_VARS > 0)
+  if (xine_list_size (q->q.events) == 0)
+    xine_refs_add (&q->pending, -1);
+#endif
   return first_event;
 }
 
 xine_event_t *xine_event_wait (xine_event_queue_t *queue) {
-
+  xine_event_queue_private_t *q = (xine_event_queue_private_t *)queue;
   xine_event_t  *event;
 
-  pthread_mutex_lock (&queue->lock);
-  event = xine_event_wait_locked(queue);
-  pthread_mutex_unlock (&queue->lock);
-
+  pthread_mutex_lock (&q->q.lock);
+  event = xine_event_wait_locked (q);
+  pthread_mutex_unlock (&q->q.lock);
   return event;
 }
 
-static int xine_event_queue_unref_unlock (xine_event_queue_private_t *queue) {
+static int xine_event_queue_unref_unlock (xine_event_queue_private_t *q) {
   int refs;
 
-  queue->refs -= 1;
-  refs = queue->refs;
-  pthread_mutex_unlock (&queue->q.lock);
+  q->refs -= 1;
+  refs = q->refs;
+  pthread_mutex_unlock (&q->q.lock);
 
   if (refs == 0) {
-    xine_list_delete (queue->q.events);
-    xine_list_delete (queue->free_events);
-    pthread_mutex_destroy (&queue->q.lock);
-    pthread_cond_destroy (&queue->q.new_event);
-    pthread_cond_destroy (&queue->q.events_processed);
-    free (queue);
+#if (HAVE_ATOMIC_VARS > 0)
+    xine_refs_sub (&q->pending, 1);
+#endif
+    xine_list_delete (q->q.events);
+    xine_list_delete (q->free_events);
+    pthread_mutex_destroy (&q->q.lock);
+    pthread_cond_destroy (&q->q.new_event);
+    pthread_cond_destroy (&q->q.events_processed);
+    if (q->select)
+      xine_sarray_delete (q->select);
+    free (q);
   }
 
   return refs;
@@ -175,17 +219,17 @@ static int xine_event_queue_unref_unlock (xine_event_queue_private_t *queue) {
 void xine_event_free (xine_event_t *event) {
   /* XXX; assuming this was returned by xine_event_get () or xine_event_wait () before. */
   xine_event_private_t *e = (xine_event_private_t *)event;
-  xine_event_queue_private_t *queue;
+  xine_event_queue_private_t *q;
 
   if (!e)
     return;
-  queue = e->queue;
-  if (!queue)
+  q = e->queue;
+  if (!q)
     return;
-  if (PTR_IN_RANGE (e, &queue->revents, sizeof (queue->revents))) {
-    pthread_mutex_lock (&queue->q.lock);
-    xine_list_push_back (queue->free_events, e);
-    xine_event_queue_unref_unlock (queue);
+  if (PTR_IN_RANGE (e, &q->revents, sizeof (q->revents))) {
+    pthread_mutex_lock (&q->q.lock);
+    xine_list_push_back (q->free_events, e);
+    xine_event_queue_unref_unlock (q);
   } else {
     free (event);
   }
@@ -194,7 +238,7 @@ void xine_event_free (xine_event_t *event) {
 void xine_event_send (xine_stream_t *s, const xine_event_t *event) {
   xine_stream_private_t *stream = (xine_stream_private_t *)s, *mstream;
   xine_list_iterator_t ite;
-  xine_event_queue_private_t *queue;
+  xine_event_queue_private_t *q;
   void *data;
   struct timeval now = {0, 0};
 
@@ -204,38 +248,50 @@ void xine_event_send (xine_stream_t *s, const xine_event_t *event) {
   data = (event->data_length <= 0) ? NULL : event->data;
 
   gettimeofday (&now, NULL);
-  pthread_mutex_lock (&mstream->event_queues_lock);
+  pthread_mutex_lock (&mstream->event.lock);
 
   ite = NULL;
-  while ((queue = xine_list_next_value (mstream->event_queues, &ite))) {
+  while ((q = xine_list_next_value (mstream->event.queues, &ite))) {
     xine_event_private_t *new_event;
 
     /* XINE_EVENT_VDR_DISCONTINUITY: .data == NULL, .data_length == DISC_* */
     if (!data) {
       xine_list_iterator_t it2 = NULL;
-      pthread_mutex_lock (&queue->q.lock);
-      new_event = xine_list_next_value (queue->free_events, &it2);
+      pthread_mutex_lock (&q->q.lock);
+      if (q->select) {
+        if (xine_sarray_binary_search (q->select, (void *)(intptr_t)event->type) < 0) {
+          q->num_all += 1;
+          q->num_skip += 1;
+          pthread_mutex_unlock (&q->q.lock);
+          continue;
+        }
+      }
+      new_event = xine_list_next_value (q->free_events, &it2);
       if (new_event) {
-        xine_list_remove (queue->free_events, it2);
-        queue->refs += 1;
+        xine_list_remove (q->free_events, it2);
+        q->refs += 1;
       } else {
         new_event = malloc (sizeof (*new_event));
         if (!new_event) {
-          pthread_mutex_unlock (&queue->q.lock);
+          pthread_mutex_unlock (&q->q.lock);
           continue;
         }
-        queue->num_alloc += 1;
+        q->num_alloc += 1;
       }
       new_event->e.data        = NULL;
-      new_event->queue         = queue;
+      new_event->queue         = q;
       new_event->e.type        = event->type;
       new_event->e.data_length = event->data_length;
       new_event->e.stream      = &stream->s;
       new_event->e.tv          = now;
-      queue->num_all += 1;
-      xine_list_push_back (queue->q.events, new_event);
-      pthread_cond_signal (&queue->q.new_event);
-      pthread_mutex_unlock (&queue->q.lock);
+      q->num_all += 1;
+      xine_list_push_back (q->q.events, new_event);
+#if (HAVE_ATOMIC_VARS > 0)
+      if (xine_list_size (q->q.events) == 1)
+        xine_refs_add (&q->pending, 1);
+#endif
+      pthread_cond_signal (&q->q.new_event);
+      pthread_mutex_unlock (&q->q.lock);
       continue;
     }
 
@@ -243,15 +299,23 @@ void xine_event_send (xine_stream_t *s, const xine_event_t *event) {
      * if list tail has an earlier instance, update it without signal. */
     if ((event->type == XINE_EVENT_PROGRESS) || (event->type == XINE_EVENT_NBC_STATS)) {
       xine_event_t *e2;
-      pthread_mutex_lock (&queue->q.lock);
+      pthread_mutex_lock (&q->q.lock);
+      if (q->select) {
+        if (xine_sarray_binary_search (q->select, (void *)(intptr_t)event->type) < 0) {
+          q->num_all += 1;
+          q->num_skip += 1;
+          pthread_mutex_unlock (&q->q.lock);
+          continue;
+        }
+      }
       do {
         xine_list_iterator_t it2 = NULL;
-        e2 = xine_list_prev_value (queue->q.events, &it2);
+        e2 = xine_list_prev_value (q->q.events, &it2);
         if (!it2)
           break;
         if (e2 && (e2->type == event->type) && e2->data)
           break;
-        e2 = xine_list_prev_value (queue->q.events, &it2);
+        e2 = xine_list_prev_value (q->q.events, &it2);
         if (!it2)
           break;
         if (e2 && (e2->type == event->type) && e2->data)
@@ -264,116 +328,199 @@ void xine_event_send (xine_stream_t *s, const xine_event_t *event) {
           if (pd1->description && pd2->description && !strcmp (pd1->description, pd2->description)) {
             pd2->percent = pd1->percent;
             e2->tv = now;
-            queue->num_all += 1;
-            queue->num_skip += 1;
-            pthread_mutex_unlock (&queue->q.lock);
+            q->num_all += 1;
+            q->num_skip += 1;
+            pthread_mutex_unlock (&q->q.lock);
             continue;
           }
         } else { /* XINE_EVENT_NBC_STATS */
           size_t l = event->data_length < e2->data_length ? event->data_length : e2->data_length;
           memcpy (e2->data, data, l);
           e2->tv = now;
-          queue->num_all += 1;
-          queue->num_skip += 1;
-          pthread_mutex_unlock (&queue->q.lock);
+          q->num_all += 1;
+          q->num_skip += 1;
+          pthread_mutex_unlock (&q->q.lock);
           continue;
         }
       }
-      pthread_mutex_unlock (&queue->q.lock);
+      pthread_mutex_unlock (&q->q.lock);
     }
 
     if (event->data_length <= MAX_REUSE_DATA) {
       xine_list_iterator_t it2 = NULL;
-      pthread_mutex_lock (&queue->q.lock);
-      new_event = xine_list_next_value (queue->free_events, &it2);
+      pthread_mutex_lock (&q->q.lock);
+      if (q->select) {
+        if (xine_sarray_binary_search (q->select, (void *)(intptr_t)event->type) < 0) {
+          q->num_all += 1;
+          q->num_skip += 1;
+          pthread_mutex_unlock (&q->q.lock);
+          continue;
+        }
+      }
+      new_event = xine_list_next_value (q->free_events, &it2);
       if (new_event) {
-        xine_list_remove (queue->free_events, it2);
-        queue->refs += 1;
+        xine_list_remove (q->free_events, it2);
+        q->refs += 1;
         new_event->e.data = (uint8_t *)new_event + sizeof (*new_event);
         xine_small_memcpy (new_event->e.data, data, event->data_length);
-        new_event->queue         = queue;
+        new_event->queue         = q;
         new_event->e.type        = event->type;
         new_event->e.data_length = event->data_length;
         new_event->e.stream      = &stream->s;
         new_event->e.tv          = now;
-        queue->num_all += 1;
-        xine_list_push_back (queue->q.events, new_event);
-        pthread_cond_signal (&queue->q.new_event);
-        pthread_mutex_unlock (&queue->q.lock);
+        q->num_all += 1;
+        xine_list_push_back (q->q.events, new_event);
+#if (HAVE_ATOMIC_VARS > 0)
+        if (xine_list_size (q->q.events) == 1)
+          xine_refs_add (&q->pending, 1);
+#endif
+        pthread_cond_signal (&q->q.new_event);
+        pthread_mutex_unlock (&q->q.lock);
         continue;
       }
-      pthread_mutex_unlock (&queue->q.lock);
+      pthread_mutex_unlock (&q->q.lock);
     }
 
+    pthread_mutex_lock (&q->q.lock);
+    if (q->select) {
+      if (xine_sarray_binary_search (q->select, (void *)(intptr_t)event->type) < 0) {
+        q->num_all += 1;
+        q->num_skip += 1;
+        pthread_mutex_unlock (&q->q.lock);
+        continue;
+      }
+    }
     new_event = malloc (sizeof (*new_event) + event->data_length);
-    if (!new_event)
+    if (!new_event) {
+      pthread_mutex_unlock (&q->q.lock);
       continue;
+    }
     new_event->e.data = (uint8_t *)new_event + sizeof (*new_event);
     memcpy (new_event->e.data, data, event->data_length);
-    new_event->queue         = queue;
+    new_event->queue         = q;
     new_event->e.type        = event->type;
     new_event->e.data_length = event->data_length;
     new_event->e.stream      = &stream->s;
     new_event->e.tv          = now;
-    pthread_mutex_lock (&queue->q.lock);
-    queue->num_all += 1;
-    queue->num_alloc += 1;
-    xine_list_push_back (queue->q.events, new_event);
-    pthread_cond_signal (&queue->q.new_event);
-    pthread_mutex_unlock (&queue->q.lock);
+    q->num_all += 1;
+    q->num_alloc += 1;
+    xine_list_push_back (q->q.events, new_event);
+#if (HAVE_ATOMIC_VARS > 0)
+    if (xine_list_size (q->q.events) == 1)
+      xine_refs_add (&q->pending, 1);
+#endif
+    pthread_cond_signal (&q->q.new_event);
+    pthread_mutex_unlock (&q->q.lock);
   }
 
-  pthread_mutex_unlock (&mstream->event_queues_lock);
+  pthread_mutex_unlock (&mstream->event.lock);
 }
 
 
+#if (HAVE_ATOMIC_VARS > 0)
+static void xine_event_dummy_cb (void *data) {
+  (void)data;
+}
+#endif
+
 xine_event_queue_t *xine_event_new_queue (xine_stream_t *s) {
   xine_stream_private_t *stream = (xine_stream_private_t *)s;
-  xine_event_queue_private_t *queue;
+  xine_event_queue_private_t *q;
   uint32_t n;
 
   if (!stream)
     return NULL;
-  queue = malloc (sizeof (*queue));
-  if (!queue)
+  q = malloc (sizeof (*q));
+  if (!q)
     return NULL;
 
-  queue->refs = 1;
-  queue->num_all = 0;
-  queue->num_alloc = 0;
-  queue->num_skip = 0;
+  q->refs = 1;
+  q->flush = 0;
+  q->lasttime.tv_sec = 0;
+  q->lasttime.tv_usec = 0;
+  q->num_all = 0;
+  q->num_alloc = 0;
+  q->num_skip = 0;
 
-  queue->q.events = xine_list_new ();
-  if (!queue->q.events) {
-    free (queue);
+#if (HAVE_ATOMIC_VARS > 0)
+  xine_refs_init (&q->pending, xine_event_dummy_cb, NULL);
+#endif
+
+  q->q.events = xine_list_new ();
+  if (!q->q.events) {
+    free (q);
     return NULL;
   }
-  queue->free_events = xine_list_new ();
-  if (!queue->free_events) {
-    xine_list_delete (queue->q.events);
-    free (queue);
+  q->free_events = xine_list_new ();
+  if (!q->free_events) {
+    xine_list_delete (q->q.events);
+    free (q);
     return NULL;
   }
+  q->select = NULL;
   for (n = 0; n < MAX_REUSE_EVENTS; n++)
-    xine_list_push_back (queue->free_events, &queue->revents[n]);
+    xine_list_push_back (q->free_events, &q->revents[n]);
 
   xine_refs_add (&stream->refs, 1);
 
-  pthread_mutex_init (&queue->q.lock, NULL);
-  pthread_cond_init (&queue->q.new_event, NULL);
-  pthread_cond_init (&queue->q.events_processed, NULL);
-  queue->q.stream = &stream->s;
-  queue->q.listener_thread = NULL;
-  queue->q.callback_running = 0;
+  pthread_mutex_init (&q->q.lock, NULL);
+  pthread_cond_init (&q->q.new_event, NULL);
+  pthread_cond_init (&q->q.events_processed, NULL);
+  q->q.stream = &stream->s;
+  q->q.listener_thread = NULL;
+  q->q.callback_running = 0;
 
   {
     xine_stream_private_t *mstream = stream->side_streams[0];
-    pthread_mutex_lock (&mstream->event_queues_lock);
-    xine_list_push_back (mstream->event_queues, &queue->q);
-    pthread_mutex_unlock (&mstream->event_queues_lock);
+    pthread_mutex_lock (&mstream->event.lock);
+    xine_list_push_back (mstream->event.queues, &q->q);
+    pthread_mutex_unlock (&mstream->event.lock);
   }
 
-  return &queue->q;
+  return &q->q;
+}
+
+static int xine_event_type_cmp (void *a, void *b) {
+  int d = (intptr_t)a, e = (intptr_t)b;
+  return (d > e) - (d < e);
+}
+
+void xine_event_select (xine_event_queue_t *queue, const int *types) {
+  xine_event_queue_private_t *q = (xine_event_queue_private_t *)queue;
+
+  if (!q)
+    return;
+
+  if (!types) {
+    pthread_mutex_lock (&q->q.lock);
+    if (q->select) {
+      xine_sarray_delete (q->select);
+      q->select = NULL;
+    }
+    pthread_mutex_unlock (&q->q.lock);
+  } else {
+    uint32_t n;
+
+    pthread_mutex_lock (&q->q.lock);
+    if (!q->select) {
+      for (n = 0; types[n] != XINE_EVENT_QUIT; n++) ;
+      q->select = xine_sarray_new (n + 1, xine_event_type_cmp);
+      if (!q->select) {
+        pthread_mutex_unlock (&q->q.lock);
+        return;
+      }
+      xine_sarray_set_mode (q->select, XINE_SARRAY_MODE_UNIQUE);
+    }
+    xine_sarray_clear (q->select);
+    n = 0;
+    while (1) {
+      xine_sarray_add (q->select, (void *)(intptr_t)types[n]);
+      if (types[n] == XINE_EVENT_QUIT)
+        break;
+      n++;
+    }
+    pthread_mutex_unlock (&q->q.lock);
+  }
 }
 
 void xine_event_dispose_queue (xine_event_queue_t *queue) {
@@ -387,15 +534,15 @@ void xine_event_dispose_queue (xine_event_queue_t *queue) {
   {
     xine_stream_private_t *mstream = stream->side_streams[0];
     xine_list_iterator_t  ite;
-    pthread_mutex_lock (&mstream->event_queues_lock);
-    ite = xine_list_find (mstream->event_queues, q);
+    pthread_mutex_lock (&mstream->event.lock);
+    ite = xine_list_find (mstream->event.queues, q);
     if (!ite) {
-      pthread_mutex_unlock (&mstream->event_queues_lock);
+      pthread_mutex_unlock (&mstream->event.lock);
       xprintf (stream->s.xine, XINE_VERBOSITY_DEBUG, "events: tried to dispose queue which is not in list\n");
       return;
     }
-    xine_list_remove (mstream->event_queues, ite);
-    pthread_mutex_unlock (&mstream->event_queues_lock);
+    xine_list_remove (mstream->event.queues, ite);
+    pthread_mutex_unlock (&mstream->event.lock);
   }
 
   /*
@@ -421,6 +568,10 @@ void xine_event_dispose_queue (xine_event_queue_t *queue) {
       qevent->queue         = q;
       gettimeofday (&qevent->e.tv, NULL);
       xine_list_push_back (q->q.events, qevent);
+#if (HAVE_ATOMIC_VARS > 0)
+      if (xine_list_size (q->q.events) == 1)
+         xine_refs_add (&q->pending, 1);
+#endif
       pthread_cond_signal (&q->q.new_event);
     }
     q->num_all += 1;
@@ -434,7 +585,7 @@ void xine_event_dispose_queue (xine_event_queue_t *queue) {
   if (q->q.listener_thread) {
     void *p;
     pthread_join (*q->q.listener_thread, &p);
-    _x_freep (&q->q.listener_thread);
+    q->q.listener_thread = NULL;
   }
 
   /*
@@ -457,6 +608,7 @@ void xine_event_dispose_queue (xine_event_queue_t *queue) {
       }
     }
     xine_list_clear (q->q.events);
+    q->flush = 0;
     pthread_cond_signal (&q->q.events_processed);
     num_all = q->num_all;
     num_alloc = q->num_alloc;
@@ -472,36 +624,42 @@ void xine_event_dispose_queue (xine_event_queue_t *queue) {
 
 
 static void *listener_loop (void *queue_gen) {
+  xine_event_queue_private_t *q = queue_gen;
+  int last_type;
 
-  xine_event_queue_private_t *queue = queue_gen;
-  int running = 1;
+  pthread_mutex_lock (&q->q.lock);
 
-  pthread_mutex_lock (&queue->q.lock);
+  do {
+    xine_event_t *event = xine_event_wait_locked (q);
+    q->lasttime = event->tv;
+    q->q.callback_running = 1;
+    pthread_mutex_unlock (&q->q.lock);
+    last_type = event->type;
 
-  while (running) {
-    xine_event_t *event = xine_event_wait_locked (&queue->q);
-    queue->q.callback_running = 1;
-    pthread_mutex_unlock (&queue->q.lock);
-    if (event->type == XINE_EVENT_QUIT)
-      running = 0;
+    q->q.callback (q->q.user_data, event);
 
-    queue->q.callback (queue->q.user_data, event);
-
-    if (PTR_IN_RANGE (event, &queue->revents, sizeof (queue->revents))) {
-      pthread_mutex_lock (&queue->q.lock);
-      queue->refs -= 1;
+    if (PTR_IN_RANGE (event, &q->revents, sizeof (q->revents))) {
+      pthread_mutex_lock (&q->q.lock);
+      q->refs -= 1;
       /* refs == 0 cannot heppen here. */
-      xine_list_push_back (queue->free_events, event);
+      xine_list_push_back (q->free_events, event);
     } else {
       free (event);
-      pthread_mutex_lock (&queue->q.lock);
+      pthread_mutex_lock (&q->q.lock);
     }
-    queue->q.callback_running = 0;
-    if (xine_list_empty (queue->q.events))
-      pthread_cond_signal (&queue->q.events_processed);
-  }
+    q->q.callback_running = 0;
+    if (q->flush > 0) {
+      q->flush -= 1;
+      if (q->flush <= 0)
+        pthread_cond_signal (&q->q.events_processed);
+    }
+  } while (last_type != XINE_EVENT_QUIT);
 
-  pthread_mutex_unlock (&queue->q.lock);
+  if (q->flush > 0) {
+    q->flush = 0;
+    pthread_cond_signal (&q->q.events_processed);
+  }
+  pthread_mutex_unlock (&q->q.lock);
   return NULL;
 }
 
@@ -509,32 +667,28 @@ static void *listener_loop (void *queue_gen) {
 int xine_event_create_listener_thread (xine_event_queue_t *queue,
                                        xine_event_listener_cb_t callback,
                                        void *user_data) {
+  xine_event_queue_private_t *q = (xine_event_queue_private_t *)queue;
   int err;
 
   _x_assert(queue != NULL);
   _x_assert(callback != NULL);
 
-  if (queue->listener_thread) {
-    xprintf (queue->stream->xine, XINE_VERBOSITY_NONE,
+  if (q->q.listener_thread) {
+    xprintf (q->q.stream->xine, XINE_VERBOSITY_NONE,
              "events: listener thread already created\n");
     return 0;
   }
 
-  queue->listener_thread = malloc (sizeof (pthread_t));
-  if (!queue->listener_thread) {
-    return 0;
-  }
+  q->q.listener_thread = &q->handler;
+  q->q.callback        = callback;
+  q->q.user_data       = user_data;
 
-  queue->callback        = callback;
-  queue->user_data       = user_data;
-
-  if ((err = pthread_create (queue->listener_thread,
-                             NULL, listener_loop, queue)) != 0) {
-    xprintf (queue->stream->xine, XINE_VERBOSITY_NONE,
+  if ((err = pthread_create (q->q.listener_thread, NULL, listener_loop, q)) != 0) {
+    xprintf (q->q.stream->xine, XINE_VERBOSITY_NONE,
              "events: can't create new thread (%s)\n", strerror(err));
-    _x_freep(&queue->listener_thread);
-    queue->callback        = NULL;
-    queue->user_data       = NULL;
+    q->q.listener_thread = NULL;
+    q->q.callback        = NULL;
+    q->q.user_data       = NULL;
     return 0;
   }
 
@@ -543,39 +697,69 @@ int xine_event_create_listener_thread (xine_event_queue_t *queue,
 
 void _x_flush_events_queues (xine_stream_t *s) {
   xine_stream_private_t *stream = (xine_stream_private_t *)s;
+  pthread_t self = pthread_self ();
+  struct timespec ts = {0, 0};
+  struct timeval tv;
 
   if (!stream)
     return;
   stream = stream->side_streams[0];
+  xine_gettime (&ts);
+  tv.tv_sec = ts.tv_sec;
+  tv.tv_usec = ts.tv_nsec / 1000;
+  ts.tv_sec += 1;
 
   while (1) {
-    xine_event_queue_private_t *queue;
+    xine_event_queue_private_t *q;
     xine_list_iterator_t ite = NULL;
     int list_locked = 1;
 
-    pthread_mutex_lock (&stream->event_queues_lock);
-    while ((queue = xine_list_next_value (stream->event_queues, &ite))) {
-      pthread_mutex_lock (&queue->q.lock);
-      /* we might have been called from the very same function that
-       * processes events, therefore waiting here would cause deadlock.
-       * check only queues with listener threads which are not
-       * currently executing their callback functions. */
-      if (queue->q.listener_thread && !queue->q.callback_running && !xine_list_empty (queue->q.events)) {
-        /* make sure this queue does not go away, then unlock list to prevent freezes. */
-        queue->refs += 1;
-        pthread_mutex_unlock (&stream->event_queues_lock);
-        do {
-          pthread_cond_wait (&queue->q.events_processed, &queue->q.lock);
-        } while (!xine_list_empty (queue->q.events));
-        xine_event_queue_unref_unlock (queue);
-        /* list may have changed, restart. */
-        list_locked = 0;
-        break;
+    pthread_mutex_lock (&stream->event.lock);
+    while ((q = xine_list_next_value (stream->event.queues, &ite))) {
+      pthread_mutex_lock (&q->q.lock);
+      /* never wait for self. */
+      if (q->q.listener_thread && !pthread_equal (self, q->handler) && (q->flush == 0)) {
+        /* count pending past events */
+        if ((q->lasttime.tv_sec < tv.tv_sec) ||
+            ((q->lasttime.tv_sec == tv.tv_sec) && (q->lasttime.tv_usec <= tv.tv_usec))) {
+          xine_list_iterator_t ite2 = NULL;
+          q->flush = q->q.callback_running ? 1 : 0;
+          while (1) {
+            xine_event_t *e = xine_list_next_value (q->q.events, &ite2);
+            if (!ite2)
+              break;
+            if ((e->tv.tv_sec > tv.tv_sec) ||
+                ((e->tv.tv_sec == tv.tv_sec) && (e->tv.tv_usec > tv.tv_usec)))
+              break;
+            q->flush += 1;
+          }
+        }
+        if (q->flush > 0) {
+          int err, n = q->flush;
+          /* make sure this queue does not go away, then unlock list to prevent freezes. */
+          q->refs += 1;
+          pthread_mutex_unlock (&stream->event.lock);
+          do {
+            /* paranoia: cyclic wait? */
+            err = pthread_cond_timedwait (&q->q.events_processed, &q->q.lock, &ts);
+          } while ((q->flush > 0) && (err != ETIMEDOUT));
+          xine_event_queue_unref_unlock (q);
+          if (err == ETIMEDOUT) {
+            xprintf (stream->s.xine, XINE_VERBOSITY_DEBUG,
+              "events: warning: _x_flush_events_queues (%p) timeout.\n", (void *)stream);
+          } else {
+            xprintf (stream->s.xine, XINE_VERBOSITY_DEBUG,
+              "events: flushed %d events for stream %p.\n", n, (void *)stream);
+          }
+          /* list may have changed, restart. */
+          list_locked = 0;
+          break;
+        }
       }
-      pthread_mutex_unlock (&queue->q.lock);
+      pthread_mutex_unlock (&q->q.lock);
     }
     if (list_locked) {
-      pthread_mutex_unlock (&stream->event_queues_lock);
+      pthread_mutex_unlock (&stream->event.lock);
       break;
     }
   }
