@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -22,9 +22,14 @@
  * some helper functions for post plugins
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #define POST_INTERNAL
 
 #include <xine/post.h>
+#include "xine_private.h"
 
 #include <stdarg.h>
 #include <pthread.h>
@@ -63,11 +68,8 @@ static int  post_frame_draw       (vo_frame_t *vo_img, xine_stream_t *stream);
 static void post_frame_free       (vo_frame_t *vo_img);
 static void post_frame_dispose    (vo_frame_t *vo_img);
 
-static vo_frame_t *post_intercept_video_frame (vo_frame_t *frame, post_video_port_t *port, int usage) {
+static vf_alias_t *post_new_video_alias (post_video_port_t *port, int usage) {
   vf_alias_t *new_frame;
-
-  if (usage && port->frame_lock)
-    pthread_mutex_lock (port->frame_lock);
   /* get a free frame slot */
   pthread_mutex_lock (&port->usage_lock);
   if (port->free_frame_slots) {
@@ -79,6 +81,27 @@ static vo_frame_t *post_intercept_video_frame (vo_frame_t *frame, post_video_por
   if (usage)
     port->usage_count++;
   pthread_mutex_unlock (&port->usage_lock);
+  return new_frame;
+}
+
+static void post_free_unused_video_alias (post_video_port_t *port, vf_alias_t *f) {
+  /* put the now free slot into the free frames list */
+  pthread_mutex_lock (&port->usage_lock);
+  f->frame.next = port->free_frame_slots;
+  port->free_frame_slots = &f->frame;
+  port->usage_count--;
+  if (port->usage_count || !port->post->dispose_pending) {
+    pthread_mutex_unlock (&port->usage_lock);
+  } else {
+    pthread_mutex_unlock (&port->usage_lock);
+    port->post->dispose (port->post);
+  }
+}
+
+static vo_frame_t *post_intercept_video_frame (post_video_port_t *port, vo_frame_t *frame, vf_alias_t *new_frame, int usage) {
+
+  if (usage && port->frame_lock)
+    pthread_mutex_lock (port->frame_lock);
 
   /* make a copy and attach the original */
   xine_fast_memcpy (&new_frame->frame, frame, sizeof (vo_frame_t));
@@ -97,9 +120,13 @@ static vo_frame_t *post_intercept_video_frame (vo_frame_t *frame, post_video_por
   /* Optimization: dont NULL stream ref, will often be reused later */
   if ((new_frame->frame.free == post_frame_free) &&
     new_frame->frame.stream && (new_frame->stream != new_frame->frame.stream)) {
-    _x_refcounter_inc (new_frame->frame.stream->refcounter);
-    if (new_frame->stream)
-      _x_refcounter_dec (new_frame->stream->refcounter);
+    xine_stream_private_t *s;
+    s = (xine_stream_private_t *)new_frame->frame.stream;
+    xine_refs_add (&s->refs, 1);
+    if (new_frame->stream) {
+      s = (xine_stream_private_t *)new_frame->stream;
+      xine_refs_sub (&s->refs, 1);
+    }
     new_frame->stream = new_frame->frame.stream;
   }
 
@@ -137,7 +164,8 @@ static vo_frame_t *post_restore_video_frame (vo_frame_t *frame, post_video_port_
   if ((frame->free == post_frame_free) && !port->stream) {
     vf_alias_t *f = (vf_alias_t *)frame;
     if (f->stream) {
-      _x_refcounter_dec (f->stream->refcounter);
+      xine_stream_private_t *s = (xine_stream_private_t *)f->stream;
+      xine_refs_sub (&s->refs, 1);
        f->stream = NULL;
     }
   }
@@ -164,30 +192,37 @@ static vo_frame_t *post_video_get_frame (xine_video_port_t *port_gen, uint32_t w
 static void post_video_enable_ovl (xine_video_port_t *port_gen, int ovl_enable);
 static video_overlay_manager_t *post_video_get_overlay_manager (xine_video_port_t *port_gen);
 
-static void post_video_port_ref (xine_video_port_t *port_gen) {
+static int post_video_port_ref (xine_video_port_t *port_gen) {
   post_video_port_t *port = (post_video_port_t *)port_gen;
+  int n = -1;
   if (!port)
-    return;
+    return n;
   if  ((port->new_port.get_capabilities    == post_video_get_capabilities)
     || (port->new_port.get_frame           == post_video_get_frame)
     || (port->new_port.enable_ovl          == post_video_enable_ovl)
     || (port->new_port.get_overlay_manager == post_video_get_overlay_manager)) {
     pthread_mutex_lock (&port->usage_lock);
-    port->usage_count++;
+    n = (++port->usage_count);
     pthread_mutex_unlock (&port->usage_lock);
   }
+  return n;
 }
 
-static void post_video_port_unref (xine_video_port_t *port_gen) {
+int _x_post_video_port_ref (xine_video_port_t *port_gen) {
+  return post_video_port_ref (port_gen);
+}
+
+static int post_video_port_unref (xine_video_port_t *port_gen) {
   post_video_port_t *port = (post_video_port_t *)port_gen;
+  int n = -1;
   if (!port)
-    return;
+    return n;
   if  ((port->new_port.get_capabilities    == post_video_get_capabilities)
     || (port->new_port.get_frame           == post_video_get_frame)
     || (port->new_port.enable_ovl          == post_video_enable_ovl)
     || (port->new_port.get_overlay_manager == post_video_get_overlay_manager)) {
     pthread_mutex_lock (&port->usage_lock);
-    port->usage_count--;
+    n = (--port->usage_count);
     if (port->usage_count || !port->post->dispose_pending) {
       pthread_mutex_unlock (&port->usage_lock);
     } else {
@@ -195,6 +230,11 @@ static void post_video_port_unref (xine_video_port_t *port_gen) {
       port->post->dispose (port->post);
     }
   }
+  return n;
+}
+
+int _x_post_video_port_unref (xine_video_port_t *port_gen) {
+  return post_video_port_unref (port_gen);
 }
 
 static uint32_t post_audio_get_capabilities (xine_audio_port_t *port_gen);
@@ -203,30 +243,37 @@ static int post_audio_control (xine_audio_port_t *port_gen, int cmd, ...);
 static void post_audio_put_buffer (xine_audio_port_t *port_gen, audio_buffer_t *buf,
   xine_stream_t *stream);
 
-static void post_audio_port_ref (xine_audio_port_t *port_gen) {
+static int post_audio_port_ref (xine_audio_port_t *port_gen) {
   post_audio_port_t *port = (post_audio_port_t *)port_gen;
+  int n = -1;
   if (!port)
-    return;
+    return n;
   if  ((port->new_port.get_capabilities == post_audio_get_capabilities)
     || (port->new_port.get_buffer       == post_audio_get_buffer)
     || (port->new_port.control          == post_audio_control)
     || (port->new_port.put_buffer       == post_audio_put_buffer)) {
     pthread_mutex_lock (&port->usage_lock);
-    port->usage_count++;
+    n = (++port->usage_count);
     pthread_mutex_unlock (&port->usage_lock);
   }
+  return n;
 }
 
-static void post_audio_port_unref (xine_audio_port_t *port_gen) {
+int _x_post_audio_port_ref (xine_audio_port_t *port_gen) {
+  return post_audio_port_ref (port_gen);
+}
+
+static int post_audio_port_unref (xine_audio_port_t *port_gen) {
   post_audio_port_t *port = (post_audio_port_t *)port_gen;
+  int n = -1;
   if (!port)
-    return;
+    return n;
   if  ((port->new_port.get_capabilities == post_audio_get_capabilities)
     || (port->new_port.get_buffer       == post_audio_get_buffer)
     || (port->new_port.control          == post_audio_control)
     || (port->new_port.put_buffer       == post_audio_put_buffer)) {
     pthread_mutex_lock (&port->usage_lock);
-    port->usage_count--;
+    n = (--port->usage_count);
     if (port->usage_count || !port->post->dispose_pending) {
       pthread_mutex_unlock (&port->usage_lock);
     } else {
@@ -234,8 +281,14 @@ static void post_audio_port_unref (xine_audio_port_t *port_gen) {
       port->post->dispose (port->post);
     }
   }
+  return n;
 }
 
+int _x_post_audio_port_unref (xine_audio_port_t *port_gen) {
+  return post_audio_port_unref (port_gen);
+}
+
+
 void _x_post_init(post_plugin_t *post, int num_audio_inputs, int num_video_inputs) {
   post->input  = xine_list_new();
   post->output = xine_list_new();
@@ -257,8 +310,8 @@ static uint32_t post_video_get_capabilities(xine_video_port_t *port_gen) {
 static void post_video_open(xine_video_port_t *port_gen, xine_stream_t *stream) {
   post_video_port_t *port = (post_video_port_t *)port_gen;
 
-  _x_post_rewire(port->post);
-  _x_post_inc_usage(port);
+  _x_post_inc_usage (port);
+  _x_post_rewire (port->post);
   if (port->port_lock) pthread_mutex_lock(port->port_lock);
   (port->original_port->open) (port->original_port, stream);
   if (port->port_lock) pthread_mutex_unlock(port->port_lock);
@@ -270,15 +323,20 @@ static vo_frame_t *post_video_get_frame(xine_video_port_t *port_gen, uint32_t wi
     uint32_t height, double ratio, int format, int flags) {
   post_video_port_t *port = (post_video_port_t *)port_gen;
   vo_frame_t *frame;
+  /* We always need to ref this port here to protect it from
+   * possible port rewiring inside get_frame ().
+   * In the intercept case, we save an extra unref. */
+  vf_alias_t *alias = post_new_video_alias (port, 1);
 
-  _x_post_rewire(port->post);
   if (port->port_lock) pthread_mutex_lock(port->port_lock);
   frame = port->original_port->get_frame(port->original_port,
     width, height, ratio, format, flags);
   if (port->port_lock) pthread_mutex_unlock(port->port_lock);
 
   if (frame && (!port->intercept_frame || port->intercept_frame(port, frame))) {
-    frame = post_intercept_video_frame (frame, port, 1);
+    frame = post_intercept_video_frame (port, frame, alias, 1);
+  } else {
+    post_free_unused_video_alias (port, alias);
   }
 
   return frame;
@@ -326,7 +384,8 @@ static void post_video_close(xine_video_port_t *port_gen, xine_stream_t *stream)
     f = (vf_alias_t *)port->free_frame_slots;
     while (f) {
       if ((f->frame.free == post_frame_free) && f->stream) {
-        _x_refcounter_dec (f->stream->refcounter);
+        xine_stream_private_t *s = (xine_stream_private_t *)f->stream;
+        xine_refs_sub (&s->refs, 1);
         f->stream = NULL;
       }
       f = (vf_alias_t *)f->frame.next;
@@ -423,8 +482,7 @@ static int post_video_rewire(xine_post_out_t *output_gen, void *data) {
   if (!new_port)
     return 0;
 
-  this->running_ticket->lock_port_rewiring(this->running_ticket, -1);
-  this->running_ticket->revoke(this->running_ticket, 1);
+  this->running_ticket->revoke (this->running_ticket, XINE_TICKET_FLAG_REWIRE);
 
   if (new_port) {
     /* The wiring itself. */
@@ -442,8 +500,7 @@ static int post_video_rewire(xine_post_out_t *output_gen, void *data) {
   }
   input_port->original_port = new_port;
 
-  this->running_ticket->issue(this->running_ticket, 1);
-  this->running_ticket->unlock_port_rewiring(this->running_ticket);
+  this->running_ticket->issue (this->running_ticket, XINE_TICKET_FLAG_REWIRE);
 
   return 1;
 }
@@ -592,7 +649,8 @@ static void post_frame_dispose(vo_frame_t *vo_img) {
 
 
 vo_frame_t *_x_post_intercept_video_frame(vo_frame_t *frame, post_video_port_t *port) {
-  return post_intercept_video_frame (frame, port, 0);
+  vf_alias_t *alias = post_new_video_alias (port, 0);
+  return post_intercept_video_frame (port, frame, alias, 0);
 }
 
 vo_frame_t *_x_post_restore_video_frame(vo_frame_t *frame, post_video_port_t *port) {
@@ -605,9 +663,13 @@ void _x_post_frame_copy_down(vo_frame_t *from, vo_frame_t *to) {
     vf_alias_t *f = (vf_alias_t *)to;
     f->frame.stream = from->stream;
     if (f->frame.stream && (f->frame.stream != f->stream)) {
-      _x_refcounter_inc (f->frame.stream->refcounter);
-      if (f->stream)
-        _x_refcounter_dec (f->stream->refcounter);
+      xine_stream_private_t *s;
+      s = (xine_stream_private_t *)f->frame.stream;
+      xine_refs_add (&s->refs, 1);
+      if (f->stream) {
+        s = (xine_stream_private_t *)f->stream;
+        xine_refs_sub (&s->refs, 1);
+      }
       f->stream = f->frame.stream;
     }
   }
@@ -636,9 +698,13 @@ void _x_post_frame_copy_up(vo_frame_t *to, vo_frame_t *from) {
     vf_alias_t *f = (vf_alias_t *)to;
     f->frame.stream = from->stream;
     if (f->frame.stream && (f->frame.stream != f->stream)) {
-      _x_refcounter_inc (f->frame.stream->refcounter);
-      if (f->stream)
-        _x_refcounter_dec (f->stream->refcounter);
+      xine_stream_private_t *s;
+      s = (xine_stream_private_t *)f->frame.stream;
+      xine_refs_add (&s->refs, 1);
+      if (f->stream) {
+        s = (xine_stream_private_t *)f->stream;
+        xine_refs_sub (&s->refs, 1);
+      }
       f->stream = f->frame.stream;
     }
   }
@@ -656,15 +722,20 @@ void _x_post_frame_u_turn(vo_frame_t *frame, xine_stream_t *stream) {
     vf_alias_t *f = (vf_alias_t *)frame;
     f->frame.stream = stream;
     if (f->frame.stream && (f->frame.stream != f->stream)) {
-      _x_refcounter_inc (f->frame.stream->refcounter);
-      if (f->stream)
-        _x_refcounter_dec (f->stream->refcounter);
+      xine_stream_private_t *s;
+      s = (xine_stream_private_t *)f->frame.stream;
+      xine_refs_add (&s->refs, 1);
+      if (f->stream) {
+        s = (xine_stream_private_t *)f->stream;
+        xine_refs_sub (&s->refs, 1);
+      }
       f->stream = f->frame.stream;
     }
   }
 
   if (stream) {
-    _x_extra_info_merge(frame->extra_info, stream->video_decoder_extra_info);
+    xine_stream_private_t *s = (xine_stream_private_t *)stream;
+    _x_extra_info_merge (frame->extra_info, s->video_decoder_extra_info);
     stream->metronom->got_video_frame(stream->metronom, frame);
   }
 }
@@ -801,8 +872,8 @@ static int post_audio_open(xine_audio_port_t *port_gen, xine_stream_t *stream,
   post_audio_port_t *port = (post_audio_port_t *)port_gen;
   int result;
 
-  _x_post_rewire(port->post);
-  _x_post_inc_usage(port);
+  _x_post_inc_usage (port);
+  _x_post_rewire (port->post);
   if (port->port_lock) pthread_mutex_lock(port->port_lock);
   result = (port->original_port->open) (port->original_port, stream, bits, rate, mode);
   if (port->port_lock) pthread_mutex_unlock(port->port_lock);
@@ -816,11 +887,13 @@ static int post_audio_open(xine_audio_port_t *port_gen, xine_stream_t *stream,
 static audio_buffer_t *post_audio_get_buffer(xine_audio_port_t *port_gen) {
   post_audio_port_t *port = (post_audio_port_t *)port_gen;
   audio_buffer_t *buf;
-
-  _x_post_rewire(port->post);
+  /* Ugly: we always need to ref this port here to protect it from
+   * possible port rewiring inside get_buffer (). */
+  _x_post_inc_usage (port);
   if (port->port_lock) pthread_mutex_lock(port->port_lock);
   buf = port->original_port->get_buffer(port->original_port);
   if (port->port_lock) pthread_mutex_unlock(port->port_lock);
+  _x_post_dec_usage (port);
   return buf;
 }
 
@@ -901,8 +974,7 @@ static int post_audio_rewire(xine_post_out_t *output_gen, void *data) {
   if (!new_port)
     return 0;
 
-  this->running_ticket->lock_port_rewiring(this->running_ticket, -1);
-  this->running_ticket->revoke(this->running_ticket, 1);
+  this->running_ticket->revoke (this->running_ticket, XINE_TICKET_FLAG_REWIRE);
 
   post_audio_port_ref (new_port);
   if (input_port->original_port->status(input_port->original_port, input_port->stream,
@@ -913,8 +985,7 @@ static int post_audio_rewire(xine_post_out_t *output_gen, void *data) {
   post_audio_port_unref (input_port->original_port);
   input_port->original_port = new_port;
 
-  this->running_ticket->issue(this->running_ticket, 1);
-  this->running_ticket->unlock_port_rewiring(this->running_ticket);
+  this->running_ticket->issue (this->running_ticket, XINE_TICKET_FLAG_REWIRE);
 
   return 1;
 }
@@ -999,7 +1070,7 @@ int _x_post_dispose(post_plugin_t *this) {
     pthread_mutex_unlock(&port->usage_lock);
   }
 
-  xprintf (this->xine, XINE_VERBOSITY_DEBUG, "post: _x_post_dispose (%p): %d refs.\n", this, in_use);
+  xprintf (this->xine, XINE_VERBOSITY_DEBUG, "post: _x_post_dispose (%p): %d refs.\n", (void*)this, in_use);
 
   if (!in_use) {
     xine_post_in_t  *input;
@@ -1014,9 +1085,8 @@ int _x_post_dispose(post_plugin_t *this) {
     _x_freep(&this->input_ids);
     _x_freep(&this->output_ids);
 
-    for (ite = xine_list_front(this->input); ite;
-         ite = xine_list_next(this->input, ite)) {
-      input = xine_list_get_value(this->input, ite);
+    ite = NULL;
+    while ((input = xine_list_next_value (this->input, &ite))) {
       switch (input->type) {
       case XINE_POST_DATA_VIDEO:
 	{
@@ -1033,8 +1103,10 @@ int _x_post_dispose(post_plugin_t *this) {
             int n = 0;
             do {
               vf_alias_t *next = (vf_alias_t *)f->frame.next;
-              if ((f->frame.free == post_frame_free) && f->stream)
-                _x_refcounter_dec (f->stream->refcounter);
+              if ((f->frame.free == post_frame_free) && f->stream) {
+                xine_stream_private_t *s = (xine_stream_private_t *)f->stream;
+                xine_refs_sub (&s->refs, 1);
+              }
               free (f);
               n++;
               f = next;
@@ -1065,9 +1137,8 @@ int _x_post_dispose(post_plugin_t *this) {
 	break;
       }
     }
-    for (ite = xine_list_front(this->output); ite;
-         ite = xine_list_next(this->output, ite)) {
-      output = xine_list_get_value(this->output, ite);
+    ite = NULL;
+    while ((output = xine_list_next_value (this->output, &ite))) {
       switch (output->type) {
       case XINE_POST_DATA_VIDEO:
 	if (output->rewire == post_video_rewire) {
@@ -1101,3 +1172,5 @@ int _x_post_dispose(post_plugin_t *this) {
 
   return 0;
 }
+
+
