@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2019 the xine project
+ * Copyright (C) 2000-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -115,7 +115,6 @@ static void *video_decoder_loop (void *stream_gen) {
   xine_stream_private_t *stream = (xine_stream_private_t *)stream_gen;
   xine_private_t *xine = (xine_private_t *)stream->s.xine;
   xine_ticket_t   *running_ticket = xine->port_ticket;
-  buf_element_t   *buf;
   int              running = 1;
   int              restart = 1;
   int              streamtype;
@@ -155,6 +154,8 @@ static void *video_decoder_loop (void *stream_gen) {
   running_ticket->acquire (running_ticket, 0);
 
   while (running) {
+    int handled, ignore;
+    buf_element_t *buf;
 
     lprintf ("getting buffer...\n");
 
@@ -171,7 +172,12 @@ static void *video_decoder_loop (void *stream_gen) {
 
         if ((buf->type & 0xffff0000) == BUF_VIDEO_UNKNOWN)
           break;
-        if (_x_stream_info_get (&stream->s, XINE_STREAM_INFO_IGNORE_VIDEO))
+        xine_rwlock_rdlock (&stream->info_lock);
+        handled = stream->stream_info[XINE_STREAM_INFO_VIDEO_HANDLED];
+        ignore  = stream->stream_info[XINE_STREAM_INFO_IGNORE_VIDEO];
+        xine_rwlock_unlock (&stream->info_lock);
+        (void)handled; /* dont optimize away the read. */
+        if (ignore)
           break;
 
         /* at first frame contents after start or seek, read first_frame_flag.
@@ -181,9 +187,9 @@ static void *video_decoder_loop (void *stream_gen) {
           if (!(buf->decoder_flags & (BUF_FLAG_PREVIEW | BUF_FLAG_HEADER)) && (buf->size != 4)) {
             int first_frame_flag;
             restart = 0;
-            pthread_mutex_lock (&stream->first_frame_lock);
-            first_frame_flag = stream->first_frame_flag;
-            pthread_mutex_unlock (&stream->first_frame_lock);
+            pthread_mutex_lock (&stream->first_frame.lock);
+            first_frame_flag = stream->first_frame.flag;
+            pthread_mutex_unlock (&stream->first_frame.lock);
             /* use first_frame_flag here, so gcc does not optimize it away. */
             xprintf (stream->s.xine, XINE_VERBOSITY_DEBUG,
               "video_decoder: first_frame_flag = %d.\n", first_frame_flag);
@@ -216,13 +222,13 @@ static void *video_decoder_loop (void *stream_gen) {
           video_br_num      = 20;
           video_br_value    = 0;
   
-          _x_stream_info_set (&stream->s, XINE_STREAM_INFO_VIDEO_HANDLED, (stream->video_decoder_plugin != NULL));
+          handled = (stream->video_decoder_plugin != NULL);
+          xine_rwlock_wrlock (&stream->info_lock);
+          stream->stream_info[XINE_STREAM_INFO_VIDEO_HANDLED] = handled;
+          xine_rwlock_unlock (&stream->info_lock);
         }
 
-        if (stream->video_decoder_plugin)
-          stream->video_decoder_plugin->decode_data (stream->video_decoder_plugin, buf);
-
-        /* video_br_add */
+        /* video_br_add. some decoders reset buf->pts, do this first. */
         if (buf->pts) {
           int64_t d = buf->pts - video_br_lasttime;
           if (d > 0) {
@@ -237,13 +243,15 @@ static void *video_decoder_loop (void *stream_gen) {
                   video_br_bytes >>= 1;
                   video_br_time  >>= 1;
                 }
-                br = (uint64_t)video_br_bytes * 90000 * 8 / video_br_time;
+                br = xine_uint_mul_div (video_br_bytes, 90000 * 8, video_br_time);
                 bdiff = br - video_br_value;
                 if (bdiff < 0)
                   bdiff = -bdiff;
                 if (bdiff > (br >> 6)) {
                   video_br_value = br;
-                  _x_stream_info_set (&stream->s, XINE_STREAM_INFO_VIDEO_BITRATE, br);
+                  xine_rwlock_wrlock (&stream->info_lock);
+                  stream->stream_info[XINE_STREAM_INFO_VIDEO_BITRATE] = br;
+                  xine_rwlock_unlock (&stream->info_lock);
                 }
               }
             }
@@ -255,13 +263,21 @@ static void *video_decoder_loop (void *stream_gen) {
         }
         video_br_lastsize += buf->size;
 
-        if (buf->type != buftype_unknown &&
-            !_x_stream_info_get (&stream->s, XINE_STREAM_INFO_VIDEO_HANDLED)) {
+        if (stream->video_decoder_plugin)
+          stream->video_decoder_plugin->decode_data (stream->video_decoder_plugin, buf);
+
+        /* no need to lock again. it may have been reset from this thread inside
+         * video_decoder_plugin->decode_data (), if at all.
+         * XXX: should we try a different decoder then? */
+        handled = stream->stream_info[XINE_STREAM_INFO_VIDEO_HANDLED];
+        if (!handled && (buf->type != buftype_unknown)) {
+          const char *vname = _x_buf_video_name (buf->type);
+
           xine_log (stream->s.xine, XINE_LOG_MSG,
-                    _("video_decoder: no plugin available to handle '%s'\n"), _x_buf_video_name( buf->type ) );
+            _("video_decoder: no plugin available to handle '%s'\n"), vname);
 
           if (!_x_meta_info_get (&stream->s, XINE_META_INFO_VIDEOCODEC))
-	    _x_meta_info_set_utf8 (&stream->s, XINE_META_INFO_VIDEOCODEC, _x_buf_video_name( buf->type ));
+	    _x_meta_info_set_utf8 (&stream->s, XINE_META_INFO_VIDEOCODEC, vname);
 
           buftype_unknown = buf->type;
 
@@ -339,16 +355,16 @@ static void *video_decoder_loop (void *stream_gen) {
 
           case BUFTYPE_SUB (BUF_CONTROL_HEADERS_DONE):
 
-            pthread_mutex_lock (&stream->counter_lock);
-            stream->header_count_video++;
+            pthread_mutex_lock (&stream->counter.lock);
+            stream->counter.headers_video++;
             if (stream->audio_thread_created) {
               /* avoid useless wakes on an incomplete pair */
-              if (stream->header_count_video <= stream->header_count_audio)
-                pthread_cond_broadcast (&stream->counter_changed);
+              if (stream->counter.headers_video <= stream->counter.headers_audio)
+                pthread_cond_broadcast (&stream->counter.changed);
             } else {
-              pthread_cond_broadcast (&stream->counter_changed);
+              pthread_cond_broadcast (&stream->counter.changed);
             }
-            pthread_mutex_unlock (&stream->counter_lock);
+            pthread_mutex_unlock (&stream->counter.lock);
             restart = 1;
             break;
 
@@ -422,32 +438,32 @@ static void *video_decoder_loop (void *stream_gen) {
             }
             running_ticket->release (running_ticket, 0);
             /* wait for audio to reach this marker, if necessary */
-            pthread_mutex_lock (&stream->counter_lock);
-            stream->finished_count_video++;
-            lprintf ("reached end marker # %d\n", stream->finished_count_video);
+            pthread_mutex_lock (&stream->counter.lock);
+            stream->counter.finisheds_video++;
+            lprintf ("reached end marker # %d\n", stream->counter.finisheds_video);
             if (stream->audio_thread_created) {
-              if (stream->finished_count_video > stream->finished_count_audio) {
+              if (stream->counter.finisheds_video > stream->counter.finisheds_audio) {
                 do {
                   struct timespec ts = {0, 0};
                   xine_gettime (&ts);
                   ts.tv_sec += 1;
                   /* use timedwait to workaround buggy pthread broadcast implementations */
-                  pthread_cond_timedwait (&stream->counter_changed, &stream->counter_lock, &ts);
-                } while (stream->finished_count_video > stream->finished_count_audio);
-              } else if (stream->finished_count_video == stream->finished_count_audio) {
-                pthread_cond_broadcast (&stream->counter_changed);
+                  pthread_cond_timedwait (&stream->counter.changed, &stream->counter.lock, &ts);
+                } while (stream->counter.finisheds_video > stream->counter.finisheds_audio);
+              } else if (stream->counter.finisheds_video == stream->counter.finisheds_audio) {
+                pthread_cond_broadcast (&stream->counter.changed);
               }
             } else {
-              pthread_cond_broadcast (&stream->counter_changed);
+              pthread_cond_broadcast (&stream->counter.changed);
             }
-            pthread_mutex_unlock (&stream->counter_lock);
+            pthread_mutex_unlock (&stream->counter.lock);
             /* Wake up xine_play if it's waiting for a frame */
-            pthread_mutex_lock (&stream->first_frame_lock);
-            if (stream->first_frame_flag) {
-              stream->first_frame_flag = 0;
-              pthread_cond_broadcast(&stream->first_frame_reached);
+            pthread_mutex_lock (&stream->first_frame.lock);
+            if (stream->first_frame.flag) {
+              stream->first_frame.flag = 0;
+              pthread_cond_broadcast(&stream->first_frame.reached);
             }
-            pthread_mutex_unlock (&stream->first_frame_lock);
+            pthread_mutex_unlock (&stream->first_frame.lock);
             running_ticket->acquire (running_ticket, 0);
             break;
 
@@ -470,7 +486,9 @@ static void *video_decoder_loop (void *stream_gen) {
 
           case BUFTYPE_SUB (BUF_CONTROL_RESET_DECODER):
             _x_extra_info_reset (stream->video_decoder_extra_info);
-            stream->video_seek_count++;
+            /* bump seek count, and inform audio decoder about this. */
+            stream->video_seek_count += 1;
+            (void)stream->s.audio_fifo->size (stream->s.audio_fifo);
             /* running_ticket->acquire(running_ticket, 0); */
             if (stream->video_decoder_plugin)
               stream->video_decoder_plugin->reset (stream->video_decoder_plugin);
@@ -602,6 +620,8 @@ int _x_video_decoder_init (xine_stream_t *s) {
         "Higher values mean smoother playback for unreliable inputs, but also increased "
         "latency and memory consumption."),
       20, NULL, NULL);
+    if (num_buffers < 50)
+      num_buffers = 50;
     if (num_buffers > 5000)
       num_buffers = 5000;
 
