@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2019 the xine project
+ * Copyright (C) 2000-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -185,6 +185,17 @@ int xine_config_unregister_callbacks (xine_t *self,
   return self->config->unregister_callbacks (self->config, key, changed_cb, cb_data, cb_data_size);
 }
 
+int xine_config_lookup_num(xine_t *this, const char *key, int def_value) {
+  return this->config->lookup_num(this->config, key, def_value);
+}
+
+char *xine_config_lookup_string(xine_t *this, const char *key) {
+  return this->config->lookup_string(this->config, key);
+}
+
+void xine_config_free_string(xine_t *this, char **value) {
+  this->config->free_string(this->config, value);
+}
 
 /*
  * helper function:
@@ -308,7 +319,7 @@ void xine_config_update_entry (xine_t *this, const xine_cfg_entry_t *entry) {
   default:
     xprintf (this, XINE_VERBOSITY_DEBUG,
 	     "xine_interface: error, unknown config entry type %d\n", entry->type);
-    _x_abort();
+    break;
   }
 }
 
@@ -751,8 +762,8 @@ uint32_t xine_get_stream_info (xine_stream_t *s, int info) {
     return 0;
 
   case XINE_STREAM_INFO_HAS_CHAPTERS:
-    if (stream->demux_plugin)
-      if (stream->demux_plugin->get_capabilities (stream->demux_plugin) & DEMUX_CAP_CHAPTERS)
+    if (stream->demux.plugin)
+      if (stream->demux.plugin->get_capabilities (stream->demux.plugin) & DEMUX_CAP_CHAPTERS)
         return 1;
     if (stream->s.input_plugin)
       if (stream->s.input_plugin->get_capabilities (stream->s.input_plugin) & INPUT_CAP_CHAPTERS)
