@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2019 the xine project
+ * Copyright (C) 2000-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -71,14 +71,21 @@ EXTERN_C_START
 #    define XINE_ATINT_T atomic_int
 #    define XINE_ATINIT(xatfa_refs,xatfa_n) atomic_init (&(xatfa_refs), (xatfa_n))
 #    define XINE_ATFA(xatfa_refs,xatfa_n) atomic_fetch_add_explicit (&(xatfa_refs), (xatfa_n), memory_order_acq_rel)
+#    define XINE_ATGET(xatfa_refs) atomic_load_explicit (&(xatfa_refs), memory_order_acquire)
 #  elif (HAVE_ATOMIC_VARS == 2)
 #    define XINE_ATINT_T int
 #    define XINE_ATINIT(xatfa_refs,xatfa_n) __atomic_store_n (&(xatfa_refs), (xatfa_n), __ATOMIC_RELAXED)
 #    define XINE_ATFA(xatfa_refs,xatfa_n) __atomic_fetch_add (&(xatfa_refs), (xatfa_n), __ATOMIC_ACQ_REL)
+#    define XINE_ATGET(xatfa_refs) __atomic_load_n (&(xatfa_refs), __ATOMIC_ACQUIRE)
 #  else /* HAVE_ATOMIC_VARS == 3 */
-#    define XINE_ATINT_T int
+#    define XINE_ATINT_T volatile int
 #    define XINE_ATINIT(xatfa_refs,xatfa_n) xatfa_refs = xatfa_n
 #    define XINE_ATFA(xatfa_refs,xatfa_n) __sync_fetch_and_add (&(xatfa_refs), (xatfa_n))
+#    if defined (ARCH_X86)
+#      define XINE_ATGET(xatfa_refs) (xatfa_refs)
+#    else
+#      define XINE_ATGET(xatfa_refs) __sync_fetch_and_add (&(xatfa_refs), 0)
+#    endif
 #  endif
 
 typedef struct {
@@ -105,6 +112,10 @@ static inline int xine_refs_sub (xine_refs_t *refs, int n) {
   return v;
 }
 
+static inline int xine_refs_get (xine_refs_t *refs) {
+  return XINE_ATGET (refs->refs);
+}
+
 #else
 
 typedef struct {
@@ -144,6 +155,14 @@ static inline int xine_refs_sub (xine_refs_t *refs, int n) {
   return v;
 }
 
+static inline int xine_refs_get (xine_refs_t *refs) {
+  int v;
+  pthread_mutex_lock (&refs->mutex);
+  v = refs->refs;
+  pthread_mutex_unlock (&refs->mutex);
+  return v;
+}
+
 #endif
 
 /**
@@ -247,6 +266,39 @@ static inline int xine_gettime (struct timespec *ts) {
 }
 #endif
 
+#if (defined(__GNUC__) || defined(__clang__)) && defined(ARCH_X86)
+static inline uint32_t xine_uint_mul_div (uint32_t num, uint32_t mul, uint32_t den) {
+  register uint32_t eax = num, edx;
+  /* if result > 0xffffffff, return 0xffffffff without math exception. */
+  __asm__ __volatile__ (
+      "mull\t%2\n"
+    "\tmovl\t%3, %2\n"
+    "\tshrl\t%2\n"
+    "\taddl\t%2, %0\n"
+    "\tadcl\t$0, %1\n"
+    "\tcmpl\t%1, %3\n"
+    "\tjbe\t1f\n"
+    "\tdivl\t%3\n"
+    "\tjmp\t2f\n"
+    "1:\n"
+    "\txorl\t%0, %0\n"
+    "\tnotl\t%0\n"
+    "2:\n"
+    : "=a" (eax), "=d" (edx), "=r" (mul), "=g" (den)
+    : "0"  (eax),             "2"  (mul), "3"  (den)
+    : "cc"
+  );
+  (void)mul;
+  (void)den;
+  (void)edx;
+  return eax;
+}
+#else
+static inline uint32_t xine_uint_mul_div (uint32_t num, uint32_t mul, uint32_t den) {
+  return ((uint64_t)num * mul + (den >> 1)) / den;
+}
+#endif
+
 static inline int32_t xine_str2int32 (const char **s) {
   const uint8_t *p = (const uint8_t *)*s;
   uint8_t z;
@@ -383,7 +435,7 @@ static inline void xine_uint64_2str (char **s, uint64_t v) {
 
 /* A little helper for integers whose size is not obvious, like off_t and time_t. */
 #define xine_uint2str(s,v) do { \
-  if (sizeof (v) == 64) \
+  if (sizeof (v) == 8) \
     xine_uint64_2str (s, v); \
   else \
     xine_uint32_2str (s, v); \
@@ -408,6 +460,13 @@ typedef struct {
 
   int                        flags;
   int                        network_timeout;
+  enum {
+    XINE_IP_PREF_AUTO = 0,
+    XINE_IP_PREF_4,
+    XINE_IP_PREF_4_6,
+    XINE_IP_PREF_6_4
+  }                          ip_pref;
+
   uint32_t                   join_av:1;
 
   /* lock controlling speed change access.
@@ -423,9 +482,9 @@ typedef struct {
   pthread_mutex_t            speed_change_lock;
   pthread_cond_t             speed_change_done;
   /* set when pauseing with port ticket granted, for XINE_PARAM_VO_SINGLE_STEP. */
-  /* special values for set_speed_internal () */
-# define XINE_LIVE_PAUSE_ON 0x7ffffffd
-# define XINE_LIVE_PAUSE_OFF 0x7ffffffc
+  /* special values for set_speed_internal (). now defined in xine/xine_internal.h. */
+  /* # define XINE_LIVE_PAUSE_ON 0x7ffffffd */
+  /* # define XINE_LIVE_PAUSE_OFF 0x7ffffffc */
 } xine_private_t;
   
 typedef struct xine_stream_private_st {
@@ -445,7 +504,6 @@ typedef struct xine_stream_private_st {
   uint32_t                   finished_naturally:1;
 
   input_class_t             *eject_class;
-  demux_plugin_t            *demux_plugin;
 
 /*  vo_driver_t               *video_driver;*/
   pthread_t                  video_thread;
@@ -501,45 +559,60 @@ typedef struct xine_stream_private_st {
   char                      *meta_info[XINE_STREAM_INFO_MAX];
 
   /* seeking slowdown */
-  pthread_mutex_t            first_frame_lock;
-  pthread_cond_t             first_frame_reached;
-  /* 3: wait for first frame to decode (stream start).
-   * 2: wait for first frame to display (stream seek).
-   * 1: after 2, first frame is decoded but not yet displayed.
-   * 0: waiting done.
-   */
-  uint32_t                   first_frame_flag:2;
+  struct {
+    pthread_mutex_t          lock;
+    pthread_cond_t           reached;
+    /* 3: wait for first frame to decode (stream start).
+     * 2: wait for first frame to display (stream seek).
+     * 1: after 2, first frame is decoded but not yet displayed.
+     * 0: waiting done.
+     */
+    uint32_t                 flag:2;
+  } first_frame;
 
   /* wait for headers sent / stream decoding finished */
-  pthread_mutex_t            counter_lock;
-  pthread_cond_t             counter_changed;
-  int                        header_count_audio;
-  int                        header_count_video;
-  int                        finished_count_audio;
-  int                        finished_count_video;
-  /* set of id_flag values */
-  uint32_t                   start_buffers_sent;
-  int                        num_demuxers_running;
+  struct {
+    pthread_mutex_t          lock;
+    pthread_cond_t           changed;
+    int                      headers_audio;
+    int                      headers_video;
+    int                      finisheds_audio;
+    int                      finisheds_video;
+    int                      demuxers_running;
+    /* network buffering control. */
+    int                      nbc_refs;
+    xine_nbc_t              *nbc;
+  } counter;
 
   /* event mechanism */
-  xine_list_t               *event_queues;
-  pthread_mutex_t            event_queues_lock;
+  struct {
+    pthread_mutex_t          lock;
+    xine_list_t             *queues;
+  } event;
 
   /* demux thread stuff */
-  pthread_t                  demux_thread;
-  pthread_mutex_t            demux_lock;
-  pthread_mutex_t            demux_action_lock;
-  pthread_cond_t             demux_resume;
-  /* used in _x_demux_... functions to synchronize order of pairwise A/V buffer operations */
-  pthread_mutex_t            demux_pair_mutex;
-  uint32_t                   demux_action_pending:1;
-  uint32_t                   demux_thread_created:1;
-  uint32_t                   demux_thread_running:1;
-  /* filter out duplicate seek discontinuities from side streams */
-  uint32_t                   demux_max_seek_bufs;
+  struct {
+    demux_plugin_t          *plugin;
+    pthread_t                thread;
+    pthread_mutex_t          lock;
+    pthread_mutex_t          action_lock;
+    pthread_cond_t           resume;
+    /* used in _x_demux_... functions to synchronize order of pairwise A/V buffer operations */
+    pthread_mutex_t          pair;
+    /* next 2 protected by action_lock */
+    uint32_t                 action_pending;
+    uint32_t                 input_caps;
+    uint32_t                 thread_created:1;
+    uint32_t                 thread_running:1;
+    /* filter out duplicate seek discontinuities from side streams */
+    uint32_t                 max_seek_bufs;
+    /* set of id_flag values */
+    uint32_t                 start_buffers_sent;
+  } demux;
 
-  extra_info_t              *current_extra_info;
-  pthread_mutex_t            current_extra_info_lock;
+#define XINE_NUM_CURR_EXTRA_INFOS 2
+  xine_refs_t                current_extra_info_index;
+  extra_info_t               current_extra_info[XINE_NUM_CURR_EXTRA_INFOS];
   int                        video_seek_count;
 
   int                        delay_finish_event; /* delay event in 1/10 sec units. 0=>no delay, -1=>forever */
@@ -555,22 +628,22 @@ typedef struct xine_stream_private_st {
 
   xine_refs_t                refs;
 
-  xine_keyframes_entry_t    *index_array;
-  pthread_mutex_t            index_mutex;
-  int                        index_size, index_used, index_lastadd;
-
-  /* network buffering control. these 2 fields are protected by index_mutex (do we need our own??). */
-  int                        nbc_refs;
-  xine_nbc_t                *nbc;
+  struct {
+    pthread_mutex_t          lock;
+    xine_keyframes_entry_t  *array;
+    int                      size, used, lastadd;
+  } index;
 
   uint32_t                   disable_decoder_flush_at_discontinuity;
 
   /* _x_find_input_plugin () recursion protection */
   input_class_t             *query_input_plugins[2];
 
-  extra_info_t               ei[3];
+  extra_info_t               ei[2];
 } xine_stream_private_t;
 
+void xine_current_extra_info_set (xine_stream_private_t *stream, const extra_info_t *info) INTERNAL;
+
 /* Nasty net_buf_ctrl helper: inform about something outside its regular callbacks. */
 #define XINE_NBC_EVENT_AUDIO_DRY 1
 void xine_nbc_event (xine_stream_private_t *stream, uint32_t type) INTERNAL;
