@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2018 the xine project
+ *  Copyright (C) 2002-2020 the xine project
  *
  *  This file is part of xine, a free video player.
  *
@@ -23,10 +23,6 @@
 #include "config.h"
 #endif
 
-#ifdef XINE_COMPILE
-# include "config.h"
-#endif
-
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -145,7 +141,7 @@ static void xml_parser_free_props(xml_property_t *current_property) {
 }
 
 static void xml_parser_free_tree_rec(xml_node_t *current_node, int free_next) {
-  lprintf("xml_parser_free_tree_rec: %s\n", current_node ? current_node->name : NULL);
+  lprintf("xml_parser_free_tree_rec: %s\n", current_node ? current_node->name : "(null)");
 
   if (current_node) {
     /* properties */
